#<profile4xi profile = "12" />
# =================================================================================================
#<program4xi program = "bpu2image4xi.py">
#<abstract4xi abstract = "BPU 2D image management"/>
#<refer4xi reference = "image2d4xi.py"/>
#<verify4xi verification = "10" target = "bpu2image4xi5test.py">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting; [Rel 7.21]"/>
# ================================================================================================= 

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'BPU2Image4Xi',                        # ACK 5.31  p76 
            
            'copy2image4bpu',                      # ACK 5.3.3 p88
            'copy2imagetobuffer4bpu',              # ACK 5.3.4 p90
            'copy2buffertoimage4bpu',              # ACK 5.3.4 p92
            'create2image4bpu',                    # ACK 5.31  p76            

            'get2imageinfo4bpu',                   # ACK 5.3.6 p97 nb: modernized so that you can also access only selected elements, if you want           
            'get2supportedimageformats4bpu',       # ACK 5.3.2 p82 nb: a separate function to get format whereas using get2imageinfo4bpu you can access any element by key such as 'FORMAT' 
                              
            'manoeuvre2copyimage4bpu',             # ACK modernized 
            'manoeuvre2copyimagetobuffer4bpu',     # ACK modernized        
            'manoeuvre2copybuffertoimage4bpu',     # ACK modernized
            'manoeuvre2mapimage4bpu',              # ACK modernized                                           
            'manoeuvre2readimage4bpu',             # ACK modernized
            'manoeuvre2writeimage4bpu',            # ACK modernized            
            'map2image4bpu',                       # ACK 5.3.5 p94 
            
            'read2image4bpu',                      # ACK 5.3.3 p84  
            'retain2image4bpu',                    # ACK 5.4.1 p99
            'release2image4bpu',                   # ACK 5.4.1 p99

            'set2imagecallback4bpu',               # ACK modernized
            'set2image4bpu',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    

            'trace2image4bpu',                     # ACK modernized 
                                    
            'wait2image4bpu',                      # ACK modernized nb: checks Queue object and thereby causes delay doing something useful
            'write2image4bpu'                      # ACK 5.3.3 p84                 
          ] 
#"
#/>


# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os
from platform import processor
import sys
import threading                
import _thread   
from decimal import *
#"
#/>


# -------------------------------------------------------------------------------------------------
# Scope4Xi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
import version4xi

# -------------------------------------------------------------------------------------------------
# Buffer module                           
# -------------------------------------------------------------------------------------------------
from bpu2buffer4xi import BPU2Buffer4Xi
from bpu2buffer4xi import create2buffer4bpu
from bpu2buffer4xi import create2subbuffer4bpu
from bpu2buffer4xi import get2bufferinfo4bpu
from bpu2buffer4xi import release2buffer4bpu
from bpu2buffer4xi import retain2buffer4bpu
from bpu2buffer4xi import set2buffer4bpu
from bpu2buffer4xi import trace2buffer4bpu

# -------------------------------------------------------------------------------------------------
# Command queue module                           
# -------------------------------------------------------------------------------------------------
from bpu2commandqueue4xi import BPU2CommandQueue4Xi
from bpu2commandqueue4xi import create2commandqueue4bpu 
from bpu2commandqueue4xi import flush2commandqueue4bpu
from bpu2commandqueue4xi import finish2commandqueue4bpu
from bpu2commandqueue4xi import get2commandqueueinfo4bpu
from bpu2commandqueue4xi import release2commandqueue4bpu
from bpu2commandqueue4xi import retain2commandqueue4bpu
from bpu2commandqueue4xi import set2commandqueue4bpu
from bpu2commandqueue4xi import trace2commandqueue4bpu

# -------------------------------------------------------------------------------------------------
# Context module                           
# -------------------------------------------------------------------------------------------------
from bpu2context4xi import BPU2Context4Xi 
from bpu2context4xi import create2context4bpu
from bpu2context4xi import get2context4bpu
from bpu2context4xi import release2context4bpu
from bpu2context4xi import retain2context4bpu
from bpu2context4xi import set2context4bpu


# -------------------------------------------------------------------------------------------------
# Device module                        
# -------------------------------------------------------------------------------------------------
from bpu2device4xi import BPU4Xi
from bpu2device4xi import init2bpu4xi 
from bpu2device4xi import set2bpu4xi
from bpu2device4xi import get2bpu4xi

# -------------------------------------------------------------------------------------------------
# Event module                           
# -------------------------------------------------------------------------------------------------
from bpu2event4xi import BPU2Event4Xi
from bpu2event4xi import create2event4bpu
from bpu2event4xi import get2eventinfo4bpu
from bpu2event4xi import release2event4bpu
from bpu2event4xi import retain2event4bpu
from bpu2event4xi import set2eventcallback4bpu 
from bpu2event4xi import set2event4bpu
from bpu2event4xi import trace2event4bpu
from bpu2event4xi import wait2event4bpu


# -------------------------------------------------------------------------------------------------
# Platform module                           
# -------------------------------------------------------------------------------------------------
from platformids4xi  import get2platformids4xi
from platforminfo4xi import PlatformInfo4Xi                                                                        
from platforminfo4xi import init2platforminfo4xi
from platforminfo4xi import set2platforminfo4xi
from platforminfo4xi import get2platforminfo4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from arena4xi import Arena4Xi
from cpu_corecount4xi import cpu_corecount4xi
from dummyprocess4xi import DummyProcess4Xi
from print4xi import print4xi
from pipe4xi import Pipe4Xi
from queue4xi import Queue4Xi  
from time4xi import time4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi


#"
#/>

# -------------------------------------------------------------------------------------------------
# Global constants                            
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global bpu2image4xi                                                     # 2D-IMAGES FOR BPU

global bpu2trace4xi                                                     # TRACE

global bpu2buffer4xi                                                    # BUFFER FOR BPU

global control2readimage4bpu                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH BPU  
global state2read4bpu                                                   # STATE FOR COMMAND AND CONTROL
global max2readstate4bpu                                                # MAX FOR STATE
global ack2readinfo4bpu                                                 # ACK FOR IMAGE INFO
global message2readget4bpu                                              # QUEUE GET
global message2readput4bpu                                              # QUEUE PUT
global message2readreceive4bpu                                          # PIPE RECEIVE
global message2readsend4bpu                                             # PIPE SEND

global control2writeimage4bpu                                           # COMMAND AND CONTROL TO WRITE 2D IMAGES WITH BPU 
global state2write4bpu                                                  # STATE FOR COMMAND AND CONTROL
global max2writestate4bpu                                               # MAX FOR STATE
global ack2writeinfo4bpu                                                # ACK FOR IMAGE INFO
global message2writeget4bpu                                             # QUEUE GET
global message2writeput4bpu                                             # QUEUE PUT
global message2writereceive4bpu                                         # PIPE RECEIVE
global message2writesend4bpu                                            # PIPE SEND

global control2copyimage4bpu                                            # COMMAND AND CONTROL TO COPY 2D IMAGES WITH BPU 
global state2copyimage4bpu                                              # STATE FOR COMMAND AND CONTROL
global max2copyimagestate4bpu                                           # MAX FOR STATE
global ack2copyimageinfo4bpu                                            # ACK FOR IMAGE INFO
global message2copyimageget4bpu                                         # QUEUE GET
global message2copyimageput4bpu                                         # QUEUE PUT
global message2copyimagereceive4bpu                                     # PIPE RECEIVE
global message2copyimagesend4bpu                                        # PIPE SEND

global control2copyimagetobuffer4bpu                                    # COMMAND AND CONTROL TO COPY 2D IMAGES TO BUFFER WITH BPU 
global state2copyimagetobuffer4bpu                                      # STATE FOR COMMAND AND CONTROL
global max2copyimagetobufferstate4bpu                                   # MAX FOR STATE
global ack2copyimagetobufferinfo4bpu                                    # ACK FOR IMAGE INFO
global message2copyimagetobufferget4bpu                                 # QUEUE GET
global message2copyimagetobufferput4bpu                                 # QUEUE PUT
global message2copyimagetobufferreceive4bpu                             # PIPE RECEIVE
global message2copyimagetobuffersend4bpu                                # PIPE SEND

global control2copybuffertoimage4bpu                                    # COMMAND AND CONTROL TO COPY BUFFER TO 2D IMAGE WITH BPU 
global state2copybuffertoimage4bpu                                      # STATE FOR COMMAND AND CONTROL
global max2copybuffertoimagestate4bpu                                   # MAX FOR STATE
global ack2copybuffertoimageinfo4bpu                                    # ACK FOR IMAGE INFO
global message2copybuffertoimageget4bpu                                 # QUEUE GET
global message2copybuffertoimageput4bpu                                 # QUEUE PUT
global message2copybuffertoimagereceive4bpu                             # PIPE RECEIVE
global message2copybuffertoimagesend4bpu                                # PIPE SEND                      

global control2mapimage4bpu                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH BPU  
global state2map4bpu                                                   # STATE FOR COMMAND AND CONTROL
global max2mapstate4bpu                                                # MAX FOR STATE
global ack2mapinfo4bpu                                                 # ACK FOR IMAGE INFO
global message2mapget4bpu                                              # QUEUE GET
global message2mapput4bpu                                              # QUEUE PUT
global message2mapreceive4bpu                                          # PIPE RECEIVE
global message2mapsend4bpu                                             # PIPE SEND
            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "

#"
#"/>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "trace2image4bpu" usage = "
#
#   print('=' *33)          
#   print('(Step 15A/25) OF BPU2IMAGE4XI5TEST - TRYING TO TRACE BPU: ', time4xi())      
#   print('=' *33)      
#   trace2bpu = None
#   try:
#       trace2bpu = trace2image4bpu() 
#       print('(Step 15B/25) OF BPU2IMAGE4XI5TEST - TRACE2BPU: ', trace2bpu)  
#       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4bpu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4bpu', 'METHOD': 'trace2image4bpu'}
#   except:
#       print('(Step 15Z1/25) OF BPU2IMAGE4XI5TEST *** FAILED TO TRACE BPU: ', trace2bpu)       
#">

# -------------------------------------------------------------------------------------------------   
def trace2image4bpu(task4trace2bpu=None):  

    '''
    creates callback to bpu2image4xi
    '''
    
    global bpu2trace4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "trace2image4bpu" 
    method2print = "trace2image4bpu"

    message2print = "{(Step 1A/33) started" + "; trace2image4bpu-started: " + str(trace2image4bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # <function trace2image4bpu at 0x0000000003AF0348> 
  
             
    message2print = "{(Step 1B/33) passed" + "; task4trace2bpu-to-trace2image4bpu: " + str(task4trace2bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              # 
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    bpu2trace4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
      
    if task4trace2bpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY BPUIZED CALLBACK !
        # -----------------------------------------------------------------------------------------    
        task4trace2bpu = {}                                                                                            
            
        task4trace2bpu['CLASS']   = "trace2image4bpu"                                                               # BPUIZE ZPU TRACE!
        task4trace2bpu['METHOD']  = "trace2image4bpu" 
        task4trace2bpu['MESSAGE'] = "{(Step 2Z0/33) failed" + "; trace2image4bpu-failed " + str(bpu2trace4xi)  + "; time: " + time4xi() + "}"               
  
        
        message2print = "{(Step 2A/33) set" + "; task4trace2bpu-set-by-trace2image4bpu to: " + str(task4trace2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4bpu-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4bpu',
        #  'METHOD': 'trace2image4bpu'}
        #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}
 
    
        code2ack = code2ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # PARAMETER GIVEN BY BPUIZED CALLBACK !
    # -----------------------------------------------------------------------------------------          
    message2print = "{(Step 3A/33) try" + "; task4trace2bpu-try-by-trace2image4bpu: " + str(task4trace2bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)    
    #  { trace2image4bpu : trace2image4bpu ; (Step 3A/33) try; task4trace2bpu-try-by-trace2image4bpu: 
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4bpu-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'trace2image4bpu',
    #  'METHOD': 'trace2image4bpu'}
    #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}

    try:
        bpu2trace4xi ['CLASS']  = task4trace2bpu['CLASS']                                                      # BPUIZE BPU TRACE!
        bpu2trace4xi ['METHOD']  = task4trace2bpu['METHOD']
        bpu2trace4xi ['MESSAGE'] = task4trace2bpu['MESSAGE']
        
        message2print = "{(Step 3B/33) set" + "; bpu2trace4xi-set-by-trace2image4bpu to: " + str(bpu2trace4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4bpu : trace2image4bpu ; (Step 3B/33) set; bpu2trace4xi-set-by-trace2image4bpu to: 
        #                     {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4bpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}',
        #                      'CLASS': 'trace2image4bpu',
        #                      'METHOD': 'trace2image4bpu'}
        #                     ; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3B-'         
    except:
        message2print = "{(Step 3Z1/33) failed" + "; bpu-trace-failed-by-trace2image4bpu: " + str(task4trace2bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z1-' 
        
    try:
        # =================================
        # SIMULATE BPU TRACE
        # =================================
        class2print   = task4trace2bpu['CLASS']                                                                # BPUIZE BPU TRACE!
        method2print  = task4trace2bpu['METHOD']
        
        message2print = task4trace2bpu['MESSAGE']
        print4xi(class2print, method2print, message2print)                                                     # BPUIZED TRACE FOR OPENCL CALLBACK!
        # { trace2image4bpu : trace2image4bpu ; (Step 2Z0/33) failed; trace2image4bpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}
        
        code2ack = code2ack + '3C-'         
    except:
        class2print = "trace2image4bpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4bpu"   
        message2print = "{(Step 3Z2/33) failed" + "; simulation-of-bpu-trace-failed-by-trace2image4bpu: " + str(task4trace2bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z2-' 
        
    try:
        # =================================
        # NORMAL TRACE
        # =================================                     
        class2print = "trace2image4bpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4bpu"                
        message2print = "{(Step 3D/33) performed" + "; bpu-trace-performed-by-trace2image4bpu: " + str(task4trace2bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4bpu : trace2image4bpu ; (Step 3D/33) performed; bpu-trace-performed-by-trace2image4bpu: 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4bpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
        # 'CLASS': 'trace2image4bpu', 'METHOD': 'trace2image4bpu'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3D-' 
    except:
        message2print = "{(Step 3Z3/33) failed" + "; bpu-trace-failed-by-trace2image4bpu: " + str(task4trace2bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z3-' 

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; bpu2trace4xi-returning-by-trace2image4bpu: " + str(bpu2trace4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4bpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
    #  'CLASS': 'trace2image4bpu', 
    #  'METHOD': 'trace2image4bpu'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

    
    return bpu2trace4xi
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2imagecallback4bpu" usage = "
#
#   print('(Step 14A/700) OF BPU2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE CALLBACK OF BPU DEVICE ', time4xi())  
#   try:                               
#       response4setcallback2image = set2imagecallback4bpu()                                                                                                 
#       print('(Step 14B/700) OF BPU2IMAGE4XI5TEST - SET 2D IMAGE CALLBACK OF BPU DEVICE TO: ', response4setcallback2image)   
#       # <function trace2image4bpu at 0x0000000004E2CAC8>
#   except:
#       print('(Step 14Z1/700) OF BPU2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE CALLBACK OF BPU DEVICE ', time4xi())  
# 
#">

# ------------------------------------------------------------------------------------------------- 
 
def set2imagecallback4bpu(task4set2imagecallback=None):

    '''
    registers callback function to bpu2image4xi
    '''
    global bpu2image4xi
    
    class2print = "set2imagecallback4bpu" 
    method2print = "set2imagecallback4bpu"
    
    message2print = "{(Step 1A/70) started" + "; set2imagecallback4bpu-started: " + str(set2imagecallback4bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function set2imagecallback4bpu at 0x000000000390B748>
    
    try:
    
        bpu2image4xi ['CALLBACK'] = trace2image4bpu
        
        message2print = "{(Step 1B/70) set callback2bpu" + "; imagecallback4bpu-set-by-set2imagecallback4bpu: " + str(bpu2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4bpu at 0x0000000003A9E348> 
        
        message2print = "{(Step 1C/70) returning" + "; callback-of-bpu2image4xi-returning-by-set2imagecallback4bpu: " + str(bpu2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4bpu at 0x0000000003864A48> 
        
        return bpu2image4xi ['CALLBACK']      
        
    except:
        message2print = "{(Step 1Z1/70) failed" + "; callback2image-failed-by-set2imagecallback4bpu: " + str(callback2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         

        message2print = "{(Step 1Z2/70) returning" + "; trace2image4bpu-returning-by-set2imagecallback4bpu: " + str(trace2image4bpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)      
            
        return trace2image4bpu
            
#</function4xi> 




# -------------------------------------------------------------------------------------------------
#<function4xi function = "create2image4bpu" usage = "
#   #1     
#   print('=' *33)          
#   print('(Step 2A/700) OF BPU2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE4BPU ', time4xi())      
#   print('=' *33)        
#   # 
#   try:
#       response4create2Dimage = create2image4bpu()                                                                 
#       print('(Step 2B/700) OF BPU2IMAGE4XI5TEST - CREATED 2D IMAGE4BPU ', time4xi())
#       keys2response = response4create2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 2Z1/700) OF BPU2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE4BPU *** ', time4xi())
#   #2
#   print('(Step 123A/700) OF BPU2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE for BPU ', time4xi()) 
#   # 
#   try:
#       xi2bpu = BPU2Image4Xi()                            
#       response4create2Dimage = xi2bpu.create2image()                                  
#       keys2response = response4create2Dimage.keys()
#       print('(Step 122B/700) OF BPU2IMAGE4XI5TEST - CREATED 2D IMAGE ', time4xi())  
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 123Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE for BPU ', time4xi())    
#">

# -------------------------------------------------------------------------------------------------   
def create2image4bpu(task4imagecreate2bpu=None):

    '''
    creates 2D image for BPU
    '''
    
    global bpu2image4xi                                                                                              
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "create2image4bpu" 
    method2print = "create2image4bpu"

    message2print = "{(Step 1A/70) started" + "; create2image4bpu-started: " + str(create2image4bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function create2image4bpu at 0x00000000038D97C8> 
  
             
    message2print = "{(Step 1B/70) passed" + "; task4imagecreate2bpu-to-create2image4bpu: " + str(task4imagecreate2bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    bpu2image4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4imagecreate2bpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO BPU! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecreate2bpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4imagecreate2bpu-created-by-create2image4bpu to: " + str(task4imagecreate2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            function2callback4bpu = trace2image4bpu                                                                 # CALLBACK FUNCTION FOR BPU
            
            message2print = "{(Step 1D/70) initialized" + "; function2callback4bpu-created-by-create2image4bpu to: " + str(function2callback4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4bpu at 0x00000000038D9348>
            
            code2ack  = code2ack + '1D-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "create2image4bpu"                                                          # MODULE OF BPUIZED OPENCL MODULE
            args2callback['METHOD']  = "create2image4bpu"                                                          # FUNCTION OF BPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create2image4bpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR BPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args2callback-created-by-create2image4bpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4bpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4bpu',
            #  'METHOD': 'create2image4bpu'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

            code2ack  = code2ack + '1E-'
            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - EXTRA                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecreate2bpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO BPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecreate2bpu['FUNCTION2CALLBACK'] = function2callback4bpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4imagecreate2bpu-created-by-create2image4bpu to: " + str(task4imagecreate2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4bpu at 0x00000000038D9348>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4bpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4bpu',
            #  'METHOD': 'create2image4bpu'}}; time: 00:54:15 09/06/11 Pacific Daylight Time}
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - EXTRA                      
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-create2image4bpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-create2image4bpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 00:54:15 09/06/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-create2image4bpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecreate2bpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            context2bpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2bpu-trying-by-create2image4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2bpu = get2context4bpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2bpu-got-by-create2image4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4bpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2context4bpu',
                #  'METHOD': 'create2context4bpu'},
                #  'FUNCTION2CALLBACK': <function trace2context4bpu at 0x00000000037D0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0, 'PROPERTIES': None}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2bpu-failed-by-create2image4bpu: " + str(context2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecreate2bpu ['CONTEXT'] = context2bpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - EXTRA                            
            # ---------------------------------------------------------------------------------------------    
            device2bpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2bpu-trying-by-create2image4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2bpu = get2bpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2bpu-got-by-create2image4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318,
                #  'ID2PLATFORM': '0000000001509568',
                #  'PROFILING2QUEUE': 'ACK',
                #  'EXECUTE2KERNELS4OPENCL': 'ACK',
                #  'NATIVE2WIDTHLONG4VECTOR': 2,
                #  'MAX2UNITS4COMPUTE': 4,
                #  'SIZE2MEMORY4GLOBAL': 3221225472,
                #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
                #  'MAX2SIZE4WORKGROUP': 1024,
                #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
                #  'SIZE2CACHE': 65536,
                #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
                #  'TYPE4CACHE': 'Read/Write',
                #  'PREFERRED2WIDTHINT4VECTOR': 4,
                #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
                #  'NATIVE2WIDTHINT4VECTOR': 4,
                #  'SIZE2BUFFER4CONSTANT': 65536,
                #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
                #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
                #  'MAX2NUMBER4CONSTANTARGS': 8,
                #  'VERSION2DRIVER': '2.0',
                #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics 
              
                #  'AVAILABLE2COMPILER': 'ACK',
                #  'MAX2DIMENSIONS4WORKITEMS': 3,
                #  'SUPPORT2CORRECTION4ERROR': 0,
                #  'BITS2ADDRESS': 64,
                #  'AVAILABLE': 'ACK',
                #  'SIZE2MEMORY4LOCAL': 32768,
                #  'VENDOR': 'AuthenticAMD',
                #  'SUPPORT2IMAGE': 'NACK',
                #  'MAX2SIZE4KERNELARGUMENT': 4096,
                #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-
                #  'OUT-OF-ORDER2QUEUE': 'NACK',
                #  'ALIGNMENT2ADDRESS4BASE': 1024,
                #  'PROFILE': 'FULL_PROFILE',
                #  'TYPE2MEMORY4LOCAL': 'Global',
                #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
                #  'PREFERRED2WIDTHLONG4VECTOR': 2,
                #  'MAX2FREQUENCY4CLOCK': 3210,
                #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
                #  'ID': 4033,
                #  'MAX2WORKITEM2': 1024,
                #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
                #  'ROUND2ZERO': 'ACK',
                #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
                #  'MAX2WORKITEM1': 1024,
                #  'MAX2WORKITEM0': 1024,
                #  'SIZE2LINE4CACHE': 64,
                #  'MAX2ALLOCATION4MEMORY': 1073741824,
                #  'NATIVE2WIDTHCHAR4VECTOR': 16,
                #  'QUIET2NANS': 'ACK',
                #  'NATIVE2WIDTHSHORT4VECTOR': 8,
                #  'ROUND2NEAREST4EVEN': 'ACK',
                #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
                #  'NACK2RETURN': ' ',
                #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
                #  'ENDIANESS2DEVICE': 'Little',
                #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
                #  'ROUND2+VE3INFINITY': 'ACK',
                #  'TYPE': 'CL_DEVICE_TYPE_BPU',
                #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2bpu-failed-by-create2image4bpu: " + str(device2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
                
            task4imagecreate2bpu ['DEVICE'] = device2bpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) ERROR CODE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            task4imagecreate2bpu ['CODE2ERROR'] = 0             
            code2ack  = code2ack + '5B-'              

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - EXTRA                          
            # ---------------------------------------------------------------------------------------------         
            queue2bpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2bpu-trying-by-create2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2bpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2bpu-got-by-create2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003991B70> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2bpu-failed-by-create2image4bpu: " + str(queue2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagecreate2bpu ['QUEUE'] = queue2bpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - EXTRA                              
            # ---------------------------------------------------------------------------------------------         
            pipe2bpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2bpu-trying-by-create2image4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2bpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2bpu-got-by-create2image4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2bpu-failed-by-create2image4bpu: " + str(pipe2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecreate2bpu ['PIPE'] = pipe2bpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2bpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2bpu-trying-by-create2image4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2bpu = get2commandqueueinfo4bpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2bpu-got-by-create2image4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4bpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2commandqueue4bpu',
                #  'METHOD': 'create2commandqueue4bpu'},
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4bpu at 0x000000000372C848>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
                #  'INFO2PRIVATE': None,
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A7A6D8>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0 ... 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4bpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4bpu', 
                #  'METHOD': 'create2context4bpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4bpu at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2bpu-failed-by-create2image4bpu: " + str(commandqueueinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagecreate2bpu ['COMMANDQUEUE'] = commandqueueinfo2bpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2bpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2bpu-trying-by-create2image4bpu: " + str(bufferinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2bpu = get2bufferinfo4bpu()                                                                   # SYSTEM CREATES BUFFER FOR BPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2bpu-got-by-create2image4bpu: " + str(bufferinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4bpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2buffer4bpu',
                #                    'METHOD': 'create2buffer4bpu'},
                #  'FUNCTION2CALLBACK': <function trace2buffer4bpu at 0x0000000003B2B348>,
                #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003ADD2F0>,
                #  'FLAGS': 0,
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4bpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4bpu', 
                #  'METHOD': 'create2context4bpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4bpu at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2bpu-failed-by-create2image4bpu: " + str(bufferinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecreate2bpu ['BUFFER'] = bufferinfo2bpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            eventinfo2bpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2bpu-trying-by-create2image4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2bpu = get2eventinfo4bpu()                                                                     # SYSTEM CREATES EVENT IF NECESSAY   
                message2print = "{(Step 10B/70) got" + "; eventinfo2bpu-got-by-create2image4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4bpu-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2event4bpu',
                #                    'METHOD': 'create2event4bpu'},
                #  'FUNCTION2CALLBACK': <function trace2event4bpu at 0x0000000003747D48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A02898>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4bpu-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}', 
                #                                                      'CLASS': 'create2context4bpu', 'METHOD': 'create2context4bpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4bpu at 0x00000000036F0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0,
                #  'PROPERTIES': None},
                #  'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
                
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2bpu-failed-by-create2image4bpu: " + str(eventinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecreate2bpu ['EVENT'] = eventinfo2bpu             
            
            # ---------------------------------------------------------------------------------------------
            # (11) FLAGS - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            flags2bpu = 0
           
            message2print = "{(Step 11A/70) trying" + "; flags2bpu-trying-by-create2image4bpu: " + str(flags2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2bpu = 0                                                              
                message2print = "{(Step 11B/70) set" + "; flags2bpu-set-by-create2image4bpu to: " + str(flags2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; flags2bpu-failed-by-create2image4bpu: " + str(flags2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecreate2bpu ['FLAGS'] = flags2bpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) WIDTH - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            width2bpu = 0

            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI ETC...
            width2layer = 10                                                                       # SCALED TO 10 EACH LAYER            
            
            message2print = "{(Step 14A/70) trying" + "; width2bpu-trying-by-create2image4bpu: " + str(width2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                width2bpu = int(width2layer * (quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra))
                width2bpu = 2 * width2bpu                                                          # POS and NEG X  2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 14B/70) set" + "; width2bpu-set-by-create2image4bpu to: " + str(width2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; width2bpu-failed-by-create2image4bpu: " + str(width2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecreate2bpu ['WIDTH'] = width2bpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) HEIGHT - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            height2bpu = 0
            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI
            
            quantity2total = quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra
            height2layer = 10                                                                      # SCALED TO 10 
                       
            message2print = "{(Step 15A/70) trying" + "; height2bpu-trying-by-create2image4bpu: " + str(height2bpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
                        
            try:
                height2bpu = height2layer * quantity2total
                
                height2bpu = 2 * height2bpu                                                        # POS and NEG Y   2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 15B2/70) set" + "; height2bpu-set-by-create2image4bpu to: " + str(height2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; height2bpu-failed-by-create2image4bpu: " + str(height2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '15Z-'
             
                        
            task4imagecreate2bpu ['HEIGHT'] = height2bpu 
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2bpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2bpu-trying-by-create2image4bpu: " + str(rowpitch2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2bpu = int(width2bpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2bpu-set-by-create2image4bpu to: " + str(rowpitch2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                             
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2bpu-failed-by-create2image4bpu: " + str(rowpitch2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagecreate2bpu ['ROWPITCH'] = rowpitch2bpu     
            
            # ---------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED                           
            # ---------------------------------------------------------------------------------------------             
            pointer2bpu = None
            size2bpu = int(width2bpu * height2bpu)                                                                  # CALCULATED MEMORY
            if size2bpu == 0:
                size2bpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2bpu-trying-by-create2image4bpu: " + str(pointer2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2bpu = Arena4Xi(size2bpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B1/70) got" + "; arena2bpu-got-by-create2image4bpu: " + str(arena2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041C0D68>
                
                size2arena = arena2bpu.size                                                                         # 
                              
                message2print = "{(Step 17B2/70) got" + "; size2arena-got-by-create2buffer4bpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2bpu.buffer
                
                message2print = "{(Step 17B3/70) got" + "; buffer2arena-got-by-create2buffer4bpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x0000000003AAF298> 
                
                code2ack  = code2ack + '17B-' 
                
                if size2arena < size2bpu:
                    size2bpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17C/70) set" + "; size2bpu-set-by-create2buffer4bpu to: " + str(size2bpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17C-'
                else:
                    pass
                    
                pointer2bpu = buffer2arena
                message2print = "{(Step 17D/70) set" + "; pointer2bpu-set-by-create2buffer4bpu to: " + str(pointer2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x0000000003AAF298>
                
                code2ack  = code2ack + '17D-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2bpu-failed-by-create2image4bpu: " + str(pointer2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagecreate2bpu ['POINTER'] = pointer2bpu                            
            task4imagecreate2bpu ['SIZE']    = size2bpu                                                             # EXTRA
            
            
            # ---------------------------------------------------------------------------------------------
            # (18) FORMAT - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            format2bpu = {}
           
            message2print = "{(Step 18A/70) trying" + "; format2bpu-trying-by-create2image4bpu: " + str(format2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                format2bpu['ORDER'] = 'CL_RGBA'                                                              
                message2print = "{(Step 18B/70) set" + "; format2bpu-set-by-create2image4bpu to: " + str(format2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'ORDER': 'CL_RGBA'}
                 
                code2ack  = code2ack + '18B-'
                
                format2bpu['TYPE'] = 'CL_UNORM_INT8'                                                              
                message2print = "{(Step 18C/70) set" + "; format2bpu-set-by-create2image4bpu to: " + str(format2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
                
                code2ack  = code2ack + '18C-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; format2bpu-failed-by-create2image4bpu: " + str(format2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
        
            task4imagecreate2bpu ['FORMAT'] = format2bpu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) DEPTH - MUST BE 0                               
            # ---------------------------------------------------------------------------------------------             
            depth2bpu = 0
            message2print = "{(Step 19A/70) set" + "; depth2bpu-set-by-create2image4bpu to: " + str(depth2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
            code2ack  = code2ack + '19A-' 
            task4imagecreate2bpu ['DEPTH'] = depth2bpu
                                                                                                                                       
        else:
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4imagecreate2bpu-yielded-by-create2image4bpu: " + str(task4imagecreate2bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-' 

    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4imagecreate2bpu.keys()        
    message2print = "{(Step 19A/70) using" + "; keys2task-using-by-create2image4bpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['BUFFER',
    #  'CODE2ERROR',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'FLAGS',
    #  'FORMAT',
    #  'FUNCTION2CALLBACK',
    #  'HEIGHT',
    #  'PIPE',
    #  'PLATFORM',
    #  'POINTER',
    #  'QUEUE',
    #  'ROWPITCH',
    #  'SIZE',
    #  'WIDTH']; time: 00:53:55 10/05/11 Pacific Daylight Time}
  
    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - CREATE 2D IMAGE FOR BPU! 
    # ---------------------------------------------------------------------------------------------    
    bpu2image4xi = {}                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; bpu2image4xi-initialized-by-create2image4bpu to: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {} 
    
    message2print = "{(Step 21A0/70) try" + "; data2callback-try-by-create2image4bpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA 
        # -----------------------------------------------------------------------------------------               
        bpu2image4xi['DATA2CALLBACK'] = task4imagecreate2bpu['DATA2CALLBACK']
        
        message2print = "{(Step 21A1/70) initialized" + "; data2callback-created-by-create2image4bpu to: " + str(bpu2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4bpu-failed ; time: 05:56:13 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4bpu',
        #  'METHOD': 'create2image4bpu'}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-bpu2image4xi-yielded-by-create2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-create2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        bpu2image4xi['FUNCTION2CALLBACK'] = task4imagecreate2bpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) initialized" + "; function2callback-created-by-create2image4bpu to: " + str(bpu2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # <function trace2image4bpu at 0x0000000003B04348>  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-bpu2image4xi-yielded-by-create2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform-try-by-create2image4bpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        bpu2image4xi['PLATFORM'] = task4imagecreate2bpu['PLATFORM']
        
        message2print = "{(Step 22C1/70) initialized" + "; platform-created-by-create2image4bpu to: " + str(bpu2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 05:56:13 09/06/11 Pacific Daylight Time}
  
        code2ack  = code2ack + '22C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-bpu2image4xi-yielded-by-create2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2bpu-try-by-create2image4bpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        bpu2image4xi['CONTEXT'] = task4imagecreate2bpu['CONTEXT']
        
        message2print = "{(Step 23B/70) initialized" + "; context-created-by-create2image4bpu to: " + str(bpu2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4bpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2context4bpu',
        #  'METHOD': 'create2context4bpu'},
        #  'FUNCTION2CALLBACK': <function trace2context4bpu at 0x00000000036B0148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
        #  'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-bpu2image4xi-yielded-by-create2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-create2image4bpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        bpu2image4xi['DEVICE'] = task4imagecreate2bpu['DEVICE']
        
        message2print = "{(Step 24B/70) initialized" + "; device-created-by-create2image4bpu to: " + str(bpu2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 ...
        
        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-bpu2image4xi-yielded-by-create2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-create2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        bpu2image4xi['CODE2ERROR'] = task4imagecreate2bpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) initialized" + "; code2error-created-by-create2image4bpu to: " + str(bpu2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-bpu2image4xi-yielded-by-create2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-create2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        bpu2image4xi['QUEUE'] = task4imagecreate2bpu['QUEUE']
        
        message2print = "{(Step 26B/70) initialized" + "; queue-created-by-create2image4bpu to: " + str(bpu2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003ED99E8>  
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-bpu2image4xi-yielded-by-create2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-create2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        bpu2image4xi['PIPE'] = task4imagecreate2bpu['PIPE']
        
        message2print = "{(Step 27B/70) initialized" + "; pipe-created-by-create2image4bpu to: " + str(bpu2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 376>, <read-write PipeConnection, handle 380>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-bpu2image4xi-yielded-by-create2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-create2image4bpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        bpu2image4xi['COMMANDQUEUE']  = task4imagecreate2bpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) initialized" + "; commandqueue-created-by-create2image4bpu to: " + str(bpu2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4bpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2commandqueue4bpu',
        #  'METHOD': 'create2commandqueue4bpu'},
        #  'FUNCTION2CALLBACK': <function trace2commandqueue4bpu at 0x00000000037BC848>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
        #  'INFO2PRIVATE': None,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000432B3C8>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-create2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
              
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-create2image4bpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        bpu2image4xi['BUFFER']  = task4imagecreate2bpu['BUFFER']  
 
        message2print = "{(Step 29B/70) initialized" + "; bufferinfo-created-by-create2image4bpu to: " + str(bpu2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4bpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2buffer4bpu',
        #  'METHOD': 'create2buffer4bpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4bpu at 0x0000000003AFC348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003AAF2F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4bpu-failed ...
        
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-create2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-create2image4bpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        bpu2image4xi['EVENT']  = task4imagecreate2bpu['EVENT']  
 
        message2print = "{(Step 30B/70) initialized" + "; eventinfo-created-by-create2image4bpu to: " + str(bpu2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4bpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}',
        #                     'CLASS': 'create2event4bpu',
        #                     'METHOD': 'create2event4bpu'},
        #  'FUNCTION2CALLBACK': <function trace2event4bpu at 0x0000000003707D48>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000039C3860>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-create2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-create2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        bpu2image4xi['FLAGS']  = task4imagecreate2bpu['FLAGS']  
 
        message2print = "{(Step 31B/70) initialized" + "; flags-created-by-create2image4bpu to: " + str(bpu2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-create2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-create2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        bpu2image4xi['WIDTH']  = task4imagecreate2bpu['WIDTH']  
 
        message2print = "{(Step 34H/70) initialized" + "; width-created-by-create2image4bpu to: " + str(bpu2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-create2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-create2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        bpu2image4xi['HEIGHT']  = task4imagecreate2bpu['HEIGHT']  
 
        message2print = "{(Step 35H/70) initialized" + "; height-created-by-create2image4bpu to: " + str(bpu2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-create2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-create2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        bpu2image4xi['ROWPITCH']  = task4imagecreate2bpu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) initialized" + "; rowpitch-created-by-create2image4bpu to: " + str(bpu2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-create2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-create2image4bpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        bpu2image4xi['POINTER']  = task4imagecreate2bpu['POINTER']  
 
        message2print = "{(Step 37B/70) initialized" + "; pointer-created-by-create2image4bpu to: " + str(bpu2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <mmap.mmap object at 0x0000000003C050E0>
         
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-create2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-create2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        bpu2image4xi['FORMAT']  = task4imagecreate2bpu['FORMAT']  
 
        message2print = "{(Step 38B/70) initialized" + "; format-created-by-create2image4bpu to: " + str(bpu2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-create2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '38Z1-'

    message2print = "{(Step 39A/70) try" + "; depth-try-by-create2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (39) DEPTH - MUST BE 0                              
        # ---------------------------------------------------------------------------------------------             
                        
        bpu2image4xi['DEPTH']  = task4imagecreate2bpu['DEPTH']  
 
        message2print = "{(Step 39B/70) initialized" + "; depth-created-by-create2image4bpu to: " + str(bpu2image4xi['DEPTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '39B-'
    except: 
        message2print = "{(Step 39Z1/70) yielded" + "; depth-yielded-by-create2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '39Z1-'
                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    bpu2image4xi ['ACK2RETURN']  = code2ack
    bpu2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; bpu2image4xi-returning-by-create2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    # 'DEPTH'
    # 'WIDTH'
    # 'HEIGHT'
    return bpu2image4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2image4bpu" usage = "
#    
#   print('(Step 4B/33) OF BPU2IMAGE4XI5TEST - TRYING TO GET PLATFORMIDS4XI', time4xi())
#   platform2xi = {}
#   try:
#       platform2xi = get2platformids4xi()
#       print('(Step 4B/33) of CONTEXT4BPU5TEST - GOT PLATFORMIDS4XI', platform2xi) 
#   except:
#       print('(Step 4Z/33) of CONTEXT4BPU5TEST *** FAILED TO GET PLATFORMIDS4XI', time4xi())     
#   
#   print('(Step 2A/33) OF DEVICE4BPU5TEST - TRYING TO INITIALIZE DEVICE4BPU ', time4xi())   
#   print('=' *33)        
#   try:
#       device2bpu = init2bpu4xi()                                                                                
#       print('(Step 2B/33) OF DEVICE4BPU5TEST - INITIALIZED DEVICE4BPU TO: ', device2bpu)        
#   except:
#       print('(Step 2Z/33) OF DEVICE4BPU5TEST *** FAILED TO INITIALIZE DEVICE4BPU', time4xi()) 
#   print('(Step 3A/33) OF BPU2IMAGE4XI5TEST - TRYING TO CREATE CONTEXT4BPU ', time4xi())    
#   print('=' *33)        
#   try:
#       context2bpu = create2context4bpu()                                                                               
#       print('(Step 3B/33) OF BPU2IMAGE4XI5TEST - CREATED CONTEXT4BPU: ', context2bpu)                                  
#   except:
#       print('(Step 3Z/33) OF BPU2IMAGE4XI5TEST *** FAILED TO CREATE CONTEXT4BPU', time4xi()) 
#          
#   task4set2image = {}
#   task4set2image  ['CONTEXT'] = context2bpu  
# 
#   ... other data for set operation ...
#    
#        
#   response4set2Dimage = {} 
#   print('(Step 11A/700) OF BPU2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE4BPU ', time4xi())   
#   try:                               
#       response4set2Dimage = set2image4bpu(task4set2image)                                                                                                         
#                                                                                     
#       print('(Step 11B/700) OF BPU2IMAGE4XI5TEST - SET 2D IMAGE4BPU ', time4xi())
#       keys2response = response4set2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4set2Dimage[key2response])
#           # 
#   except:
#       print('(Step 11Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE4BPU *** ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def set2image4bpu(task4imageset2bpu=None):

    '''
    sets 2D images for BPU
    '''
    global bpu2image4xi  
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set2image4bpu"
    method2print = "set2image4bpu"

    message2print = "{(Step 1A/33) started" + "; set2image4bpu-started: " + str(set2image4bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set2image4bpu at 0x0000000003A62848> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageset2bpu-to-set2image4bpu: " + str(task4imageset2bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE2ERROR': 0,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003D4A710>,
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4bpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4bpu',
    #              'METHOD': 'create2context4bpu'},
    #              'FUNCTION2CALLBACK': <function trace2context4bpu at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #  'COMMANDQUEUE': {'CODE2ERROR': 0,
    #              'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4bpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2commandqueue4bpu',
    #              'METHOD': 'create2commandqueue4bpu'},
    #  'FUNCTION2CALLBACK': <function trace2commandqueue4bpu at 0x00000000037BC848>,
    #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
    #  'INFO2PRIVATE': None,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003EDA780>,
    #  'NACK2RETURN': ' ',
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4bpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4bpu',
    #              'METHOD': 'create2context4bpu'},
    #              'FUNCTION2CALLBACK': <function trace2context4bpu at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'],
    #                           'ack2return': ' 2C-2E-2F-2G-',
    #                           'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #              'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318,
    #                          'ID2PLATFORM': '0000000001509568',
    #  'PROPERTIES': None,
    #  'PIPE': (<read-write PipeConnection, handle 552>, <read-write PipeConnection, handle 556>)},
    #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK',
    #              'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 
    #              'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 
    #              'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 
            
    code2ack  = ' '
    code2nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if bpu2image4xi is None:
            bpu2image4xi = create2image4bpu()                                                                              # INITIALIZE IMAGE FOR BPU 
            message2print = "{(Step 1C/70) set" + "; bpu2image4xi-set-by-set2image4bpu to: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4bpu-skipped-by-set2image4bpu: " + str(create2image4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function create2image4bpu at 0x0000000003A627C8>;                                  
    except:
        try:
            bpu2image4xi = create2image4bpu()
            message2print = "{(Step 1E/70) set" + "; bpu2image4xi-set-by-set2image4bpu to: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            bpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; bpu2image4xi-yielded-by-set2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET            
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data2callback-try-by-set2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        bpu2image4xi['DATA2CALLBACK'] = task4imageset2bpu['DATA2CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data2callback-set-by-set2image4bpu to: " + str(bpu2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-bpu2image4xi-yielded-by-set2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-set2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        bpu2image4xi['FUNCTION2CALLBACK'] = task4imageset2bpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function2callback-set-by-set2image4bpu to: " + str(bpu2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-bpu2image4xi-yielded-by-set2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        bpu2image4xi['PLATFORM'] = task4imageset2bpu['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set2image4bpu to: " + str(bpu2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # yielded  
        code2ack  = code2ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-bpu2image4xi-yielded-by-set2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2bpu-try-by-set2image4bpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        bpu2image4xi['CONTEXT'] = task4imageset2bpu['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set2image4bpu to: " + str(bpu2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4bpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #                    'CLASS': 'create2context4bpu',
        #                    'METHOD': 'create2context4bpu'},
        #  'FUNCTION2CALLBACK': <function trace2context4bpu at 0x0000000003770148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 07:13:59 09/06/11 Pacific Daylight Time} 

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-bpu2image4xi-yielded-by-set2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set2image4bpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        bpu2image4xi['DEVICE'] = task4imageset2bpu['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set2image4bpu to: " + str(bpu2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0',
        #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics ...
        #  'AVAILABLE2COMPILER': 'ACK',
        #  'MAX2DIMENSIONS4WORKITEMS': 3,
        #  'SUPPORT2CORRECTION4ERROR': 0,
        #  'BITS2ADDRESS': 64,
        #  'AVAILABLE': 'ACK',
        #  'SIZE2MEMORY4LOCAL': 32768,
        #  'VENDOR': 'AuthenticAMD',
        #  'SUPPORT2IMAGE': 'NACK',
        #  'MAX2SIZE4KERNELARGUMENT': 4096,
        #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-...
        #  'OUT-OF-ORDER2QUEUE': 'NACK',
        #  'ALIGNMENT2ADDRESS4BASE': 1024,
        #  'PROFILE': 'FULL_PROFILE',
        #  'TYPE2MEMORY4LOCAL': 'Global',
        #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
        #  'PREFERRED2WIDTHLONG4VECTOR': 2,
        #  'MAX2FREQUENCY4CLOCK': 3210,
        #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
        #  'ID': 4033,
        #  'MAX2WORKITEM2': 1024,
        #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
        #  'ROUND2ZERO': 'ACK',
        #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
        #  'MAX2WORKITEM1': 1024,
        #  'MAX2WORKITEM0': 1024,
        #  'SIZE2LINE4CACHE': 64,
        #  'MAX2ALLOCATION4MEMORY': 1073741824,
        #  'NATIVE2WIDTHCHAR4VECTOR': 16,
        #  'QUIET2NANS': 'ACK',
        #  'NATIVE2WIDTHSHORT4VECTOR': 8,
        #  'ROUND2NEAREST4EVEN': 'ACK',
        #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
        #  'NACK2RETURN': ' ',
        #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
        #  'ENDIANESS2DEVICE': 'Little',
        #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
        #  'ROUND2+VE3INFINITY': 'ACK',
        #  'TYPE': 'CL_DEVICE_TYPE_BPU',
        #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 07:13:59 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-bpu2image4xi-yielded-by-set2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-set2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        bpu2image4xi['CODE2ERROR'] = task4imageset2bpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code2error-set-by-set2image4bpu to: " + str(bpu2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-bpu2image4xi-yielded-by-set2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        bpu2image4xi['QUEUE'] = task4imageset2bpu['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set2image4bpu to: " + str(bpu2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003D4A710> 
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-bpu2image4xi-yielded-by-set2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        bpu2image4xi['PIPE'] = task4imageset2bpu['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set2image4bpu to: " + str(bpu2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-bpu2image4xi-yielded-by-set2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set2image4bpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        bpu2image4xi['COMMANDQUEUE']  = task4imageset2bpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set2image4bpu to: " + str(bpu2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set2image4bpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        bpu2image4xi['BUFFER']  = task4imageset2bpu['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set2image4bpu to: " + str(bpu2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set2image4bpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        bpu2image4xi['EVENT']  = task4imageset2bpu['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set2image4bpu to: " + str(bpu2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-set2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        bpu2image4xi['FLAGS']  = task4imageset2bpu['FLAGS']  
 
        message2print = "{(Step 31B/70) set" + "; flags-set-by-set2image4bpu to: " + str(bpu2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-set2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-set2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        bpu2image4xi['WIDTH']  = task4imageset2bpu['WIDTH']  
 
        message2print = "{(Step 34H/70) set" + "; width-set-by-set2image4bpu to: " + str(bpu2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-set2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-set2image4bpu: None"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        bpu2image4xi['HEIGHT']  = task4imageset2bpu['HEIGHT']  
 
        message2print = "{(Step 35H/70) set" + "; height-set-by-set2image4bpu to: " + str(bpu2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-set2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-set2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        bpu2image4xi['ROWPITCH']  = task4imageset2bpu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) set" + "; rowpitch-set-by-set2image4bpu to: " + str(bpu2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-set2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-set2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        bpu2image4xi['POINTER']  = task4imageset2bpu['POINTER']  
 
        message2print = "{(Step 37B/70) set" + "; pointer-set-by-set2image4bpu to: " + str(bpu2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-set2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-set2image4bpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        bpu2image4xi['FORMAT']  = task4imageset2bpu['FORMAT']  
 
        message2print = "{(Step 38B/70) set" + "; format-set-by-set2image4bpu to: " + str(bpu2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-set2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '38Z1-'
                        
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    bpu2image4xi ['ACK2RETURN']  = code2ack
    bpu2image4xi ['NACK2RETURN'] = code2nack 
    
    message2print = "{(Step 70/70) returning" + "; bpu2image4xi-returning-by-set2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-', 'ROWPITCH': 800, 'DATA2CALLBACK': ...
    
    
    return bpu2image4xi

           
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2image4bpu" usage = "
#                                                                                                              
#   # -----------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4BPU                      
#   # -----------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF BPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4BPU ', time4xi())    
#   response4copy2image = {} 
#   task4copy2image = None
#   try:                               
#       response4copy2image = copy2image4bpu(task4copy2image)                                                                                                          
#       print('(Step 16B/700) OF BPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGE4BPU : ', response4copy2image)
#       queue2copy4bpu      = response4copy2image['QUEUE']
#       channel2copypublish4bpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copyconsume4bpu = response4copy2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4BPU ', time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4BPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF BPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4BPU ', time4xi())     
#   try:                               
#       message2copyimagesend4bpu = "<copy4xi>copy2copy4bpu</copy4xi>"
#       channel2copypublish4bpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4bpu.send(message2copyimagesend4bpu)
#       print("(Step 17B/700) OF BPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4BPU: ", channel2copypublish4bpu, " MESSAGE2SEND4BPU: " , message2copyimagesend4bpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF BPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4BPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4BPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi()) 
#   try:        
#       response2manoeuvre4bpu = manoeuvre2copyimage4bpu(queue2copy4bpu, channel2copyconsume4bpu, channel2copypublish4bpu)
#       print("(Step 18D/700) OF BPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", manoeuvre2copyimage4bpu)
#   except:
#       print("(Step 18Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # -----------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4BPU ', time4xi())     
#   try:
#       channel2copyconsume4bpu = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4bpu = channel2copyconsume4bpu.recv()
#       print("(Step 19B/700) OF BPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4BPU: ", channel2copyconsume4bpu, " CONTROL & COMMAND: ", message2copyimagereceive4bpu)                                                                                                  
#       # <roger4xi>roger2copy4bpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4BPU ', time4xi())         
#   # -----------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4BPU TO RECEIVE                
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF BPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4BPU ', time4xi())     
#   try:                               
#       message2copyimagesend4bpu = "<copy4xi>copy2copy4bpu</copy4xi>"
#       channel2copypublish4bpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4bpu.send(message2copyimagesend4bpu)
#       print("(Step 20B/700) OF BPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4BPU: ", channel2copypublish4bpu, " MESSAGE2SEND4BPU: " , message2copyimagesend4bpu)                                                                                                
#       # <copy4xi>copy2copy4bpu</copy4xi> 
#   except:
#        print('(Step 20Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4BPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4BPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi()) 
#   try:        
#       response2manoeuvre4bpu = manoeuvre2copyimage4bpu(queue2copy4bpu, channel2copyconsume4bpu, channel2copypublish4bpu)
#       print("(Step 21B/700) OF BPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", manoeuvre2copyimage4bpu)
#   except:
#       print("(Step 21Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4BPU ', time4xi())     
#   try:
#       channel2copyconsume4bpu = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4bpu = channel2copyconsume4bpu.recv()
#       print("(Step 22B/700) OF BPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4BPU: ", channel2copyconsume4bpu, " CONTROL & COMMAND: ", message2copyimagereceive4bpu)                                                                                                  
#       # <copy4xi>copy2copy4bpu</copy4xi> 
#   except:
#       print('(Step 22Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4BPU ', time4xi())          
#   # ----------------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # ----------------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copy4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4BPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi()) 
#   try:        
#       response2manoeuvre4bpu = manoeuvre2copyimage4bpu(queue2copy4bpu, channel2copyconsume4bpu, channel2copypublish4bpu)
#       print("(Step 25B/700) OF BPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", manoeuvre2copyimage4bpu)
#   except:
#       print("(Step 25Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4bpu = channel2copyconsume4bpu.recv()
#       print("(Step 26D/700) OF BPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4BPU: ", channel2copyconsume4bpu, " CONTROL & COMMAND: ", message2copyimagereceive4bpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4bpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -----------------------------------------------------------------------------------------------
#   (27) LET MANOEUVRE2IMAGE4BPU GO TO STATE 0               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi()) 
#   try:        
#       response2manoeuvre4bpu = manoeuvre2copyimage4bpu(queue2copy4bpu, channel2copyconsume4bpu, channel2copypublish4bpu)
#       print("(Step 27B/700) OF BPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4BPU ", response2manoeuvre4bpu)
#   except:
#       print("(Step 27Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2image4bpu(task4imagecopy2bpu=None):

    '''
    copies 2D images for BPU
    '''
    global bpu2image4xi                                                                                             # 2D-IMAGES FOR BPU  
    global control2copyimage4bpu                                                                                    # COMMAND AND CONTROL FOR BPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2image4bpu"
    method2print = "copy2image4bpu"

    message2print = "{(Step 0A/33) started" + "; copy2image4bpu-started: " + str(copy2image4bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                               
    # <function copy2image4bpu at 0x00000000038538C8>
    
    message2print = "{(Step 0B/33) passed" + "; task4imagecopy2bpu-to-copy2image4bpu: " + str(task4imagecopy2bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE BPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if bpu2image4xi is None:
            bpu2image4xi = create2image4bpu()                                                                       # INITIALIZE IMAGE FOR BPU 
            message2print = "{(Step 0C/70) created" + "; bpu2image4xi-created-by-copy2image4bpu to: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        else:
            message2print = "{(Step 0D/70) skipped" + "; create2image4bpu-skipped-by-copy2image4bpu: " + str(copy2image4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function copy2image4bpu at 0x00000000038538C8>                                 
    except:
        try:
            bpu2image4xi = create2image4bpu()
            message2print = "{(Step 0E/70) set" + "; bpu2image4xi-set-by-copy2image4bpu to: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            bpu2image4xi = {}
            message2print = "{(Step 0Z/70) yielded" + "; bpu2image4xi-yielded-by-copy2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '0Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagecopy2bpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecopy2bpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagecopy2bpu-created-by-copy2image4bpu to: " + str(task4imagecopy2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {} 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4bpu = trace2image4bpu                                                                 # CALLBACK FUNCTION FOR BPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4bpu-created-by-copy2image4bpu to: " + str(function2callback4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4bpu at 0x000000000384E9C8> 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4bpu"                                                            # MODULE OF BPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4bpu"                                                            # FUNCTION OF BPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4bpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR BPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2image4bpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4bpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4bpu',
            #  'METHOD': 'copy2image4bpu'}; time: 07:01:44 10/04/11 Pacific Daylight Time}


            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2bpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO BPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2bpu['FUNCTION2CALLBACK'] = function2callback4bpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagecopy2bpu-created-by-copy2image4bpu to: " + str(task4imagecopy2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4bpu at 0x000000000384E9C8>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4bpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4bpu',
            #  'METHOD': 'copy2image4bpu'}}; time: 07:01:44 10/04/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2image4bpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2image4bpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 07:01:44 10/04/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2image4bpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecopy2bpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2bpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2bpu-trying-by-copy2image4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2bpu = get2context4bpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2bpu-got-by-copy2image4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) ...
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2bpu-failed-by-copy2image4bpu: " + str(context2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecopy2bpu ['CONTEXT'] = context2bpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2bpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2bpu-trying-by-copy2image4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2bpu = get2bpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2bpu-got-by-copy2image4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2bpu-failed-by-copy2image4bpu: " + str(device2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagecopy2bpu ['DEVICE'] = device2bpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE SOURCE OBJECT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------         
            imagesourceinfo2bpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2bpu-trying-by-copy2image4bpu: " + str(imagesourceinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagesourceinfo2bpu = get2imageinfo4bpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imagesourceinfo2bpu-got-by-copy2image4bpu: " + str(imagesourceinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2bpu-failed-by-copy2image4bpu: " + str(imagesourceinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagecopy2bpu ['IMAGESOURCE'] = imagesourceinfo2bpu         
                   
            # ---------------------------------------------------------------------------------------------
            # (6) IMAGE DESTINATION OBJECT - REQUIRED                        
            # ---------------------------------------------------------------------------------------------                         
            imagedestinationinfo2bpu = {}
             
            message2print = "{(Step 6A/70) trying" + "; imagedestinationinfo2bpu-trying-by-copy2image4bpu: " + str(imagedestinationinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagedestinationinfo2bpu = get2imageinfo4bpu()                                                                         
                
                message2print = "{(Step 6B/70) got" + "; imagedestinationinfo2bpu-got-by-copy2image4bpu: " + str(imagedestinationinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                # note: source and destination will be the same !             
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; imagedestinationinfo2bpu-failed-by-copy2image4bpu: " + str(imagedestinationinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'          
                      
            task4imagecopy2bpu ['IMAGEDESTINATION'] = imagedestinationinfo2bpu     
            
            # ---------------------------------------------------------------------------------------------
            # (7) QUEUE - REQUIRED BY BPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2bpu = None
            
            message2print = "{(Step 7A/70) trying" + "; queue2bpu-trying-by-copy2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2bpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 7B/70) got" + "; queue2bpu-got-by-copy2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x000000000453EB38>
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; queue2bpu-failed-by-copy2image4bpu: " + str(queue2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecopy2bpu ['QUEUE'] = queue2bpu

            
            # ---------------------------------------------------------------------------------------------
            # (8) PIPE - REQUIRED BY BPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2bpu = None
           
            message2print = "{(Step 8A/70) trying" + "; pipe2bpu-trying-by-copy2image4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2bpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 8B/70) got" + "; pipe2bpu-got-by-copy2image4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 612>, <read-write PipeConnection, handle 616>)
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; pipe2bpu-failed-by-copy2image4bpu: " + str(pipe2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
                
            task4imagecopy2bpu ['PIPE'] = pipe2bpu 
            
            # ---------------------------------------------------------------------------------------------
            # (9) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2bpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2bpu-trying-by-copy2image4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2bpu = get2commandqueueinfo4bpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 9B/70) got" + "; commandqueueinfo2bpu-got-by-copy2image4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4bpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4bpu', 
                #  'METHOD': 'create2commandqueue4bpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4bpu at 0x00000000037EDA48> ...
                
                code2ack  = code2ack + '9B-'  
            except:
                message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2bpu-failed-by-copy2image4bpu: " + str(commandqueueinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecopy2bpu ['COMMANDQUEUE'] = commandqueueinfo2bpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (10) BUFFER - AUTOMATIC BY BPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2bpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; bufferinfo2bpu-trying-by-copy2image4bpu: " + str(bufferinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2bpu = get2bufferinfo4bpu()                                                               # SYSTEM CREATES BUFFER FOR BPU, IF NECESSARY
                message2print = "{(Step 10B/70) got" + "; bufferinfo2bpu-got-by-copy2image4bpu: " + str(bufferinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE' ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; bufferinfo2bpu-failed-by-copy2image4bpu: " + str(bufferinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecopy2bpu ['BUFFER'] = bufferinfo2bpu 
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2bpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventinfo2bpu-trying-by-copy2image4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2bpu = get2eventinfo4bpu()                                                                    
                message2print = "{(Step 11B/70) got" + "; eventinfo2bpu-got-by-copy2image4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4bpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
                #  'CLASS': 'create2event4bpu',
                #  'METHOD': 'create2event4bpu'},
                #  'FUNCTION2CALLBACK': <function trace2event4bpu at 0x0000000003803F48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000423B668>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventinfo2bpu-failed-by-copy2image4bpu: " + str(eventinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecopy2bpu ['EVENT'] = eventinfo2bpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2bpu = {}
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlist2bpu-trying-by-copy2image4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2bpu = {}                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlist2bpu-set-by-copy2image4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2bpu-failed-by-copy2image4bpu: " + str(eventwaitlist2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagecopy2bpu ['EVENTWAITLIST'] = eventwaitlist2bpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (13) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2bpu = 0
            
            message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2bpu-trying-by-copy2image4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2bpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2bpu = eventwaitlistquantity2bpu + 1    
                
                message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2bpu-set-by-copy2image4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0  
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2bpu-failed-by-copy2image4bpu: " + str(eventwaitlistquantity2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagecopy2bpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2bpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN FOR SOURCE - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin4source2bpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin4source2bpu-trying-by-copy2image4bpu: " + str(origin4source2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4source2bpu['X'] = 0 
                origin4source2bpu['Y'] = 0  
                origin4source2bpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin4source2bpu-set-by-copy2image4bpu to: " + str(origin4source2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin4source2bpu-failed-by-copy2image4bpu: " + str(origin4source2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecopy2bpu ['SOURCEORIGIN'] = origin4source2bpu
            
            # ---------------------------------------------------------------------------------------------
            # (15) ORIGIN FOR DESTINATION - REQUIRED                             
            # ---------------------------------------------------------------------------------------------              
            origin4destination2bpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; origin4destination2bpu-trying-by-copy2image4bpu: " + str(origin4destination2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4destination2bpu['X'] = 100 
                origin4destination2bpu['Y'] = 100  
                origin4destination2bpu['Z'] = 100                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; origin4destination2bpu-set-by-copy2image4bpu to: " + str(origin4destination2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; origin4destination2bpu-failed-by-copy2image4bpu: " + str(origin4destination2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagecopy2bpu ['DESTINATIONORIGIN'] = origin4destination2bpu
                        
            
            
            # --------------------------------------------------------------------------------------------------------
            # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
            # --------------------------------------------------------------------------------------------------------             
            region4copy2bpu = {}                                                                                                         
           
            message2print = "{(Step 16A/70) trying" + "; region4copy2bpu-trying-by-copy2image4bpu: " + str(region4copy2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                region4copy2bpu['X'] = 10 
                region4copy2bpu['Y'] = 10  
                region4copy2bpu['Z'] = 10                                                                           
                message2print = "{(Step 16B/70) set" + "; region4copy2bpu-set-by-copy2image4bpu to: " + str(region4copy2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 

                code2ack  = code2ack + '16B-'                
                 
            except:
                message2print = "{(Step 16Z/70) failed" + "; region4copy2bpu-failed-by-copy2image4bpu: " + str(region4copy2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '16Z-'
            
            task4imagecopy2bpu ['REGION'] = region4copy2bpu              
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2bpu.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-copy2image4bpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:21:58 10/05/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4bpu = trace2image4bpu                                                            # CALLBACK FUNCTION FOR BPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4bpu-created-by-copy2image4bpu to: " + str(function2callback4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                 # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4bpu"                                                        # MODULE OF BPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4bpu"                                                        # FUNCTION OF BPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4bpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR BPUIZED OPENCL TO RETURN    
            
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2image4bpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2bpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO BPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2bpu['FUNCTION2CALLBACK'] = function2callback4bpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
                
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagecopy2bpu-created-by-copy2image4bpu to: " + str(task4imagecopy2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
                 
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2image4bpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2image4bpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2image4bpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagecopy2bpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2bpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2bpu-trying-by-copy2image4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2bpu = get2context4bpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2bpu-got-by-copy2image4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2bpu-failed-by-copy2image4bpu: " + str(context2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagecopy2bpu ['CONTEXT'] = context2bpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2bpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2bpu-trying-by-copy2image4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2bpu = get2bpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2bpu-got-by-copy2image4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2bpu-failed-by-copy2image4bpu: " + str(device2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagecopy2bpu ['DEVICE'] = device2bpu
        
        if 'IMAGESOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) SOURCE IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourceimageinfo2bpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2bpu-trying-by-copy2image4bpu: " + str(sourceimageinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourceimageinfo2bpu = get2imageinfo4bpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; sourceimageinfo2bpu-got-by-copy2image4bpu: " + str(sourceimageinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2bpu-failed-by-copy2image4bpu: " + str(sourceimageinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagecopy2bpu ['IMAGESOURCE'] = sourceimageinfo2bpu         
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2bpu = {}
             
            message2print = "{(Step 26A/70) trying" + "; destinationimageinfo2bpu-trying-by-copy2image4bpu: " + str(destinationimageinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2bpu = get2imageinfo4bpu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; destinationimageinfo2bpu-got-by-copy2image4bpu: " + str(destinationimageinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; destinationimageinfo2bpu-failed-by-copy2image4bpu: " + str(destinationimageinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4imagecopy2bpu ['IMAGEDESTINATION'] = destinationimageinfo2bpu    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY BPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2bpu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2bpu-trying-by-copy2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2bpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2bpu-got-by-copy2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2bpu-failed-by-copy2image4bpu: " + str(queue2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagecopy2bpu ['QUEUE'] = queue2bpu

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY BPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2bpu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2bpu-trying-by-copy2image4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2bpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2bpu-got-by-copy2image4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2bpu-failed-by-copy2image4bpu: " + str(pipe2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4imagecopy2bpu ['PIPE'] = pipe2bpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2bpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2bpu-trying-by-copy2image4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2bpu = get2commandqueueinfo4bpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2bpu-got-by-copy2image4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2bpu-failed-by-copy2image4bpu: " + str(commandqueueinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagecopy2bpu ['COMMANDQUEUE'] = commandqueueinfo2bpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) BUFFER - AUTOMATIC BY BPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2bpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; bufferinfo2bpu-trying-by-copy2image4bpu: " + str(bufferinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2bpu = get2bufferinfo4bpu()                                                           # SYSTEM CREATES BUFFER FOR BPU, IF NECESSARY
                message2print = "{(Step 30B/70) got" + "; bufferinfo2bpu-got-by-copy2image4bpu: " + str(bufferinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; bufferinfo2bpu-failed-by-copy2image4bpu: " + str(bufferinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagecopy2bpu ['BUFFER'] = bufferinfo2bpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2bpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2bpu-trying-by-copy2image4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2bpu = get2eventinfo4bpu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2bpu-got-by-copy2image4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2bpu-failed-by-copy2image4bpu: " + str(eventinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagecopy2bpu ['EVENT'] = eventinfo2bpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2bpu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2bpu-trying-by-copy2image4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2bpu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2bpu-set-by-copy2image4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2bpu-failed-by-copy2image4bpu: " + str(eventwaitlist2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagecopy2bpu ['EVENTWAITLIST'] = eventwaitlist2bpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------
            eventwaitlistquantity2bpu = 0
            
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2bpu-trying-by-copy2image4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2bpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2bpu = eventwaitlistquantity2bpu + 1    
                
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2bpu-set-by-copy2image4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2bpu-failed-by-copy2image4bpu: " + str(eventwaitlistquantity2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
                    
            task4imagecopy2bpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2bpu       
                                             
                    
        if 'SOURCEORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) SOURCEORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            source4origin2bpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; source4origin2bpu-trying-by-copy2image4bpu: " + str(source4origin2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                source4origin2bpu['X'] = 0 
                source4origin2bpu['Y'] = 0  
                source4origin2bpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; source4origin2bpu-set-by-copy2image4bpu to: " + str(source4origin2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; source4origin2bpu-failed-by-copy2image4bpu: " + str(source4origin2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagecopy2bpu ['SOURCEORIGIN'] = source4origin2bpu
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (35) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2bpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; destination4origin2bpu-trying-by-copy2image4bpu: " + str(destination4origin2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2bpu['X'] = 100 
                destination4origin2bpu['Y'] = 100  
                destination4origin2bpu['Z'] = 100                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; destination4origin2bpu-set-by-copy2image4bpu to: " + str(destination4origin2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; destination4origin2bpu-failed-by-copy2image4bpu: " + str(destination4origin2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagecopy2bpu ['DESTINATIONSOURCEORIGIN'] = destination4origin2bpu                
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2bpu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2bpu-trying-by-copy2image4bpu: " + str(region2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2bpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2bpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2bpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2bpu-set-by-copy2image4bpu to: " + str(region2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2bpu-failed-by-copy2image4bpu: " + str(region2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4imagecopy2bpu ['REGION'] = region2bpu
        
        # ---------------------------------------------------------------------------------------------
        #  EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2bpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2image4bpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagecopy2bpu-yielded-by-copy2image4bpu: " + str(task4imagecopy2bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimage4bpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2bpu = False
    channels2bpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2bpu-trying-by-copy2image4bpu: " + str(channels2bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2bpu, publisherchannel2bpu = task4imagecopy2bpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2bpu-got-by-copy2image4bpu: " + str(consumerchannel2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 612> 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2bpu-got-by-copy2image4bpu: " + str(publisherchannel2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 616>
                              
        ack4channels2bpu = True
        
        control2copyimage4bpu['CONSUMERCHANNEL']  = consumerchannel2bpu
        control2copyimage4bpu['PUBLISHERCHANNEL'] = publisherchannel2bpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2bpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2bpu = False
    queue2bpu = None
    message2print = "{(Step 41A/70) trying" + "queue2bpu-trying-by-copy2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2bpu = task4imagecopy2bpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2bpu-got-by-copy2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004721CF8>
              
        ack4queue2bpu = True
        control2copyimage4bpu['QUEUE']  = queue2bpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2bpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4bpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4bpu-trying-by-copy2image4bpu: " + str(dummy2image4bpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4bpu = DummyProcess4Xi(target=manoeuvre2copyimage4bpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2bpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2bpu, publisherchannel2bpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4bpu-created-by-copy2image4bpu: " + str(dummy2image4bpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2image4bpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2image4bpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimage4bpu-returning-by-copy2image4bpu: " + str(control2copyimage4bpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004721CF8>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 616>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 612>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
   
    return control2copyimage4bpu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2imagetobuffer4bpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (60) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4BPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 60A/700) OF BPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4BPU ', time4xi())    
#   response4copy2imagetobuffer = {} 
#   task4copy2imagetobuffer = None
#   try:                               
#       response4copy2imagetobuffer = copy2imagetobuffer4bpu(task4copy2imagetobuffer)                                                                                                          
#       print('(Step 60B/700) OF BPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4BPU : ', response4copy2imagetobuffer)
#       queue2copyimagetobuffer4bpu      = response4copy2imagetobuffer['QUEUE']
#       channel2copyimagetobufferpublish4bpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferconsume4bpu = response4copy2imagetobuffer['CONSUMERCHANNEL']
#   except:
#       print('(Step 60Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4BPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (61) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4BPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 61A/700) OF BPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4BPU ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4bpu = "<copy4xi>copy2copyimagetobuffer4bpu</copy4xi>"
#       channel2copyimagetobufferpublish4bpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4bpu.send(message2copyimagetobuffersend4bpu)
#       print("(Step 61B/700) OF BPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4BPU: ", channel2copyimagetobufferpublish4bpu, " MESSAGE2SEND4BPU: " , message2copyimagetobuffersend4bpu)                                                                                                
#   except:
#       print('(Step 61Z1/700) OF BPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4BPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (62) LET MANOEUVRE2COPYIMAGETOBUFFER4BPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 62A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4bpu = manoeuvre2copyimagetobuffer4bpu(queue2copyimagetobuffer4bpu, channel2copyimagetobufferconsume4bpu, channel2copyimagetobufferpublish4bpu)
#       print("(Step 62B/700) OF BPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", manoeuvre2copyimagetobuffer4bpu)
#   except:
#       print("(Step 62Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (63) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 63A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4BPU ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4bpu = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4bpu = channel2copyimagetobufferconsume4bpu.recv()
#       print("(Step 63B/700) OF BPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4BPU: ", channel2copyimagetobufferconsume4bpu, " CONTROL & COMMAND: ", message2copyimagereceive4bpu)                                                                                                  
#       # 
#   except:
#       print('(Step 63Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4BPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (64) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4BPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 64A/700) OF BPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4BPU ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4bpu = "<copy4xi>copy2copyimagetobuffer4bpu</copy4xi>"
#       channel2copyimagetobufferpublish4bpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4bpu.send(message2copyimagetobuffersend4bpu)
#       print("(Step 64B/700) OF BPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4BPU: ", channel2copyimagetobufferpublish4bpu, " MESSAGE2SEND4BPU: " , message2copyimagetobuffersend4bpu)                                                                                                
#       #  
#   except:
#       print('(Step 64Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4BPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (65) LET MANOEUVRE2COPYIMAGETOBUFFER4BPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 65A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4bpu = manoeuvre2copyimagetobuffer4bpu(queue2copyimagetobuffer4bpu, channel2copyimagetobufferconsume4bpu, channel2copyimagetobufferpublish4bpu)
#       print("(Step 65B/700) OF BPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", manoeuvre2copyimagetobuffer4bpu)
#   except:
#       print("(Step 65Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (66) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 66A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4BPU ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4bpu = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4bpu = channel2copyimagetobufferconsume4bpu.recv()
#       print("(Step 66B/700) OF BPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4BPU: ", channel2copyimagetobufferconsume4bpu, " CONTROL & COMMAND: ", message2copyimagereceive4bpu)                                                                                                  
#       # 
#   except:
#       print('(Step 66Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4BPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (67) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 67A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copyimagetobuffer4dragon.put(path2dragon)             
#       print("(Step 67B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 67Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (68) LET MANOEUVRE2COPYIMAGETOBUFFER4BPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 68A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4bpu = manoeuvre2copyimagetobuffer4bpu(queue2copyimagetobuffer4bpu, channel2copyimagetobufferconsume4bpu, channel2copyimagetobufferpublish4bpu)
#       print("(Step 68B/700) OF BPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", manoeuvre2copyimagetobuffer4bpu)
#   except:
#       print("(Step 68Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (69) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 69A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4bpu = channel2copyimagetobufferconsume4bpu.recv()
#       print("(Step 69D/700) OF BPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4BPU: ", channel2copyimagetobufferconsume4bpu, " CONTROL & COMMAND: ", message2copyimagereceive4bpu)                                                                                                   
#   except:
#       print('(Step 69Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (70) LET MANOEUVRE2COPYIMAGETOBUFFER4BPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 70A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4bpu = manoeuvre2copyimagetobuffer4bpu(queue2copyimagetobuffer4bpu, channel2copyimagetobufferconsume4bpu, channel2copyimagetobufferpublish4bpu)
#       print("(Step 70B/700) OF BPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4BPU ", response2manoeuvreimagecopytobuffer4bpu)
#   except:
#       print("(Step 70Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2imagetobuffer4bpu(task4copyimagetobuffer2bpu=None):

    '''
    copies 2D images to buffer for BPU
    '''
    global bpu2image4xi                                                                                             # 2D-IMAGES FOR BPU  
    global bpu2buffer4xi                                                                                            # BUFFER FOR BPU
    
    global control2copyimagetobuffer4bpu                                                                            # COMMAND AND CONTROL FOR BPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2imagetobuffer4bpu"
    method2print = "copy2imagetobuffer4bpu"

    message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer4bpu-started: " + str(copy2imagetobuffer4bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                 
    # <function copy2imagetobuffer4bpu at 0x000000000390B948>
    
    message2print = "{(Step 1B/33) passed" + "; task4copyimagetobuffer2bpu-to-copy2imagetobuffer4bpu: " + str(task4copyimagetobuffer2bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE BPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if bpu2image4xi is None:
            bpu2image4xi = create2image4bpu()                                                                              # INITIALIZE IMAGE FOR BPU 
            message2print = "{(Step 1C/70) created" + "; bpu2image4xi-created-by-copy2imagetobuffer4bpu to: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4bpu-skipped-by-copy2imagetobuffer4bpu: " + str(copy2imagetobuffer4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4bpu-skipped-by-copy2imagetobuffer4bpu                                 
    except:
        try:
            bpu2image4xi = create2image4bpu()
            message2print = "{(Step 1E/70) set" + "; bpu2image4xi-set-by-copy2imagetobuffer4bpu to: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            bpu2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; bpu2image4xi-yielded-by-copy2imagetobuffer4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE BPU2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if bpu2buffer4xi is None:
            bpu2buffer4xi = create2buffer4bpu()                                                                     # INITIALIZE BUFFER FOR BPU 
            message2print = "{(Step 1F/70) created" + "; bpu2buffer4xi-created-by-copy2imagetobuffer4bpu to: " + str(bpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4bpu-skipped-by-copy2imagetobuffer4bpu: " + str(copy2imagetobuffer4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                  
    except:
        try:
            bpu2buffer4xi = create2buffer4bpu()
            message2print = "{(Step 1H/70) set" + "; bpu2buffer4xi-set-by-copy2imagetobuffer4bpu to: " + str(bpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0,
            #  'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4bpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}',
            #   'CLASS': 'create2buffer4bpu',
            #   'METHOD': 'create2buffer4bpu'},
            #   'FUNCTION2CALLBACK': <function trace2buffer4bpu at 0x0000000004782B48>,
            #   'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>,
            #   'FLAGS': 0,
            #   'CONTEXT': {'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': ...
            
        except:
            bpu2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; bpu2buffer4xi-yielded-by-copy2imagetobuffer4bpu: " + str(bpu2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASK          
    # -----------------------------------------------------------------------------------------          
              
    if task4copyimagetobuffer2bpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
            
        task4copyimagetobuffer2bpu = {}                                                                                                                                                                                       
            
        message2print = "{(Step 1A/70) initialized" + "; task4copyimagetobuffer2bpu-created-by-copy2imagetobuffer4bpu to: " + str(task4copyimagetobuffer2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
            
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4bpu = trace2image4bpu                                                                 # CALLBACK FUNCTION FOR BPU
            
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4bpu-created-by-copy2imagetobuffer4bpu to: " + str(function2callback4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4bpu at 0x0000000004797548> 
            
        code2ack  = code2ack + '1B1-'
           
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4bpu"                                                            # MODULE OF BPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4bpu"                                                            # FUNCTION OF BPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4bpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR BPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4bpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4bpu-failed; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4bpu', 
        #  'METHOD': 'copy2imagetobuffer4bpu'}; time: 03:40:47 10/04/11 Pacific Daylight Time}


        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2bpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO BPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2bpu['FUNCTION2CALLBACK'] = function2callback4bpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copyimagetobuffer2bpu-created-by-copy2imagetobuffer4bpu to: " + str(task4copyimagetobuffer2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4bpu at 0x0000000004797548>,
        #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4bpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4bpu', 
        #  'METHOD': 'copy2imagetobuffer4bpu'}}; time: 03:40:47 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4bpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4bpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4bpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copyimagetobuffer2bpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2bpu = {}
        message2print = "{(Step 3A/70) trying" + "; context2bpu-trying-by-copy2imagetobuffer4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2bpu = get2context4bpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2bpu-got-by-copy2imagetobuffer4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4bpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4bpu', 
            #  'METHOD': 'create2context4bpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4bpu at 0x000000000394F348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2bpu-failed-by-copy2imagetobuffer4bpu: " + str(context2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
            
        task4copyimagetobuffer2bpu ['CONTEXT'] = context2bpu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2bpu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2bpu-trying-by-copy2imagetobuffer4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2bpu = get2bpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2bpu-got-by-copy2imagetobuffer4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, 
            #  'NATIVE2WIDTHFLOAT4VECTOR': 4, 
            #  'MAX2SIZE4WORKGROUP': 1024, 
            #  'PREFERRED2WIDTHCHAR4VECTOR': 16, 
            #  'SIZE2CACHE': 65536, 
            #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 
            #  'TYPE4CACHE': 'Read/Write', 
            #  'PREFERRED2WIDTHINT4VECTOR': 4, 
            #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 
            #  'NATIVE2WIDTHINT4VECTOR': 4, 
            #  'SIZE2BUFFER4CONSTANT': 65536, 
            #  'EXECUTE2FUNCTION4NATIVE': 'ACK', 
            #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 
            #  'MAX2NUMBER4CONSTANTARGS': 8, 
            #  'VERSION2DRIVER': '2.0', 
            #  'EXTENSIONS': 'cl_amd_fp64 ...
            #  'AVAILABLE2COMPILER': 'ACK', 
            #  'MAX2DIMENSIONS4WORKITEMS': 3, 
            #  'SUPPORT2CORRECTION4ERROR': 0, 
            #  'BITS2ADDRESS': 64, 
            #  'AVAILABLE': 'ACK', 
            #  'SIZE2MEMORY4LOCAL': 32768, 
            #  'VENDOR': 'AuthenticAMD', 
            #  'SUPPORT2IMAGE': 'NACK', 
            #  'MAX2SIZE4KERNELARGUMENT': 4096, 
            #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-...
            #  'OUT-OF-ORDER2QUEUE': 'NACK', 
            #  'ALIGNMENT2ADDRESS4BASE': 1024, 
            #  'PROFILE': 'FULL_PROFILE', 
            #  'TYPE2MEMORY4LOCAL': 'Global', 
            #  'PREFERRED2WIDTHFLOAT4VECTOR': 4, 
            #  'PREFERRED2WIDTHLONG4VECTOR': 2, 
            #  'MAX2FREQUENCY4CLOCK': 3210, 
            #  'PREFERRED2WIDTHSHORT4VECTOR': 8, 
            #  'ID': 4033, 
            #  'MAX2WORKITEM2': 1024, 
            #  'NATIVE2WIDTHDOUBLE4VECTOR': 0, 
            #  'ROUND2ZERO': 'ACK', 
            #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, 
            #   AuthenticAMD', 
            #  'MAX2WORKITEM1': 1024, 
            #  'MAX2WORKITEM0': 1024, 
            #  'SIZE2LINE4CACHE': 64, 
            #  'MAX2ALLOCATION4MEMORY': 1073741824, 
            #  'NATIVE2WIDTHCHAR4VECTOR': 16, 
            #  'QUIET2NANS': 'ACK', 
            #  'NATIVE2WIDTHSHORT4VECTOR': 8, 
            #  'ROUND2NEAREST4EVEN': 'ACK', 
            #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 
            #  'NACK2RETURN': ' ', 
            #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 
            #  'ENDIANESS2DEVICE': 'Little', 
            #  'MINIMUM2ALIGNMENT4DATATYPE': 128, 
            #  'ROUND2+VE3INFINITY': 'ACK', 
            #  'TYPE': 'CL_DEVICE_TYPE_BPU', 
            #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 03:40:47 10/04/11 Pacific Daylight Time}
            
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2bpu-failed-by-copy2imagetobuffer4bpu: " + str(device2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
        
        task4copyimagetobuffer2bpu ['DEVICE'] = device2bpu
        
               
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE SOURCE OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagesourceinfo2bpu = {}
         
        message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2bpu-trying-by-copy2imagetobuffer4bpu: " + str(imagesourceinfo2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagesourceinfo2bpu = get2imageinfo4bpu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagesourceinfo2bpu-got-by-copy2imagetobuffer4bpu: " + str(imagesourceinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000042514E0>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': ...

            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2bpu-failed-by-copy2imagetobuffer4bpu: " + str(imagesourceinfo2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copyimagetobuffer2bpu ['IMAGESOURCE'] = imagesourceinfo2bpu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER DESTINATION OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        bufferdestinationinfo2bpu = {}
         
        message2print = "{(Step 6A/70) trying" + "; bufferdestinationinfo2bpu-trying-by-copy2imagetobuffer4bpu: " + str(bufferdestinationinfo2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            bufferdestinationinfo2bpu = get2bufferinfo4bpu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; bufferdestinationinfo2bpu-got-by-copy2imagetobuffer4bpu: " + str(bufferdestinationinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4bpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4bpu', 
            #  'METHOD': 'create2buffer4bpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4bpu at 0x0000000004782B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; bufferdestinationinfo2bpu-failed-by-copy2imagetobuffer4bpu: " + str(bufferdestinationinfo2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copyimagetobuffer2bpu ['BUFFERDESTINATION'] = bufferdestinationinfo2bpu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY BPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4bpu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copyimagetobuffer4bpu-trying-by-copy2imagetobuffer4bpu: " + str(queue2copyimagetobuffer4bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4bpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copyimagetobuffer4bpu-got-by-copy2imagetobuffer4bpu: " + str(queue2copyimagetobuffer4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x00000000042B2208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copyimagetobuffer4bpu-failed-by-copy2imagetobuffer4bpu: " + str(queue2copyimagetobuffer4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copyimagetobuffer2bpu ['QUEUE'] = queue2copyimagetobuffer4bpu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY BPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2bpu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2bpu-trying-by-copy2imagetobuffer4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2bpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2bpu-got-by-copy2imagetobuffer4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 660>, <read-write PipeConnection, handle 664>)
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2bpu-failed-by-copy2imagetobuffer4bpu: " + str(pipe2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copyimagetobuffer2bpu ['PIPE'] = pipe2bpu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2bpu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2bpu-trying-by-copy2imagetobuffer4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2bpu = get2commandqueueinfo4bpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2bpu-got-by-copy2imagetobuffer4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4bpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2commandqueue4bpu', 
            #  'METHOD': 'create2commandqueue4bpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2commandqueue4bpu at 0x000000000395EA48>, 
            #  'ACK2RETURN': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2bpu-failed-by-copy2imagetobuffer4bpu: " + str(commandqueueinfo2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copyimagetobuffer2bpu ['COMMANDQUEUE'] = commandqueueinfo2bpu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2bpu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2bpu-trying-by-copy2imagetobuffer4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2bpu = get2eventinfo4bpu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2bpu-got-by-copy2imagetobuffer4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4bpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2event4bpu', 
            #  'METHOD': 'create2event4bpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2event4bpu at 0x0000000003971F48>, 
            #  'ACK2RETURN': ' 1D-1E-...
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2bpu-failed-by-copy2imagetobuffer4bpu: " + str(eventinfo2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copyimagetobuffer2bpu ['EVENT'] = eventinfo2bpu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2bpu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2bpu-trying-by-copy2imagetobuffer4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2bpu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2bpu-set-by-copy2imagetobuffer4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2bpu-failed-by-copy2imagetobuffer4bpu: " + str(eventwaitlist2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copyimagetobuffer2bpu ['EVENTWAITLIST'] = eventwaitlist2bpu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2bpu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2bpu-trying-by-copy2imagetobuffer4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2bpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2bpu = eventwaitlistquantity2bpu + 1                                                                      
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2bpu-set-by-copy2imagetobuffer4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2bpu-failed-by-copy2imagetobuffer4bpu: " + str(eventwaitlistquantity2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copyimagetobuffer2bpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2bpu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (14) ORIGIN FOR SOURCE - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        origin4source2bpu = {}
        
        message2print = "{(Step 14A/70) trying" + "; origin4source2bpu-trying-by-copy2imagetobuffer4bpu: " + str(origin4source2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4source2bpu['X'] = 0 
            origin4source2bpu['Y'] = 0  
            origin4source2bpu['Z'] = 0                                                                                                                          
            message2print = "{(Step 14B/70) set" + "; origin4source2bpu-set-by-copy2imagetobuffer4bpu to: " + str(origin4source2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 0, 'X': 0, 'Z': 0} 
            code2ack  = code2ack + '14B-' 
        except:
            message2print = "{(Step 14Z/70) failed" + "; origin4source2bpu-failed-by-copy2imagetobuffer4bpu: " + str(origin4source2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '14Z-'
    
        task4copyimagetobuffer2bpu ['SOURCEORIGIN'] = origin4source2bpu
        
       
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2bpu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2bpu-trying-by-copy2imagetobuffer4bpu: " + str(region4copy2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2bpu['X'] = 10 
            region4copy2bpu['Y'] = 10  
            region4copy2bpu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2bpu-set-by-copy2imagetobuffer4bpu to: " + str(region4copy2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10} 
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2bpu-failed-by-copy2imagetobuffer4bpu: " + str(region4copy2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copyimagetobuffer2bpu ['REGION'] = region4copy2bpu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER                     
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2bpu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2bpu-set-by-copy2imagetobuffer4bpu to: " + str(offset4copy2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '17A-'                
        task4copyimagetobuffer2bpu ['OFFSET'] = offset4copy2bpu            
                    
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
    else:
        pass
                    
    # ---------------------------------------------------------------------------------------------
    # ANALYZE TASK - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2bpu.keys()
    message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2imagetobuffer4bpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}


    if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
        pass
    else:                
        # ---------------------------------------------------------------------------------------------
        # (21B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4bpu = trace2image4bpu                                                             # CALLBACK FUNCTION FOR BPU
        
        message2print = "{(Step 21B1/70) initialized" + "; function2callback4bpu-created-by-copy2imagetobuffer4bpu to: " + str(function2callback4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
            
        code2ack  = code2ack + '21B1-'
           
        args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4bpu"                                                        # MODULE OF BPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4bpu"                                                        # FUNCTION OF BPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4bpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR BPUIZED OPENCL TO RETURN    

        message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4bpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 

        code2ack  = code2ack + '21B2-'
        
        # ---------------------------------------------------------------------------------------------
        # (21C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2bpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO BPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2bpu['FUNCTION2CALLBACK'] = function2callback4bpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '21C-'
        message2print = "{(Step 21C/70) initialized" + "; task4copyimagetobuffer2bpu-created-by-copy2imagetobuffer4bpu to: " + str(task4copyimagetobuffer2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #
             
    if 'PLATFORM' in keys2task:
        pass
    else:  
        # ---------------------------------------------------------------------------------------------
        # (22) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4bpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        try:
            platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4bpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                 
            code2ack  = code2ack + '22B-'
        except:
            message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4bpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '22Z-'
                 
        task4copyimagetobuffer2bpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
    if 'CONTEXT' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (23) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2bpu = {}
        message2print = "{(Step 23A/70) trying" + "; context2bpu-trying-by-copy2imagetobuffer4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2bpu = get2context4bpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
            
            message2print = "{(Step 23B/70) got" + "; context2bpu-got-by-copy2imagetobuffer4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '23B-'         
        except:
            message2print = "{(Step 23Z/70) failed" + "; context2bpu-failed-by-copy2imagetobuffer4bpu: " + str(context2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '23Z-'
            
        task4copyimagetobuffer2bpu ['CONTEXT'] = context2bpu           
    
    if 'DEVICE' in keys2task:
        pass
    else:                   
        # ---------------------------------------------------------------------------------------------
        # (24) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2bpu = {}
        
        message2print = "{(Step 24A/70) trying" + "; device2bpu-trying-by-copy2imagetobuffer4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2bpu = get2bpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 24B/70) got" + "; device2bpu-got-by-copy2imagetobuffer4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
                
            code2ack  = code2ack + '24B-'  
        except:
            message2print = "{(Step 24Z/70) failed" + "; device2bpu-failed-by-copy2imagetobuffer4bpu: " + str(device2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '24Z-'
        
        task4copyimagetobuffer2bpu ['DEVICE'] = device2bpu
    
    if 'IMAGESOURCE' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (25) SOURCE IMAGE - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        sourceimageinfo2bpu = {}
         
        message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2bpu-trying-by-copy2imagetobuffer4bpu: " + str(sourceimageinfo2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            sourceimageinfo2bpu = get2imageinfo4bpu()                                                                         
            
            message2print = "{(Step 25B/70) got" + "; sourceimageinfo2bpu-got-by-copy2imagetobuffer4bpu: " + str(sourceimageinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '25B-'  
        except:
            message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2bpu-failed-by-copy2imagetobuffer4bpu: " + str(sourceimageinfo2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '25Z-'          
                  
        task4copyimagetobuffer2bpu ['IMAGESOURCE'] = sourceimageinfo2bpu         
    
    if 'BUFFERDESTINATION' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (26) DESTINATION BUFFER - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        destinationbufferinfo2bpu = {}
         
        message2print = "{(Step 26A/70) trying" + "; destinationbufferinfo2bpu-trying-by-copy2imagetobuffer4bpu: " + str(destinationbufferinfo2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            destinationbufferinfo2bpu = get2bufferinfo4bpu()                                                                         
            
            message2print = "{(Step 26B/70) got" + "; destinationbufferinfo2bpu-got-by-copy2imagetobuffer4bpu: " + str(destinationbufferinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '26B-'  
        except:
            message2print = "{(Step 26Z/70) failed" + "; destinationbufferinfo2bpu-failed-by-copy2imagetobuffer4bpu: " + str(destinationbufferinfo2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '26Z-'          
                  
        task4copyimagetobuffer2bpu ['BUFFERDESTINATION'] = destinationbufferinfo2bpu    
                                
    if 'QUEUE' in keys2task:
        pass
    else:   
        # ---------------------------------------------------------------------------------------------
        # (27) QUEUE - REQUIRED BY BPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4bpu = None
        
        message2print = "{(Step 27A/70) trying" + "; queue2copyimagetobuffer4bpu-trying-by-copy2imagetobuffer4bpu: " + str(queue2copyimagetobuffer4bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4bpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 27B/70) got" + "; queue2copyimagetobuffer4bpu-got-by-copy2imagetobuffer4bpu: " + str(queue2copyimagetobuffer4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '27B-'  
        except:
            message2print = "{(Step 27Z/70) failed" + "; queue2copyimagetobuffer4bpu-failed-by-copy2imagetobuffer4bpu: " + str(queue2copyimagetobuffer4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '27Z-'
            
        task4copyimagetobuffer2bpu ['QUEUE'] = queue2copyimagetobuffer4bpu
        
    if 'PIPE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) PIPE - REQUIRED BY BPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2bpu = None
       
        message2print = "{(Step 28A/70) trying" + "; pipe2bpu-trying-by-copy2imagetobuffer4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2bpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 28B/70) got" + "; pipe2bpu-got-by-copy2imagetobuffer4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '28B-'  
        except:
            message2print = "{(Step 28Z/70) failed" + "; pipe2bpu-failed-by-copy2imagetobuffer4bpu: " + str(pipe2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '28Z-'
            
        task4copyimagetobuffer2bpu ['PIPE'] = pipe2bpu 
        
    if 'COMMANDQUEUE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2bpu = {}
       
        message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2bpu-trying-by-copy2imagetobuffer4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2bpu = get2commandqueueinfo4bpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 29B/70) got" + "; commandqueueinfo2bpu-got-by-copy2imagetobuffer4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '29B-'  
        except:
            message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2bpu-failed-by-copy2imagetobuffer4bpu: " + str(commandqueueinfo2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '29Z-'
    
        task4copyimagetobuffer2bpu ['COMMANDQUEUE'] = commandqueueinfo2bpu
    
         
    if 'EVENT' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (31) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2bpu = {}
       
        message2print = "{(Step 31A/70) trying" + "; eventinfo2bpu-trying-by-copy2imagetobuffer4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2bpu = get2eventinfo4bpu()                                                                    
            message2print = "{(Step 31B/70) got" + "; eventinfo2bpu-got-by-copy2imagetobuffer4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '31B-' 
        except:
            message2print = "{(Step 31Z/70) failed" + "; eventinfo2bpu-failed-by-copy2imagetobuffer4bpu: " + str(eventinfo2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '31Z-'
    
        task4copyimagetobuffer2bpu ['EVENT'] = eventinfo2bpu 
    
         
    if 'EVENTWAITLIST' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (32) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2bpu = {}
       
        message2print = "{(Step 32A/70) trying" + "; eventwaitlist2bpu-trying-by-copy2imagetobuffer4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2bpu = {}                                                                   
            message2print = "{(Step 32B/70) set" + "; eventwaitlist2bpu-set-by-copy2imagetobuffer4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '32B-' 
        except:
            message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2bpu-failed-by-copy2imagetobuffer4bpu: " + str(eventwaitlist2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '32Z-'
    
        task4copyimagetobuffer2bpu ['EVENTWAITLIST'] = eventwaitlist2bpu                     
          
    if 'EVENTWAITLISTQUANTITY' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2bpu = 0
       
        message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2bpu-trying-by-copy2imagetobuffer4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2bpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2bpu = eventwaitlistquantity2bpu + 1                                                                   
            message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2bpu-set-by-copy2imagetobuffer4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '33B-' 
        except:
            message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2bpu-failed-by-copy2imagetobuffer4bpu: " + str(eventwaitlistquantity2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '33Z-'
    
        task4copyimagetobuffer2bpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2bpu       
                                         
                
    if 'SOURCEORIGIN' in keys2task:
        pass
    else:                    
        # ---------------------------------------------------------------------------------------------
        # (34) SOURCEORIGIN - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        source4origin2bpu = {}
        
        message2print = "{(Step 34A/70) trying" + "; source4origin2bpu-trying-by-copy2imagetobuffer4bpu: " + str(source4origin2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            source4origin2bpu['X'] = 0 
            source4origin2bpu['Y'] = 0  
            source4origin2bpu['Z'] = 0                                                                                                                          
            message2print = "{(Step 34B/70) set" + "; source4origin2bpu-set-by-copy2imagetobuffer4bpu to: " + str(source4origin2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '34B-' 
        except:
            message2print = "{(Step 34Z/70) failed" + "; source4origin2bpu-failed-by-copy2imagetobuffer4bpu: " + str(source4origin2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '34Z-'
    
        task4copyimagetobuffer2bpu ['SOURCEORIGIN'] = source4origin2bpu
             
   
             
    if 'REGION' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (36) REGION - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        region2bpu = {}
        
        message2print = "{(Step 36A/70) trying" + "; region2bpu-trying-by-copy2imagetobuffer4bpu: " + str(region2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            region2bpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
            region2bpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
            region2bpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
            message2print = "{(Step 36B/70) set" + "; region2bpu-set-by-copy2imagetobuffer4bpu to: " + str(region2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '36B-' 
        except:
            message2print = "{(Step 36Z/70) failed" + "; region2bpu-failed-by-copy2imagetobuffer4bpu: " + str(region2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '36Z-'
    
        task4copyimagetobuffer2bpu ['REGION'] = region2bpu
    
         
    if 'OFFSET' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (37) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        offset2bpu = 0
        message2print = "{(Step 37A/70) set" + "; offset2bpu-set-by-copy2imagetobuffer4bpu to: " + str(offset2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        task4copyimagetobuffer2bpu ['OFFSET'] = offset2bpu        
        code2ack  = code2ack + '37A-' 
        
    # ---------------------------------------------------------------------------------------------
    # EOF TASK
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2bpu.keys()
    message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2imagetobuffer4bpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
               


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimagetobuffer4bpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2bpu = False
    channels2bpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2bpu-trying-by-copy2imagetobuffer4bpu: " + str(channels2bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copyimagetobuffer4bpu, publisherchannel2copyimagetobuffer4bpu = task4copyimagetobuffer2bpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copyimagetobuffer4bpu-got-by-copy2imagetobuffer4bpu: " + str(consumerchannel2copyimagetobuffer4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 660>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copyimagetobuffer4bpu-got-by-copy2imagetobuffer4bpu: " + str(publisherchannel2copyimagetobuffer4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 664>                     
        ack4channels2bpu = True
        
        control2copyimagetobuffer4bpu['CONSUMERCHANNEL']  = consumerchannel2copyimagetobuffer4bpu
        
        
        control2copyimagetobuffer4bpu['PUBLISHERCHANNEL'] = publisherchannel2copyimagetobuffer4bpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2bpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copyimagetobuffer4bpu = False
    queue2copyimagetobuffer4bpu = None
    message2print = "{(Step 41A/70) trying" + "queue2copyimagetobuffer4bpu-trying-by-copy2imagetobuffer4bpu: " + str(queue2copyimagetobuffer4bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copyimagetobuffer4bpu = task4copyimagetobuffer2bpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copyimagetobuffer4bpu-got-by-copy2imagetobuffer4bpu: " + str(queue2copyimagetobuffer4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004465E48>
              
        ack4queue2copyimagetobuffer4bpu = True
        control2copyimagetobuffer4bpu['QUEUE']  = queue2copyimagetobuffer4bpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copyimagetobuffer4bpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4bpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4bpu-trying-by-copy2imagetobuffer4bpu: " + str(dummy2image4bpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4bpu = DummyProcess4Xi(target=manoeuvre2copyimagetobuffer4bpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copyimagetobuffer4bpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copyimagetobuffer4bpu, publisherchannel2copyimagetobuffer4bpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4bpu-created-by-copy2imagetobuffer4bpu: " + str(dummy2image4bpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2imagetobuffer4bpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-14B-16B-17A-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2imagetobuffer4bpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimagetobuffer4bpu-returning-by-copy2imagetobuffer4bpu: " + str(control2copyimagetobuffer4bpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004465E48>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 664>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 660>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2copyimagetobuffer4bpu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2buffertoimage4bpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (80) ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4BPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 80A/700) OF BPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4BPU ', time4xi())    
#   response4copy2buffertoimage = {} 
#   task4copy2buffertoimage = None
#   try:                               
#       response4copy2buffertoimage = copy2buffertoimage4bpu(task4copy2buffertoimage)                                                                                                          
#       print('(Step 80B/700) OF BPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4BPU : ', response4copy2buffertoimage)
#       queue2copybuffertoimage4bpu      = response4copy2buffertoimage['QUEUE']
#       channel2copybuffertoimagepublish4bpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimageconsume4bpu = response4copy2buffertoimage['CONSUMERCHANNEL']
#   except:
#       print('(Step 80Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4BPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (81) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4BPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 81A/700) OF BPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4BPU ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4bpu = "<copy4xi>copy2copybuffertoimage4bpu</copy4xi>"
#       channel2copybuffertoimagepublish4bpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4bpu.send(message2copybuffertoimagesend4bpu)
#       print("(Step 81B/700) OF BPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4BPU: ", channel2copybuffertoimagepublish4bpu, " MESSAGE2SEND4BPU: " , message2copybuffertoimagesend4bpu)                                                                                                
#   except:
#       print('(Step 81Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4BPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (82) LET MANOEUVRE2COPYIMAGETOBUFFER4BPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 82C/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4bpu = manoeuvre2copybuffertoimage4bpu(queue2copybuffertoimage4bpu, channel2copybuffertoimageconsume4bpu, channel2copybuffertoimagepublish4bpu)
#       print("(Step 82D/700) OF BPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", manoeuvre2copybuffertoimage4bpu)
#   except:
#       print("(Step 82Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (83) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 83A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4BPU ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4bpu = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4bpu = channel2copybuffertoimageconsume4bpu.recv()
#       print("(Step 83B/700) OF BPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4BPU: ", channel2copybuffertoimageconsume4bpu, " CONTROL & COMMAND: ", message2copyimagereceive4bpu)                                                                                                  
#       # 
#   except:
#       print('(Step 83Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4BPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (84) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4BPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 84A/700) OF BPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4BPU ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4bpu = "<copy4xi>copy2copybuffertoimage4bpu</copy4xi>"
#       channel2copybuffertoimagepublish4bpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4bpu.send(message2copybuffertoimagesend4bpu)
#       print("(Step 84B/700) OF BPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4BPU: ", channel2copybuffertoimagepublish4bpu, " MESSAGE2SEND4BPU: " , message2copybuffertoimagesend4bpu)                                                                                                
#       #  
#   except:
#        print('(Step 84Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4BPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (85) LET MANOEUVRE2COPYIMAGETOBUFFER4BPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 85A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4bpu = manoeuvre2copybuffertoimage4bpu(queue2copybuffertoimage4bpu, channel2copybuffertoimageconsume4bpu, channel2copybuffertoimagepublish4bpu)
#       print("(Step 85B/700) OF BPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", manoeuvre2copybuffertoimage4bpu)
#   except:
#       print("(Step 85Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (86) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 86A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4BPU ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4bpu = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4bpu = channel2copybuffertoimageconsume4bpu.recv()
#       print("(Step 86B/700) OF BPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4BPU: ", channel2copybuffertoimageconsume4bpu, " CONTROL & COMMAND: ", message2copyimagereceive4bpu)                                                                                                  
#       # 
#   except:
#       print('(Step 86Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4BPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (87) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 87A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copybuffertoimage4dragon.put(path2dragon)             
#       print("(Step 87B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 87Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (88) LET MANOEUVRE2COPYIMAGETOBUFFER4BPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 88A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4bpu = manoeuvre2copybuffertoimage4bpu(queue2copybuffertoimage4bpu, channel2copybuffertoimageconsume4bpu, channel2copybuffertoimagepublish4bpu)
#       print("(Step 88B/700) OF BPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", manoeuvre2copybuffertoimage4bpu)
#   except:
#       print("(Step 88Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (89) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 89A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4bpu = channel2copybuffertoimageconsume4bpu.recv()
#       print("(Step 89D/700) OF BPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4BPU: ", channel2copybuffertoimageconsume4bpu, " CONTROL & COMMAND: ", message2copyimagereceive4bpu)                                                                                                   
#   except:
#       print('(Step 89Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (90) LET MANOEUVRE2COPYIMAGETOBUFFER4BPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 90A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4bpu = manoeuvre2copybuffertoimage4bpu(queue2copybuffertoimage4bpu, channel2copybuffertoimageconsume4bpu, channel2copybuffertoimagepublish4bpu)
#       print("(Step 90B/700) OF BPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4BPU ", response2manoeuvreimagecopytobuffer4bpu)
#   except:
#       print("(Step 90Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4BPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2buffertoimage4bpu(task4copybuffertoimage2bpu=None):

    '''
    copies buffer to image for BPU
    '''
    global bpu2image4xi                                                                                             # 2D-IMAGES FOR BPU  
    global bpu2buffer4xi                                                                                            # BUFFER FOR BPU
    
    global control2copybuffertoimage4bpu                                                                            # COMMAND AND CONTROL FOR BPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2buffertoimage4bpu"
    method2print = "copy2buffertoimage4bpu"

    message2print = "{(Step 1A/33) started" + "; copy2buffertoimage4bpu-started: " + str(copy2buffertoimage4bpu)  + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                                
    # <function copy2buffertoimage4bpu at 0x0000000003869548>
    
    message2print = "{(Step 1B/33) passed" + "; task4copybuffertoimage2bpu-to-copy2buffertoimage4bpu: " + str(task4copybuffertoimage2bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE BPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if bpu2image4xi is None:
            bpu2image4xi = create2image4bpu()                                                                              # INITIALIZE IMAGE FOR BPU 
            message2print = "{(Step 1C/70) created" + "; bpu2image4xi-created-by-copy2buffertoimage4bpu to: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4bpu-skipped-by-copy2buffertoimage4bpu: " + str(copy2buffertoimage4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            bpu2image4xi = create2image4bpu()
            message2print = "{(Step 1E/70) set" + "; bpu2image4xi-set-by-copy2buffertoimage4bpu to: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            bpu2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; bpu2image4xi-yielded-by-copy2buffertoimage4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE BPU2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if bpu2buffer4xi is None:
            bpu2buffer4xi = create2buffer4bpu()                                                                     # INITIALIZE BUFFER FOR BPU 
            message2print = "{(Step 1F/70) created" + "; bpu2buffer4xi-created-by-copy2buffertoimage4bpu to: " + str(bpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4bpu-skipped-by-copy2buffertoimage4bpu: " + str(copy2buffertoimage4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            bpu2buffer4xi = create2buffer4bpu()
            message2print = "{(Step 1H/70) set" + "; bpu2buffer4xi-set-by-copy2buffertoimage4bpu to: " + str(bpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            bpu2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; bpu2buffer4xi-yielded-by-copy2buffertoimage4bpu: " + str(bpu2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASKS           
    # -----------------------------------------------------------------------------------------          
    if task4copybuffertoimage2bpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
        
        task4copybuffertoimage2bpu = {}                                                                                                                                                                                       
        
        message2print = "{(Step 1A/70) initialized" + "; task4copybuffertoimage2bpu-created-by-copy2buffertoimage4bpu to: " + str(task4copybuffertoimage2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
        
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4bpu = trace2image4bpu                                                                 # CALLBACK FUNCTION FOR BPU
        
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4bpu-created-by-copy2buffertoimage4bpu to: " + str(function2callback4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4bpu at 0x0000000004645548>
        
        code2ack  = code2ack + '1B1-'
       
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2buffertoimage4bpu"                                                     # MODULE OF BPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2buffertoimage4bpu"                                                     # FUNCTION OF BPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4bpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR BPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4bpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4bpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}',
        #  'CLASS': 'copy2buffertoimage4bpu', 
        #  'METHOD': 'copy2buffertoimage4bpu'}; time: 04:41:15 10/04/11 Pacific Daylight Time}

        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copybuffertoimage2bpu['DATA2CALLBACK'] = args2callback                                             # USER DATA PASSED TO BPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copybuffertoimage2bpu['FUNCTION2CALLBACK'] = function2callback4bpu                                 # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copybuffertoimage2bpu-created-by-copy2buffertoimage4bpu to: " + str(task4copybuffertoimage2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4bpu at 0x0000000004645548>, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4bpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2buffertoimage4bpu', 
        #  'METHOD': 'copy2buffertoimage4bpu'}}; time: 04:41:15 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4bpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2buffertoimage4bpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 
            #  'ack2return': ' 2C-2E-2F-2G-', 
            #  'nack2return': ' ', 
            #  'available2platform': 1, 
            #  'max2entries': 1}; time: 04:41:15 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4bpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copybuffertoimage2bpu['PLATFORM'] = platform2xi                                                    # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2bpu = {}
        message2print = "{(Step 3A/70) trying" + "; context2bpu-trying-by-copy2buffertoimage4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2bpu = get2context4bpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2bpu-got-by-copy2buffertoimage4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2bpu-failed-by-copy2buffertoimage4bpu: " + str(context2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
                
        task4copybuffertoimage2bpu ['CONTEXT'] = context2bpu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2bpu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2bpu-trying-by-copy2buffertoimage4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2bpu = get2bpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2bpu-got-by-copy2buffertoimage4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, ...
               
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2bpu-failed-by-copy2buffertoimage4bpu: " + str(device2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
            
        task4copybuffertoimage2bpu ['DEVICE'] = device2bpu
            
                   
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE DESTINATION OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagedestinationinfo2bpu = {}
             
        message2print = "{(Step 5A/70) trying" + "; imagedestinationinfo2bpu-trying-by-copy2buffertoimage4bpu: " + str(imagedestinationinfo2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagedestinationinfo2bpu = get2imageinfo4bpu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagedestinationinfo2bpu-got-by-copy2buffertoimage4bpu: " + str(imagedestinationinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004105358>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4bpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4bpu', 
            #  'METHOD': 'create2context4bpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4bpu at 0x00000000037FF348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}, 
            #  'PIPE': (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>), 
            #  'COUNT2REFERENCE': 1, 
            #  'FUNCTION2CALLBACK': <function trace2image4bpu at 0x0000000004645548>, 
            #  'COMMANDQUEUE': ...
            
            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagedestinationinfo2bpu-failed-by-copy2buffertoimage4bpu: " + str(imagedestinationinfo2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copybuffertoimage2bpu ['IMAGEDESTINATION'] = imagedestinationinfo2bpu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER SOURCE OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        buffersourceinfo2bpu = {}
         
        message2print = "{(Step 6A/70) trying" + "; buffersourceinfo2bpu-trying-by-copy2buffertoimage4bpu: " + str(buffersourceinfo2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            buffersourceinfo2bpu = get2bufferinfo4bpu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; buffersourceinfo2bpu-got-by-copy2buffertoimage4bpu: " + str(buffersourceinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4bpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4bpu', 
            #  'METHOD': 'create2buffer4bpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4bpu at 0x0000000004631B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000040EBD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': ...
            
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; buffersourceinfo2bpu-failed-by-copy2buffertoimage4bpu: " + str(buffersourceinfo2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copybuffertoimage2bpu ['BUFFERSOURCE'] = buffersourceinfo2bpu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY BPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copybuffertoimage4bpu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copybuffertoimage4bpu-trying-by-copy2buffertoimage4bpu: " + str(queue2copybuffertoimage4bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copybuffertoimage4bpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copybuffertoimage4bpu-got-by-copy2buffertoimage4bpu: " + str(queue2copybuffertoimage4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x000000000492B208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copybuffertoimage4bpu-failed-by-copy2buffertoimage4bpu: " + str(queue2copybuffertoimage4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copybuffertoimage2bpu ['QUEUE'] = queue2copybuffertoimage4bpu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY BPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2bpu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2bpu-trying-by-copy2buffertoimage4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2bpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2bpu-got-by-copy2buffertoimage4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 708>, <read-write PipeConnection, handle 712>) 
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2bpu-failed-by-copy2buffertoimage4bpu: " + str(pipe2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copybuffertoimage2bpu ['PIPE'] = pipe2bpu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2bpu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2bpu-trying-by-copy2buffertoimage4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2bpu = get2commandqueueinfo4bpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2bpu-got-by-copy2buffertoimage4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2bpu-failed-by-copy2buffertoimage4bpu: " + str(commandqueueinfo2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copybuffertoimage2bpu ['COMMANDQUEUE'] = commandqueueinfo2bpu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2bpu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2bpu-trying-by-copy2buffertoimage4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2bpu = get2eventinfo4bpu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2bpu-got-by-copy2buffertoimage4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2bpu-failed-by-copy2buffertoimage4bpu: " + str(eventinfo2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copybuffertoimage2bpu ['EVENT'] = eventinfo2bpu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2bpu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2bpu-trying-by-copy2buffertoimage4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2bpu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2bpu-set-by-copy2buffertoimage4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2bpu-failed-by-copy2buffertoimage4bpu: " + str(eventwaitlist2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copybuffertoimage2bpu ['EVENTWAITLIST'] = eventwaitlist2bpu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2bpu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2bpu-trying-by-copy2buffertoimage4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2bpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2bpu = eventwaitlistquantity2bpu + 1                                                                     
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2bpu-set-by-copy2buffertoimage4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2bpu-failed-by-copy2buffertoimage4bpu: " + str(eventwaitlistquantity2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copybuffertoimage2bpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2bpu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (15) ORIGIN FOR DESTINATION - REQUIRED                             
        # ---------------------------------------------------------------------------------------------              
        origin4destination2bpu = {}
        
        message2print = "{(Step 15A/70) trying" + "; origin4destination2bpu-trying-by-copy2buffertoimage4bpu: " + str(origin4destination2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4destination2bpu['X'] = 100 
            origin4destination2bpu['Y'] = 100  
            origin4destination2bpu['Z'] = 100                                                                                                                          
            message2print = "{(Step 15B/70) set" + "; origin4destination2bpu-set-by-copy2buffertoimage4bpu to: " + str(origin4destination2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 100, 'X': 100, 'Z': 100}
            code2ack  = code2ack + '15B-' 
        except:
            message2print = "{(Step 15Z/70) failed" + "; origin4destination2bpu-failed-by-copy2buffertoimage4bpu: " + str(origin4destination2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '15Z-'
    
        task4copybuffertoimage2bpu ['DESTINATIONORIGIN'] = origin4destination2bpu
                    
        
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2bpu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2bpu-trying-by-copy2buffertoimage4bpu: " + str(region4copy2bpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2bpu['X'] = 10 
            region4copy2bpu['Y'] = 10  
            region4copy2bpu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2bpu-set-by-copy2buffertoimage4bpu to: " + str(region4copy2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10}
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2bpu-failed-by-copy2buffertoimage4bpu: " + str(region4copy2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copybuffertoimage2bpu ['REGION'] = region4copy2bpu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER                   
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2bpu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2bpu-set-by-copy2buffertoimage4bpu to: " + str(offset4copy2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        
        task4copybuffertoimage2bpu ['OFFSET'] = offset4copy2bpu            
                                
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
        
        # ---------------------------------------------------------------------------------------------
        # ANALYZE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2bpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2buffertoimage4bpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:41:15 10/04/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4bpu = trace2image4bpu                                                             # CALLBACK FUNCTION FOR BPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4bpu-created-by-copy2buffertoimage4bpu to: " + str(function2callback4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2buffertoimage4bpu"                                                        # MODULE OF BPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2buffertoimage4bpu"                                                        # FUNCTION OF BPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4bpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR BPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4bpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4copybuffertoimage2bpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO BPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4copybuffertoimage2bpu['FUNCTION2CALLBACK'] = function2callback4bpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4copybuffertoimage2bpu-created-by-copy2buffertoimage4bpu to: " + str(task4copybuffertoimage2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4bpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2buffertoimage4bpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4bpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4copybuffertoimage2bpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2bpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2bpu-trying-by-copy2buffertoimage4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2bpu = get2context4bpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2bpu-got-by-copy2buffertoimage4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2bpu-failed-by-copy2buffertoimage4bpu: " + str(context2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4copybuffertoimage2bpu ['CONTEXT'] = context2bpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2bpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2bpu-trying-by-copy2buffertoimage4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2bpu = get2bpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2bpu-got-by-copy2buffertoimage4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2bpu-failed-by-copy2buffertoimage4bpu: " + str(device2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4copybuffertoimage2bpu ['DEVICE'] = device2bpu
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2bpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; destinationimageinfo2bpu-trying-by-copy2buffertoimage4bpu: " + str(destinationimageinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2bpu = get2imageinfo4bpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; destinationimageinfo2bpu-got-by-copy2buffertoimage4bpu: " + str(destinationimageinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; destinationimageinfo2bpu-failed-by-copy2buffertoimage4bpu: " + str(destinationimageinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4copybuffertoimage2bpu ['IMAGEDESTINATION'] = destinationimageinfo2bpu         
        
        if 'BUFFERSOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) SOURCE BUFFER - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourcebufferinfo2bpu = {}
             
            message2print = "{(Step 26A/70) trying" + "; sourcebufferinfo2bpu-trying-by-copy2buffertoimage4bpu: " + str(sourcebufferinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourcebufferinfo2bpu = get2bufferinfo4bpu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; sourcebufferinfo2bpu-got-by-copy2buffertoimage4bpu: " + str(sourcebufferinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; sourcebufferinfo2bpu-failed-by-copy2buffertoimage4bpu: " + str(sourcebufferinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4copybuffertoimage2bpu ['BUFFERSOURCE'] = sourcebufferinfo2bpu    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY BPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2copybuffertoimage4bpu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2copybuffertoimage4bpu-trying-by-copy2buffertoimage4bpu: " + str(queue2copybuffertoimage4bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2copybuffertoimage4bpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2copybuffertoimage4bpu-got-by-copy2buffertoimage4bpu: " + str(queue2copybuffertoimage4bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2copybuffertoimage4bpu-failed-by-copy2buffertoimage4bpu: " + str(queue2copybuffertoimage4bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4copybuffertoimage2bpu ['QUEUE'] = queue2copybuffertoimage4bpu
            
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY BPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2bpu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2bpu-trying-by-copy2buffertoimage4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2bpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2bpu-got-by-copy2buffertoimage4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2bpu-failed-by-copy2buffertoimage4bpu: " + str(pipe2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4copybuffertoimage2bpu ['PIPE'] = pipe2bpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2bpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2bpu-trying-by-copy2buffertoimage4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2bpu = get2commandqueueinfo4bpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2bpu-got-by-copy2buffertoimage4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2bpu-failed-by-copy2buffertoimage4bpu: " + str(commandqueueinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4copybuffertoimage2bpu ['COMMANDQUEUE'] = commandqueueinfo2bpu
        
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2bpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2bpu-trying-by-copy2buffertoimage4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2bpu = get2eventinfo4bpu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2bpu-got-by-copy2buffertoimage4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2bpu-failed-by-copy2buffertoimage4bpu: " + str(eventinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4copybuffertoimage2bpu ['EVENT'] = eventinfo2bpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2bpu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2bpu-trying-by-copy2buffertoimage4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2bpu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2bpu-set-by-copy2buffertoimage4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2bpu-failed-by-copy2buffertoimage4bpu: " + str(eventwaitlist2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4copybuffertoimage2bpu ['EVENTWAITLIST'] = eventwaitlist2bpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2bpu = 0
           
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2bpu-trying-by-copy2buffertoimage4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2bpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2bpu = eventwaitlistquantity2bpu + 1                                                                     
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2bpu-set-by-copy2buffertoimage4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2bpu-failed-by-copy2buffertoimage4bpu: " + str(eventwaitlistquantity2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4copybuffertoimage2bpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2bpu       
                                             
  
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2bpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; destination4origin2bpu-trying-by-copy2buffertoimage4bpu: " + str(destination4origin2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2bpu['X'] = 0 
                destination4origin2bpu['Y'] = 0  
                destination4origin2bpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; destination4origin2bpu-set-by-copy2buffertoimage4bpu to: " + str(destination4origin2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; destination4origin2bpu-failed-by-copy2buffertoimage4bpu: " + str(destination4origin2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4copybuffertoimage2bpu ['DESTINATIONORIGIN'] = destination4origin2bpu
             
     
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2bpu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2bpu-trying-by-copy2buffertoimage4bpu: " + str(region2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2bpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2bpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2bpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2bpu-set-by-copy2buffertoimage4bpu to: " + str(region2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2bpu-failed-by-copy2buffertoimage4bpu: " + str(region2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4copybuffertoimage2bpu ['REGION'] = region2bpu
        
             
        if 'OFFSET' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (37) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            offset2bpu = 0
            message2print = "{(Step 37A/70) set" + "; offset2bpu-set-by-copy2buffertoimage4bpu to: " + str(offset2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            task4copybuffertoimage2bpu ['OFFSET'] = offset2bpu        
            code2ack  = code2ack + '37A-' 
                    
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2bpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2buffertoimage4bpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:52:11 10/04/11 Pacific Daylight Time}
                


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copybuffertoimage4bpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2bpu = False
    channels2bpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2bpu-trying-by-copy2buffertoimage4bpu: " + str(channels2bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copybuffertoimage4bpu, publisherchannel2copybuffertoimage4bpu = task4copybuffertoimage2bpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copybuffertoimage4bpu-got-by-copy2buffertoimage4bpu: " + str(consumerchannel2copybuffertoimage4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 704>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copybuffertoimage4bpu-got-by-copy2buffertoimage4bpu: " + str(publisherchannel2copybuffertoimage4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 708>                      
        ack4channels2bpu = True
        
        control2copybuffertoimage4bpu['CONSUMERCHANNEL']  = consumerchannel2copybuffertoimage4bpu
        
        
        control2copybuffertoimage4bpu['PUBLISHERCHANNEL'] = publisherchannel2copybuffertoimage4bpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2bpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copybuffertoimage4bpu = False
    queue2copybuffertoimage4bpu = None
    message2print = "{(Step 41A/70) trying" + "queue2copybuffertoimage4bpu-trying-by-copy2buffertoimage4bpu: " + str(queue2copybuffertoimage4bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copybuffertoimage4bpu = task4copybuffertoimage2bpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copybuffertoimage4bpu-got-by-copy2buffertoimage4bpu: " + str(queue2copybuffertoimage4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003DF2198>
              
        ack4queue2copybuffertoimage4bpu = True
        control2copybuffertoimage4bpu['QUEUE']  = queue2copybuffertoimage4bpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copybuffertoimage4bpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4bpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4bpu-trying-by-copy2buffertoimage4bpu: " + str(dummy2image4bpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4bpu = DummyProcess4Xi(target=manoeuvre2copybuffertoimage4bpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copybuffertoimage4bpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copybuffertoimage4bpu, publisherchannel2copybuffertoimage4bpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4bpu-created-by-copy2buffertoimage4bpu: " + str(dummy2image4bpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2buffertoimage4bpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2buffertoimage4bpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
        
    message2print = "{(Step 70C/70) returning" + "; control2copybuffertoimage4bpu-returning-by-copy2buffertoimage4bpu: " + str(control2copybuffertoimage4bpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003DF2198>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 708>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 704>}; time: 04:52:11 10/04/11 Pacific Daylight Time}

        
    return control2copybuffertoimage4bpu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "read2image4bpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4BPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF BPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4BPU ', time4xi())    
#   respon4read2image = {} 
#   task4read2image = None
#   try:                               
#       response4read2image = read2image4bpu(task4read2image)                                                                                                          
#       print('(Step 16B/700) OF BPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4BPU : ', response4read2image)
#       queue2read4bpu      = response4read2image['QUEUE']
#       channel2readpublish4bpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readconsume4bpu = response4read2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4BPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4BPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF BPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4BPU ', time4xi())     
#   try:                               
#       message2readsend4bpu = "<copy4xi>copy2read4bpu</copy4xi>"
#       channel2readpublish4bpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4bpu.send(message2readsend4bpu)
#       print("(Step 17B/700) OF BPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4BPU: ", channel2readpublish4bpu, " MESSAGE2SEND4BPU: " , message2readsend4bpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF BPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4BPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4BPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi()) 
#   try:        
#       response2manoeuvre4bpu = manoeuvre2readimage4bpu(queue2read4bpu, channel2readconsume4bpu, channel2readpublish4bpu)
#       print("(Step 18D/700) OF BPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", manoeuvre2readimage4bpu)
#   except:
#       print("(Step 18Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4BPU ', time4xi())     
#   try:
#       channel2readconsume4bpu = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4bpu = channel2readconsume4bpu.recv()
#       print("(Step 19B/700) OF BPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4BPU: ", channel2readconsume4bpu, " CONTROL & COMMAND: ", message2readreceive4bpu)                                                                                                  
#       # <roger4xi>roger2read4bpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4BPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4BPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF BPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4BPU ', time4xi())     
#   try:                               
#       message2readsend4bpu = "<ready4xi>ready2read4bpu</ready4xi>"
#       channel2readpublish4bpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4bpu.send(message2readsend4bpu)
#       print("(Step 20B/700) OF BPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4BPU: ", channel2readpublish4bpu, " MESSAGE2SEND4BPU: " , message2readsend4bpu)                                                                                                
#       # <ready4xi>ready2read4bpu</ready4xi> 
#   except:
#        print('(Step 20Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4BPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4BPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi()) 
#   try:        
#       response2manoeuvre4bpu = manoeuvre2readimage4bpu(queue2read4bpu, channel2readconsume4bpu, channel2readpublish4bpu)
#       print("(Step 21B/700) OF BPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", manoeuvre2readimage4bpu)
#   except:
#       print("(Step 21Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4BPU ', time4xi())     
#   try:
#       channel2readconsume4bpu = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4bpu = channel2readconsume4bpu.recv()
#       print("(Step 22B/700) OF BPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4BPU: ", channel2readconsume4bpu, " CONTROL & COMMAND: ", message2readreceive4bpu)                                                                                                  
#       # <ready4xi>ready2read4bpu</ready4xi> 
#   except:
#       print('(Step 22Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4BPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2read4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4BPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi()) 
#   try:        
#       response2manoeuvre4bpu = manoeuvre2readimage4bpu(queue2read4bpu, channel2readconsume4bpu, channel2readpublish4bpu)
#       print("(Step 25B/700) OF BPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", manoeuvre2readimage4bpu)
#   except:
#       print("(Step 25Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2readreceive4bpu = channel2readconsume4bpu.recv()
#       print("(Step 26D/700) OF BPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4BPU: ", channel2readconsume4bpu, " CONTROL & COMMAND: ", message2readreceive4bpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4bpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4BPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi()) 
#   try:        
#       response2manoeuvre4bpu = manoeuvre2readimage4bpu(queue2read4bpu, channel2readconsume4bpu, channel2readpublish4bpu)
#       print("(Step 27B/700) OF BPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4BPU ", response2manoeuvre4bpu)
#   except:
#       print("(Step 27Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi())   
#">

# -------------------------------------------------------------------------------------------------   
def read2image4bpu(task4imageread2bpu=None):

    '''
    reads 2D images for BPU
    '''
    global bpu2image4xi                                                     # 2D-IMAGES FOR BPU  
    global control2readimage4bpu                                              # COMMAND AND CONTROL FOR BPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "read2image4bpu"
    method2print = "read2image4bpu"

    message2print = "{(Step 1A/33) started" + "; read2image4bpu-started: " + str(read2image4bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function read2image4bpu at 0x00000000039595C8> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageread2bpu-to-read2image4bpu: " + str(task4imageread2bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE BPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if bpu2image4xi is None:
            bpu2image4xi = create2image4bpu()                                                                              # INITIALIZE IMAGE FOR BPU 
            message2print = "{(Step 1C/70) created" + "; bpu2image4xi-created-by-read2image4bpu to: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4bpu-skipped-by-read2image4bpu: " + str(read2image4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                 
    except:
        try:
            bpu2image4xi = create2image4bpu()
            message2print = "{(Step 1E/70) set" + "; bpu2image4xi-set-by-read2image4bpu to: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            bpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; bpu2image4xi-yielded-by-read2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imageread2bpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imageread2bpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imageread2bpu-created-by-read2image4bpu to: " + str(task4imageread2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4bpu = trace2image4bpu                                                              # CALLBACK FUNCTION FOR BPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4bpu-created-by-read2image4bpu to: " + str(function2callback4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4bpu at 0x0000000004737548>;
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4bpu"                                                          # MODULE OF BPUIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4bpu"                                                          # FUNCTION OF BPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4bpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR BPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-read2image4bpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; read2image4bpu-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4bpu', 
            #  'METHOD': 'read2image4bpu'}; time: 05:06:38 10/04/11 Pacific Daylight Time} 

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2bpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO BPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2bpu['FUNCTION2CALLBACK'] = function2callback4bpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imageread2bpu-created-by-read2image4bpu to: " + str(task4imageread2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4bpu at 0x0000000004737548>, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; read2image4bpu-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4bpu', 
            #  'METHOD': 'read2image4bpu'}}; time: 05:06:38 10/04/11 Pacific Daylight Time}
             
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-read2image4bpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-read2image4bpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-read2image4bpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imageread2bpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2bpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2bpu-trying-by-read2image4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2bpu = get2context4bpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2bpu-got-by-read2image4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
    
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2bpu-failed-by-read2image4bpu: " + str(context2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imageread2bpu ['CONTEXT'] = context2bpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2bpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2bpu-trying-by-read2image4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2bpu = get2bpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2bpu-got-by-read2image4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 
                #  'ID2PLATFORM': '0000000001509568', 
                #  'PROFILING2QUEUE': 'ACK', 
                #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
                #  'NATIVE2WIDTHLONG4VECTOR': 2 ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2bpu-failed-by-read2image4bpu: " + str(device2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imageread2bpu ['DEVICE'] = device2bpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2bpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2bpu-trying-by-read2image4bpu: " + str(imageinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2bpu = get2imageinfo4bpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2bpu-got-by-read2image4bpu: " + str(imageinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041F9320>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2bpu-failed-by-read2image4bpu: " + str(imageinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imageread2bpu ['IMAGE'] = imageinfo2bpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY BPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2bpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2bpu-trying-by-read2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2bpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2bpu-got-by-read2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000004607160> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2bpu-failed-by-read2image4bpu: " + str(queue2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imageread2bpu ['QUEUE'] = queue2bpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY BPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2bpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2bpu-trying-by-read2image4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2bpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2bpu-got-by-read2image4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 408>, <read-write PipeConnection, handle 412>) 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2bpu-failed-by-read2image4bpu: " + str(pipe2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imageread2bpu ['PIPE'] = pipe2bpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2bpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2bpu-trying-by-read2image4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2bpu = get2commandqueueinfo4bpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2bpu-got-by-read2image4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2bpu-failed-by-read2image4bpu: " + str(commandqueueinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imageread2bpu ['COMMANDQUEUE'] = commandqueueinfo2bpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY BPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2bpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2bpu-trying-by-read2image4bpu: " + str(bufferinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2bpu = get2bufferinfo4bpu()                                                                   # SYSTEM CREATES BUFFER FOR BPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2bpu-got-by-read2image4bpu: " + str(bufferinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2bpu-failed-by-read2image4bpu: " + str(bufferinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imageread2bpu ['BUFFER'] = bufferinfo2bpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2bpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2bpu-trying-by-read2image4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2bpu = get2eventinfo4bpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2bpu-got-by-read2image4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2bpu-failed-by-read2image4bpu: " + str(eventinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imageread2bpu ['EVENT'] = eventinfo2bpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2bpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2bpu-trying-by-read2image4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2bpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2bpu-set-by-read2image4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {}
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2bpu-failed-by-read2image4bpu: " + str(eventwaitlist2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imageread2bpu ['EVENTWAITLIST'] = eventwaitlist2bpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2bpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2bpu-trying-by-read2image4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2bpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2bpu = eventwaitlistquantity2bpu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2bpu-set-by-read2image4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2bpu-failed-by-read2image4bpu: " + str(eventwaitlistquantity2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imageread2bpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2bpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2bpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2bpu-trying-by-read2image4bpu: " + str(blocking2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2bpu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2bpu-set-by-read2image4bpu to: " + str(blocking2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2bpu-failed-by-read2image4bpu: " + str(blocking2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imageread2bpu ['BLOCKING'] = blocking2bpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2bpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2bpu-trying-by-read2image4bpu: " + str(origin2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2bpu['X'] = 0 
                origin2bpu['Y'] = 0  
                origin2bpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2bpu-set-by-read2image4bpu to: " + str(origin2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2bpu-failed-by-read2image4bpu: " + str(origin2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imageread2bpu ['ORIGIN'] = origin2bpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2bpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2bpu-trying-by-read2image4bpu: " + str(region2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2bpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2bpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2bpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2bpu-set-by-read2image4bpu to: " + str(region2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2bpu-failed-by-read2image4bpu: " + str(region2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imageread2bpu ['REGION'] = region2bpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2bpu = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2bpu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2bpu-trying-by-read2image4bpu: " + str(rowpitch2bpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2bpu = int(width2bpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2bpu-set-by-read2image4bpu to: " + str(rowpitch2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2bpu-failed-by-read2image4bpu: " + str(rowpitch2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imageread2bpu ['ROWPITCH'] = rowpitch2bpu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2bpu = None
            height2bpu = 800
            size2bpu = int(width2bpu * height2bpu)                                                                  # CALCULATED MEMORY
            if size2bpu == 0:
                size2bpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2bpu-trying-by-read2image4bpu: " + str(pointer2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2bpu = Arena4Xi(size2bpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2bpu-got-by-read2image4bpu: " + str(arena2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x0000000004449EF0>
                
                size2arena = arena2bpu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-read2buffer4bpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000
                
                buffer2arena = arena2bpu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-read2buffer4bpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2bpu:
                    size2bpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2bpu-set-by-read2buffer4bpu to: " + str(size2bpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2bpu = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2bpu-set-by-read2buffer4bpu to: " + str(pointer2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2bpu-failed-by-read2image4bpu: " + str(pointer2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imageread2bpu ['POINTER'] = pointer2bpu                            
            task4imageread2bpu ['SIZE']    = size2bpu                                                               # AUTOMATIC BY BPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2bpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2bpu-trying-by-read2image4bpu: " + str(slicepitch2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2bpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2bpu-set-by-read2image4bpu to: " + str(slicepitch2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2bpu-failed-by-read2image4bpu: " + str(slicepitch2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imageread2bpu ['SLICEPITCH'] = slicepitch2bpu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2bpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-read2image4bpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4bpu = trace2image4bpu                                                              # CALLBACK FUNCTION FOR BPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4bpu-created-by-read2image4bpu to: " + str(function2callback4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4bpu"                                                          # MODULE OF BPUIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4bpu"                                                          # FUNCTION OF BPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4bpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR BPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-read2image4bpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2bpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO BPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2bpu['FUNCTION2CALLBACK'] = function2callback4bpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imageread2bpu-created-by-read2image4bpu to: " + str(task4imageread2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-read2image4bpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-read2image4bpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-read2image4bpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imageread2bpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2bpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2bpu-trying-by-read2image4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2bpu = get2context4bpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2bpu-got-by-read2image4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2bpu-failed-by-read2image4bpu: " + str(context2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imageread2bpu ['CONTEXT'] = context2bpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2bpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2bpu-trying-by-read2image4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2bpu = get2bpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2bpu-got-by-read2image4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2bpu-failed-by-read2image4bpu: " + str(device2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imageread2bpu ['DEVICE'] = device2bpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2bpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2bpu-trying-by-read2image4bpu: " + str(imageinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2bpu = get2imageinfo4bpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2bpu-got-by-read2image4bpu: " + str(imageinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2bpu-failed-by-read2image4bpu: " + str(imageinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imageread2bpu ['IMAGE'] = imageinfo2bpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY BPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2bpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2bpu-trying-by-read2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2bpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2bpu-got-by-read2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2bpu-failed-by-read2image4bpu: " + str(queue2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imageread2bpu ['QUEUE'] = queue2bpu
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY BPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2bpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2bpu-trying-by-read2image4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2bpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2bpu-got-by-read2image4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2bpu-failed-by-read2image4bpu: " + str(pipe2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imageread2bpu ['PIPE'] = pipe2bpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2bpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2bpu-trying-by-read2image4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2bpu = get2commandqueueinfo4bpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2bpu-got-by-read2image4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2bpu-failed-by-read2image4bpu: " + str(commandqueueinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imageread2bpu ['COMMANDQUEUE'] = commandqueueinfo2bpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY BPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2bpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2bpu-trying-by-read2image4bpu: " + str(bufferinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2bpu = get2bufferinfo4bpu()                                                                   # SYSTEM CREATES BUFFER FOR BPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2bpu-got-by-read2image4bpu: " + str(bufferinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2bpu-failed-by-read2image4bpu: " + str(bufferinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imageread2bpu ['BUFFER'] = bufferinfo2bpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2bpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2bpu-trying-by-read2image4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2bpu = get2eventinfo4bpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2bpu-got-by-read2image4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2bpu-failed-by-read2image4bpu: " + str(eventinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imageread2bpu ['EVENT'] = eventinfo2bpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2bpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2bpu-trying-by-read2image4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2bpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2bpu-set-by-read2image4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2bpu-failed-by-read2image4bpu: " + str(eventwaitlist2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imageread2bpu ['EVENTWAITLIST'] = eventwaitlist2bpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2bpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2bpu-trying-by-read2image4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2bpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2bpu = eventwaitlistquantity2bpu + 1                                                                      
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2bpu-set-by-read2image4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2bpu-failed-by-read2image4bpu: " + str(eventwaitlistquantity2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imageread2bpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2bpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2bpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2bpu-trying-by-read2image4bpu: " + str(blocking2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2bpu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2bpu-set-by-read2image4bpu to: " + str(blocking2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2bpu-failed-by-read2image4bpu: " + str(blocking2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imageread2bpu ['BLOCKING'] = blocking2bpu             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2bpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2bpu-trying-by-read2image4bpu: " + str(origin2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2bpu['X'] = 0 
                origin2bpu['Y'] = 0  
                origin2bpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2bpu-set-by-read2image4bpu to: " + str(origin2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2bpu-failed-by-read2image4bpu: " + str(origin2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imageread2bpu ['ORIGIN'] = origin2bpu
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2bpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2bpu-trying-by-read2image4bpu: " + str(region2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2bpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2bpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2bpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2bpu-set-by-read2image4bpu to: " + str(region2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2bpu-failed-by-read2image4bpu: " + str(region2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imageread2bpu ['REGION'] = region2bpu
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2bpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2bpu-trying-by-read2image4bpu: " + str(rowpitch2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2bpu = int(width2bpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2bpu-set-by-read2image4bpu to: " + str(rowpitch2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2bpu-failed-by-read2image4bpu: " + str(rowpitch2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imageread2bpu ['ROWPITCH'] = rowpitch2bpu     
             
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2bpu = None
            size2bpu = int(width2bpu * height2bpu)                                                                  # CALCULATED MEMORY
            if size2bpu == 0:
                size2bpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2bpu-trying-by-read2image4bpu: " + str(pointer2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2bpu = Arena4Xi(size2bpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2bpu-got-by-read2image4bpu: " + str(arena2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2bpu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-read2buffer4bpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2bpu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-read2buffer4bpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2bpu:
                    size2bpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2bpu-set-by-read2buffer4bpu to: " + str(size2bpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2bpu = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2bpu-set-by-read2buffer4bpu to: " + str(pointer2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2bpu-failed-by-read2image4bpu: " + str(pointer2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imageread2bpu ['POINTER'] = pointer2bpu                            
            task4imageread2bpu ['SIZE']    = size2bpu                                                               # AUTOMATIC BY BPUXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2bpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2bpu-trying-by-read2image4bpu: " + str(slicepitch2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2bpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2bpu-set-by-read2image4bpu to: " + str(slicepitch2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2bpu-failed-by-read2image4bpu: " + str(slicepitch2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imageread2bpu ['SLICEPITCH'] = slicepitch2bpu
                              
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2bpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-read2image4bpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}                
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imageread2bpu-yielded-by-read2image4bpu: " + str(task4imageread2bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2readimage4bpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2bpu = False
    channels2bpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2bpu-trying-by-read2image4bpu: " + str(channels2bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>) 
        consumerchannel2bpu, publisherchannel2bpu = task4imageread2bpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2bpu-got-by-read2image4bpu: " + str(consumerchannel2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2bpu-got-by-read2image4bpu: " + str(publisherchannel2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #                      
        ack4channels2bpu = True
        
        control2readimage4bpu['CONSUMERCHANNEL']  = consumerchannel2bpu
        
        
        control2readimage4bpu['PUBLISHERCHANNEL'] = publisherchannel2bpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2bpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2bpu = False
    queue2bpu = None
    message2print = "{(Step 41A/70) trying" + "; queue2bpu-trying-by-read2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2bpu = task4imageread2bpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2bpu-got-by-read2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
              
        ack4queue2bpu = True
        control2readimage4bpu['QUEUE']  = queue2bpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2bpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4bpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4bpu-trying-by-read2image4bpu: " + str(dummy2image4bpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4bpu = DummyProcess4Xi(target=manoeuvre2readimage4bpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2bpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2bpu, publisherchannel2bpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4bpu-created-by-read2image4bpu: " + str(dummy2image4bpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'

    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-read2image4bpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-read2image4bpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #   
        
    message2print = "{(Step 70C/70) returning" + "; control2readimage4bpu-returning-by-read2image4bpu: " + str(control2readimage4bpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003FE3B70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 416>, 
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 412>}
        
    return control2readimage4bpu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "map2image4bpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # ESTABLISH CONTROL & COMMAND                 
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 92A/700) OF BPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4BPU ', time4xi())           
#                    
#   response4map2image = {} 
#   task4map2image = None                                                                                  # WE ONLY WANT TO ESTABLISH CONTROL AND CONTROL FIRST
#
#   try:                               
#       response4map2image = map2image4bpu(task4map2image)                                                                                                          
#       print('(Step 92B/700) OF BPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH MAP2IMAGE4BPU : ', response4map2image)
#    
#       queue4map2image4bpu      = response4map2image['QUEUE']
#       channel4map2imagepublish4bpu = response4map2image['PUBLISHERCHANNEL']
#       channel4map2imageconsume4bpu = response4map2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 92Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4BPU ', time4xi()) 
#        
#   # -----------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGE4BPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------------------      
#   print('(Step 93A/700) OF BPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4BPU ', time4xi())      
#   try:                               
#       message2send4bpu = "<copy4xi>copy2mapimage4bpu</copy4xi>"
#       channel4map2imagepublish4bpu.send(message2send4bpu)
#       print("(Step 93B/700) OF BPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4BPU: ", channel4map2imagepublish4bpu, " MESSAGE2SEND4BPU: " , message2send4bpu)                                                                                                
#   except:
#       print('(Step 93Z1/700) OF BPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4BPU ', time4xi())   
# 
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4BPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4bpu is True:
#       print("(Step 94A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4BPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4bpu(queue4map2image4bpu, channel4map2imageconsume4bpu, channel4map2imagepublish4bpu)
#           print("(Step 94B/700) OF BPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4BPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#        except:
#           print("(Step 94Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4BPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#
#   # ---------------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND ROGER VIA PIPE                  
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 95A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4BPU ', time4xi())      
#   try:
#       message2receive4bpu = channel4map2imageconsume4bpu.recv()
#       print("(Step 95B/700) OF BPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4BPU: ", channel4map2imageconsume4bpu, " CONTROL & COMMAND: ", message2receive4bpu)                                                                                                  
#   except:
#       print('(Step 95Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4BPU ', time4xi())          
#   
#   # ------------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGE4BPU TO RECEIVE                
#   # ------------------------------------------------------------------------------------------------------------      
#   print('(Step 96A/700) OF BPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4BPU ', time4xi())     
#   try:                               
#       message2send4bpu = "<ready4xi>ready2mapimage4bpu</ready4xi>"
#       channel4map2imagepublish4bpu.send(message2send4bpu)
#       print("(Step 96B/700) OF BPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4BPU: ", channel4map2imagepublish4bpu, " MESSAGE2SEND4BPU: " , message2send4bpu)                                                                                                
#   except:
#       print('(Step 84Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4BPU ', time4xi())                                                
# 
#   # -------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4BPU  - LET IT RECEIVE READY AND SEND READY                
#   # -------------------------------------------------------------------------------------------------------       
#   if ack2simulate4bpu is True:
#       print("(Step 97A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4BPU ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4bpu(queue4map2image4bpu, channel4map2imageconsume4bpu, channel4map2imagepublish4bpu)
#           print("(Step 97B/700) OF BPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4BPU ", manoeuvre2copyimage4bpu)
#       except:
#           print("(Step 97Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4BPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#            
#    
#   # ---------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND READY VIA PIPE                
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 98A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4BPU ', time4xi())     
#   try:
#       message2receive4bpu = channel4map2imageconsume4bpu.recv()
#       print("(Step 98B/700) OF BPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4BPU: ", channel4map2imageconsume4bpu, " CONTROL & COMMAND: ", message2receive4bpu)                                                                                                  
#   except:
#       print('(Step 86Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4BPU ', time4xi())          
#                         
#   # -----------------------------------------------------------------------------------------
#   # IMAGE BPUXI COPY OPERATIONS 
#   # -----------------------------------------------------------------------------------------         
#   print('(Step 99A/700) OF BPU2IMAGE4XI5TEST - TRYING FILE OPERATIONS WITH IMAGE BPUXI ', time4xi())       
#   try:        
#       path2dragon = '..//..//..//..//..//web//images//png//dragonxi.png'
#       file2dragon = open(path2dragon)
#       print("(Step 99B/700) OF BPU2IMAGE4XI5TEST - FILENO: ", file2bpu.fileno( ))  
#       print("(Step 99C/700) OF BPU2IMAGE4XI5TEST - MODE: ", file2bpu.mode)
#       print("(Step 99D/700) OF BPU2IMAGE4XI5TEST - NAME: ", file2bpu.name)
#       offset2bpu = 10
#       file2bpu.seek(offset2bpu)
#       print("(Step 99E/700) OF BPU2IMAGE4XI5TEST - PERFORMED SEEK WITH OFFSET: ", offset2bpu)
#       file2bpu.close()
#       print("(Step 99F/700) OF DRAGON2IMAGE4XI5TEST - CLOSED IMAGE FILE: ", path2dragon)  
#   except:
#       print("(Step 87Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM FILE OPERATIONS WITH IMAGE: ", path2dragon)        
# 
#   # -----------------------------------------------------------------------------------------
#   # PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 100A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue4map2image4dragon.put(path2dragon)             
#       print("(Step 100B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#   except:
#       print("(Step 88Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   
#   # -------------------------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4BPU  - STATE III - GETS FROM QUEUE AND SENDS EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4bpu is True:
#       print("(Step 101A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4BPU ", time4xi()) 
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4bpu(queue4map2image4bpu, channel4map2imageconsume4bpu, channel4map2imagepublish4bpu)
#           print("(Step 101B/700) OF BPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4BPU ", manoeuvre2copyimage4bpu)
#       except:
#           print("(Step 101Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4BPU ", time4xi())     
#   else:
#       pass # trust activated thread                     
#                 
#                                  
#   # ----------------------------------------------------------------------------------------------------
#   # RECEIVE ACK OR NACK ABOUT IMAGE OPERATION CONTROL & COMMAND VIA PIPE                
#   # ----------------------------------------------------------------------------------------------------          
#   print('(Step 102A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2receive4bpu = channel4map2imageconsume4bpu.recv()
#       print("(Step 102B/700) OF BPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4BPU: ", channel4map2imageconsume4bpu, " CONTROL & COMMAND: ", message2receive4bpu)                                                                                                   
#   except:
#       print('(Step 90Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#
#   
#   # ----------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4BPU  - LET MANOEUVRE2COPYIMAGE4BPU TO GO TO STATE 0               
#   # ----------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4bpu is True:
#       print("(Step 103A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4BPU ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4bpu(queue4map2image4bpu, channel4map2imageconsume4bpu, channel4map2imagepublish4bpu)
#           print("(Step 103B/700) OF BPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4BPU ", response2manoeuvre4copy)
#       except:
#           print("(Step 103Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4BPU ", time4xi())     
#   else:
#       pass # trust activated thread  
#                                           
#">

# -------------------------------------------------------------------------------------------------   
def map2image4bpu(task4imagemap2bpu=None):

    '''
    maps 2D images for BPU
    '''
    global bpu2image4xi                                                                                             # 2D-IMAGES FOR BPU  
    global control2mapimage4bpu                                                                                     # COMMAND AND CONTROL FOR BPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "map2image4bpu"
    method2print = "map2image4bpu"

    message2print = "{(Step 1A/33) started" + "; map2image4bpu-started: " + str(map2image4bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 1B/33) passed" + "; task4imagemap2bpu-to-map2image4bpu: " + str(task4imagemap2bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE BPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if bpu2image4xi is None:
            bpu2image4xi = create2image4bpu()                                                                              # INITIALIZE IMAGE FOR BPU 
            message2print = "{(Step 1C/70) created" + "; bpu2image4xi-created-by-map2image4bpu to: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4bpu-skipped-by-map2image4bpu: " + str(map2image4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                
    except:
        try:
            bpu2image4xi = create2image4bpu()
            message2print = "{(Step 1E/70) set" + "; bpu2image4xi-set-by-map2image4bpu to: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            bpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; bpu2image4xi-yielded-by-map2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagemap2bpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR MAP - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagemap2bpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagemap2bpu-created-by-map2image4bpu to: " + str(task4imagemap2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4bpu = trace2image4bpu                                                              # CALLBACK FUNCTION FOR BPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4bpu-created-by-map2image4bpu to: " + str(function2callback4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4bpu"                                                          # MODULE OF BPUIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4bpu"                                                          # FUNCTION OF BPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4bpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR BPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-map2image4bpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2bpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO BPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2bpu['FUNCTION2CALLBACK'] = function2callback4bpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagemap2bpu-created-by-map2image4bpu to: " + str(task4imagemap2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-map2image4bpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-map2image4bpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-map2image4bpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagemap2bpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2bpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2bpu-trying-by-map2image4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2bpu = get2context4bpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2bpu-got-by-map2image4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2bpu-failed-by-map2image4bpu: " + str(context2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagemap2bpu ['CONTEXT'] = context2bpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2bpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2bpu-trying-by-map2image4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2bpu = get2bpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2bpu-got-by-map2image4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2bpu-failed-by-map2image4bpu: " + str(device2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagemap2bpu ['DEVICE'] = device2bpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2bpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2bpu-trying-by-map2image4bpu: " + str(imageinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2bpu = get2imageinfo4bpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2bpu-got-by-map2image4bpu: " + str(imageinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2bpu-failed-by-map2image4bpu: " + str(imageinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagemap2bpu ['IMAGE'] = imageinfo2bpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY BPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2bpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2bpu-trying-by-map2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2bpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2bpu-got-by-map2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2bpu-failed-by-map2image4bpu: " + str(queue2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagemap2bpu ['QUEUE'] = queue2bpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY BPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2bpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2bpu-trying-by-map2image4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2bpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2bpu-got-by-map2image4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2bpu-failed-by-map2image4bpu: " + str(pipe2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagemap2bpu ['PIPE'] = pipe2bpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2bpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2bpu-trying-by-map2image4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2bpu = get2commandqueueinfo4bpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2bpu-got-by-map2image4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2bpu-failed-by-map2image4bpu: " + str(commandqueueinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagemap2bpu ['COMMANDQUEUE'] = commandqueueinfo2bpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY BPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2bpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2bpu-trying-by-map2image4bpu: " + str(bufferinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2bpu = get2bufferinfo4bpu()                                                                   # SYSTEM CREATES BUFFER FOR BPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2bpu-got-by-map2image4bpu: " + str(bufferinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2bpu-failed-by-map2image4bpu: " + str(bufferinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagemap2bpu ['BUFFER'] = bufferinfo2bpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2bpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2bpu-trying-by-map2image4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2bpu = get2eventinfo4bpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2bpu-got-by-map2image4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2bpu-failed-by-map2image4bpu: " + str(eventinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagemap2bpu ['EVENT'] = eventinfo2bpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2bpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2bpu-trying-by-map2image4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2bpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2bpu-set-by-map2image4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2bpu-failed-by-map2image4bpu: " + str(eventwaitlist2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagemap2bpu ['EVENTWAITLIST'] = eventwaitlist2bpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2bpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2bpu-trying-by-map2image4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2bpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2bpu = eventwaitlistquantity2bpu + 1                                                                      
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2bpu-set-by-map2image4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2bpu-failed-by-map2image4bpu: " + str(eventwaitlistquantity2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagemap2bpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2bpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2bpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2bpu-trying-by-map2image4bpu: " + str(blocking2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2bpu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2bpu-set-by-map2image4bpu to: " + str(blocking2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2bpu-failed-by-map2image4bpu: " + str(blocking2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagemap2bpu ['BLOCKING'] = blocking2bpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2bpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2bpu-trying-by-map2image4bpu: " + str(origin2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2bpu['X'] = 0 
                origin2bpu['Y'] = 0  
                origin2bpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2bpu-set-by-map2image4bpu to: " + str(origin2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2bpu-failed-by-map2image4bpu: " + str(origin2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagemap2bpu ['ORIGIN'] = origin2bpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2bpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2bpu-trying-by-map2image4bpu: " + str(region2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2bpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2bpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2bpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2bpu-set-by-map2image4bpu to: " + str(region2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2bpu-failed-by-map2image4bpu: " + str(region2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagemap2bpu ['REGION'] = region2bpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2bpu = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2bpu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2bpu-trying-by-map2image4bpu: " + str(rowpitch2bpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2bpu = int(width2bpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2bpu-set-by-map2image4bpu to: " + str(rowpitch2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2bpu-failed-by-map2image4bpu: " + str(rowpitch2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagemap2bpu ['ROWPITCH'] = rowpitch2bpu     
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2bpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2bpu-trying-by-map2image4bpu: " + str(slicepitch2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2bpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2bpu-set-by-map2image4bpu to: " + str(slicepitch2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2bpu-failed-by-map2image4bpu: " + str(slicepitch2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagemap2bpu ['SLICEPITCH'] = slicepitch2bpu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) FLAGS - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            flags2bpu = 'CL_MAP_READ'                                                                
     
            message2print = "{(Step 19A/70) trying" + "; flags2bpu-trying-by-map2image4bpu: " + str(flags2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2bpu = 'CL_MAP_WRITE'  # THE REGION SPECIFIED BY (ORIGIN, REGION) IN THE IMAGE OBJECT IS BEING MAPPED FOR WRITING                                                                                                                     
                message2print = "{(Step 19B/70) set" + "; flags2bpu-set-by-map2image4bpu to: " + str(flags2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '19B-' 
            except:
                message2print = "{(Step 19Z/70) failed" + "; flags2bpu-failed-by-map2image4bpu: " + str(flags2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '19Z-'
        
            task4imagemap2bpu ['FLAGS'] = flags2bpu 
                                  
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2bpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-map2image4bpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}
 

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4bpu = trace2image4bpu                                                              # CALLBACK FUNCTION FOR BPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4bpu-created-by-map2image4bpu to: " + str(function2callback4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4bpu"                                                          # MODULE OF BPUIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4bpu"                                                          # FUNCTION OF BPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4bpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR BPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-map2image4bpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2bpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO BPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2bpu['FUNCTION2CALLBACK'] = function2callback4bpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagemap2bpu-created-by-map2image4bpu to: " + str(task4imagemap2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-map2image4bpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-map2image4bpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-map2image4bpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagemap2bpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2bpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2bpu-trying-by-map2image4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2bpu = get2context4bpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2bpu-got-by-map2image4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2bpu-failed-by-map2image4bpu: " + str(context2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagemap2bpu ['CONTEXT'] = context2bpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2bpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2bpu-trying-by-map2image4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2bpu = get2bpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2bpu-got-by-map2image4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2bpu-failed-by-map2image4bpu: " + str(device2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagemap2bpu ['DEVICE'] = device2bpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2bpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2bpu-trying-by-map2image4bpu: " + str(imageinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2bpu = get2imageinfo4bpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2bpu-got-by-map2image4bpu: " + str(imageinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2bpu-failed-by-map2image4bpu: " + str(imageinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagemap2bpu ['IMAGE'] = imageinfo2bpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY BPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2bpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2bpu-trying-by-map2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2bpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2bpu-got-by-map2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2bpu-failed-by-map2image4bpu: " + str(queue2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagemap2bpu ['QUEUE'] = queue2bpu
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY BPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2bpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2bpu-trying-by-map2image4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2bpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2bpu-got-by-map2image4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2bpu-failed-by-map2image4bpu: " + str(pipe2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagemap2bpu ['PIPE'] = pipe2bpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2bpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2bpu-trying-by-map2image4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2bpu = get2commandqueueinfo4bpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2bpu-got-by-map2image4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2bpu-failed-by-map2image4bpu: " + str(commandqueueinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagemap2bpu ['COMMANDQUEUE'] = commandqueueinfo2bpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY BPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2bpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2bpu-trying-by-map2image4bpu: " + str(bufferinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2bpu = get2bufferinfo4bpu()                                                                   # SYSTEM CREATES BUFFER FOR BPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2bpu-got-by-map2image4bpu: " + str(bufferinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2bpu-failed-by-map2image4bpu: " + str(bufferinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagemap2bpu ['BUFFER'] = bufferinfo2bpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2bpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2bpu-trying-by-map2image4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2bpu = get2eventinfo4bpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2bpu-got-by-map2image4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2bpu-failed-by-map2image4bpu: " + str(eventinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagemap2bpu ['EVENT'] = eventinfo2bpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2bpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2bpu-trying-by-map2image4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2bpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2bpu-set-by-map2image4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2bpu-failed-by-map2image4bpu: " + str(eventwaitlist2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagemap2bpu ['EVENTWAITLIST'] = eventwaitlist2bpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2bpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2bpu-trying-by-map2image4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2bpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2bpu = eventwaitlistquantity2bpu + 1                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2bpu-set-by-map2image4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2bpu-failed-by-map2image4bpu: " + str(eventwaitlistquantity2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagemap2bpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2bpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2bpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2bpu-trying-by-map2image4bpu: " + str(blocking2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2bpu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2bpu-set-by-map2image4bpu to: " + str(blocking2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2bpu-failed-by-map2image4bpu: " + str(blocking2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagemap2bpu ['BLOCKING'] = blocking2bpu             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2bpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2bpu-trying-by-map2image4bpu: " + str(origin2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2bpu['X'] = 0 
                origin2bpu['Y'] = 0  
                origin2bpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2bpu-set-by-map2image4bpu to: " + str(origin2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2bpu-failed-by-map2image4bpu: " + str(origin2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagemap2bpu ['ORIGIN'] = origin2bpu
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2bpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2bpu-trying-by-map2image4bpu: " + str(region2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2bpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2bpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2bpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2bpu-set-by-map2image4bpu to: " + str(region2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2bpu-failed-by-map2image4bpu: " + str(region2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagemap2bpu ['REGION'] = region2bpu
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2bpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2bpu-trying-by-map2image4bpu: " + str(rowpitch2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2bpu = int(width2bpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2bpu-set-by-map2image4bpu to: " + str(rowpitch2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2bpu-failed-by-map2image4bpu: " + str(rowpitch2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagemap2bpu ['ROWPITCH'] = rowpitch2bpu     
   
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2bpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2bpu-trying-by-map2image4bpu: " + str(slicepitch2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2bpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2bpu-set-by-map2image4bpu to: " + str(slicepitch2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2bpu-failed-by-map2image4bpu: " + str(slicepitch2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagemap2bpu ['SLICEPITCH'] = slicepitch2bpu
   
        if 'FLAGS' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (39) FLAGS - REQUIRED                   
            # --------------------------------------------------------------------------------------------------------             
            flags2bpu = 'CL_MAP_READ'                                                                                                             
           
            message2print = "{(Step 39A/70) trying" + "; flags2bpu-trying-by-map2image4bpu: " + str(flags2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2bpu = 'CL_MAP_WRITE'                                                          
                message2print = "{(Step 39B/70) set" + "; flags2bpu-set-by-map2image4bpu to: " + str(flags2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '39B-'                
                 
            except:
                message2print = "{(Step 39Z/70) failed" + "; flags2bpu-failed-by-map2image4bpu: " + str(flags2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '39Z-'
            
            task4imagemap2bpu ['FLAGS'] = flags2bpu
                                          
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2bpu.keys()
        message2print = "{(Step 40A/70) set" + "; keys2task-set-by-map2image4bpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}

                       
    except: 
        message2print = "{(Step 40Z/70) yielded" + "; task4imagemap2bpu-yielded-by-map2image4bpu: " + str(task4imagemap2bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '40Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2mapimage4bpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2bpu = False
    channels2bpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2bpu-trying-by-map2image4bpu: " + str(channels2bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2bpu, publisherchannel2bpu = task4imagemap2bpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2bpu-got-by-map2image4bpu: " + str(consumerchannel2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 748>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2bpu-got-by-map2image4bpu: " + str(publisherchannel2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 752>                     
        ack4channels2bpu = True
        
        control2mapimage4bpu['CONSUMERCHANNEL']  = consumerchannel2bpu
        
        
        control2mapimage4bpu['PUBLISHERCHANNEL'] = publisherchannel2bpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2bpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2bpu = False
    queue2bpu = None
    message2print = "{(Step 41A/70) trying" + "queue2bpu-trying-by-map2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2bpu = task4imagemap2bpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2bpu-got-by-map2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004B3FB70> 
              
        ack4queue2bpu = True
        control2mapimage4bpu['QUEUE']  = queue2bpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2bpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4bpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4bpu-trying-by-map2image4bpu: " + str(dummy2image4bpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4bpu = DummyProcess4Xi(target=manoeuvre2mapimage4bpu,                                            # in this module to handle queue and pipe channels
        #                                   args =(queue2bpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2bpu, publisherchannel2bpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4bpu-created-by-map2image4bpu: " + str(dummy2image4bpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-map2image4bpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-map2image4bpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-18B-19B-40D-41C- 
        
    message2print = "{(Step 70C/70) returning" + "; control2mapimage4bpu-returning-by-map2image4bpu: " + str(control2mapimage4bpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004B3FB70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 752>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 748>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2mapimage4bpu
                    
#</function4xi> 

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimage4bpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4BPU - LET IT TO RECEIVE COPY MESSAGE AND TO SEND ROGER MESSAGE VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4bpu is True:
#       print("(Step 42A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4BPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimage4bpu(queue2copy4bpu, channel2copyconsume4bpu, channel2copypublish4bpu)
#           print("(Step 42B/700) OF BPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4BPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4BPU': '<roger4xi>roger2copy4bpu</roger4xi>', 'ACK2INFO4BPU': False, 'MESSAGE2GET4BPU': '<default4xi>default2copy4bpu</default4xi>', 
#           #  'MESSAGE2RECEIVE4BPU': '<copy4xi>copy2copy4bpu</copy4xi>', 
#           'CODE2ACK4BPU': ' 0D-1C-', 'MESSAGE2PUT4BPU': '<default4xi>default2copy4bpu</default4xi>', 'CODE2NACK4BPU': ' ', 'STATE2MANOEUVRE4BPU': 1}
#        except:
#           print("(Step 42Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4BPU ", time4xi())     
#   else:
#       pass # trust activated thread
#">            

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimage4bpu(queue2copymanoeuvre4bpu, consumerchannel2copymanoeuvre4bpu, publisherchannel2copymanoeuvre4bpu):
    '''
    manoeuvres copy control and command
    '''
    global state2copyimage4bpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagestate4bpu                                                                                        # MAX FOR STATE
    global ack2copyimageinfo4bpu                                                                                         # ACK FOR IMAGE INFO
    global message2copyimageget4bpu                                 
    global message2copyimageput4bpu
    global message2copyimagereceive4bpu
    global message2copyimagesend4bpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimage4bpu" 
    method2print = "manoeuvre2copyimage4bpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimage4bpu-started: " + str(manoeuvre2copyimage4bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2copyimage4bpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2copymanoeuvre4bpu-to-manoeuvre2copyimage4bpu: " + str(queue2copymanoeuvre4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copymanoeuvre4bpu-to-manoeuvre2copyimage4bpu: " + str(consumerchannel2copymanoeuvre4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copymanoeuvre4bpu-to-manoeuvre2copyimage4bpu: " + str(publisherchannel2copymanoeuvre4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copy = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimage4bpu-try-by-manoeuvre2copyimage4bpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimage4bpu is None:
            state2copyimage4bpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimage4bpu-initialized-by-manoeuvre2copyimage4bpu to: " + str(state2copyimage4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimage4bpu-used-by-manoeuvre2copyimage4bpu: " + str(state2copyimage4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimage4bpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimage4bpu-initialized-by-manoeuvre2copyimage4bpu to: " + str(state2copyimage4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4bpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimageinfo4bpu-initialized-by-manoeuvre2copyimage4bpu to: " + str(ack2copyimageinfo4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4bpu     = '<default4xi>default2copy4bpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimageget4bpu-initialized-by-manoeuvre2copyimage4bpu to: " + str(message2copyimageget4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4bpu</default4xi>
        message2copyimageput4bpu     = '<default4xi>default2copy4bpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimageput4bpu-initialized-by-manoeuvre2copyimage4bpu to: " + str(message2copyimageput4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4bpu</default4xi>
        message2copyimagereceive4bpu = '<default4xi>default2copy4bpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagereceive4bpu-initialized-by-manoeuvre2copyimage4bpu to: " + str(message2copyimagereceive4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4bpu</default4xi>
        message2copyimagesend4bpu    = '<default4xi>default2copy4bpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagesend4bpu-initialized-by-manoeuvre2copyimage4bpu to: " + str(message2copyimagesend4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4bpu</default4xi>
        max2copyimagestate4bpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagestate4bpu-initialized-by-manoeuvre2copyimage4bpu to: " + str( max2copyimagestate4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimage4bpu = state2copyimage4bpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimage4bpu-added-by-manoeuvre2copyimage4bpu: " + str(state2copyimage4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2copyimage4bpu > max2copyimagestate4bpu:
        state2copyimage4bpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimage4bpu-set-by-manoeuvre2copyimage4bpu to: " + str(state2copyimage4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4bpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimageinfo4bpu-set-by-manoeuvre2copyimage4bpu to: " + str(ack2copyimageinfo4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4bpu     = '<default4xi>default2copy4bpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimageget4bpu-set-by-manoeuvre2copyimage4bpu to: " + str(message2copyimageget4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4bpu</default4xi>
        message2copyimageput4bpu     = '<default4xi>default2copy4bpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimageput4bpu-set-by-manoeuvre2copyimage4bpu to: " + str(message2copyimageput4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4bpu</default4xi>
        message2copyimagereceive4bpu = '<default4xi>default2copy4bpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagereceive4bpu-set-by-manoeuvre2copyimage4bpu to: " + str(message2copyimagereceive4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4bpu</default4xi>
        message2copyimagesend4bpu    = '<default4xi>default2copy4bpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagesend4bpu-set-by-manoeuvre2copyimage4bpu to: " + str(message2copyimagesend4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4bpu</default4xi>
        max2copyimagestate4bpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagestate4bpu-set-by-manoeuvre2copyimage4bpu to: " + str( max2copyimagestate4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4bpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimage4bpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4bpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4bpu = consumerchannel2copymanoeuvre4bpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagereceive4bpu-received-by-manoeuvre2copyimage4bpu: " + str(message2copyimagereceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2copy4bpu</copy4xi>
                         
            if message2copyimagereceive4bpu == '<copy4xi>copy2copy4bpu</copy4xi>':
                message2copyimagesend4bpu = '<roger4xi>roger2copy4bpu</roger4xi>'
            else:
                message2copyimagesend4bpu = '<negative4xi><negative2copy4bpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4bpu.send(message2copyimagesend4bpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagesend4bpu-sent-by-manoeuvre2copyimage4bpu: " + str(message2copyimagesend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2copy4bpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagereceive4bpu-failed-by-manoeuvre2copyimage4bpu: " + str(message2copyimagereceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4bpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimage4bpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4bpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4bpu = consumerchannel2copymanoeuvre4bpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagereceive4bpu-received-by-manoeuvre2copyimage4bpu: " + str(message2copyimagereceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2copy4bpu</ready4xi>
                         
            if message2copyimagereceive4bpu == '<ready4xi>ready2copy4bpu</ready4xi>':
                message2copyimagesend4bpu = '<ready4xi>ready2copy4bpu</ready4xi>'
            else:
                message2copyimagesend4bpu = '<negative4xi><negative2copy4bpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4bpu.send(message2copyimagesend4bpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagesend4bpu-sent-by-manoeuvre2copyimage4bpu: " + str(message2copyimagesend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagereceive4bpu-failed-by-manoeuvre2copyimage4bpu: " + str(message2copyimagereceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4bpu == 3:    
        info2image4bpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4bpu-try-by-manoeuvre2copyimage4bpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4bpu = queue2copymanoeuvre4bpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4bpu-got-by-manoeuvre2copyimage4bpu: " + str(info2image4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4bpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimageinfo4bpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimageinfo4bpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copymanoeuvre4bpu-failed-by-manoeuvre2copyimage4bpu: " + str(queue2copymanoeuvre4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagesend4bpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimageinfo4bpu is True:
                message2copyimagesend4bpu = '<ack4xi>ack2imageinfo4bpu</ack4xi>'
            else:
                message2copyimagesend4bpu = '<nack4xi>nack2imageinfo4bpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagesend4bpu-set-by-manoeuvre2copyimage4bpu to: " + str(message2copyimagesend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4bpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4bpu-failed-by-manoeuvre2copyimage4bpu: " + str(message2copyimagesend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copymanoeuvre4bpu-calling-by-manoeuvre2copyimage4bpu: " + str(publisherchannel2copymanoeuvre4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagesend4bpu-passing-to-send-by-manoeuvre2copyimage4bpu: " + str(message2copyimagesend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4bpu-failed-by-manoeuvre2copyimage4bpu: " + str(message2copyimagesend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copymanoeuvre4bpu.send(message2copyimagesend4bpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagesend4bpu-sent-by-manoeuvre2copyimage4bpu: " + str(message2copyimagesend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4bpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagesend4bpu-failed-by-manoeuvre2copyimage4bpu: " + str(message2copyimagesend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copy['ACK2INFO4BPU']        = ack2copyimageinfo4bpu
    response2manouvre4copy['CODE2ACK4BPU']        = code2ack
    response2manouvre4copy['CODE2NACK4BPU']       = code2nack
    response2manouvre4copy['MESSAGE2GET4BPU']     = message2copyimageget4bpu
    response2manouvre4copy['MESSAGE2PUT4BPU']     = message2copyimageput4bpu
    response2manouvre4copy['MESSAGE2RECEIVE4BPU'] = message2copyimagereceive4bpu
    response2manouvre4copy['MESSAGE2SEND4BPU']    = message2copyimagesend4bpu
    response2manouvre4copy['STATE2MANOEUVRE4BPU'] = state2copyimage4bpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copy-returning-by-manoeuvre2copyimage4bpu: " + str(response2manouvre4copy) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4BPU': '<roger4xi>roger2copy4bpu</roger4xi>',
    #   'ACK2INFO4BPU': False,
    #   'MESSAGE2GET4BPU': '<default4xi>default2copy4bpu</default4xi>',
    #   'MESSAGE2RECEIVE4BPU': '<copy4xi>copy2copy4bpu</copy4xi>',
    #   'CODE2ACK4BPU': ' 0D-1C-',
    #   'MESSAGE2PUT4BPU': '<default4xi>default2copy4bpu</default4xi>',
    #   'CODE2NACK4BPU': ' ',
    #   'STATE2MANOEUVRE4BPU': 1}; time: 05:25:15 10/03/11 Pacific Daylight Time}

 
    #2 {'MESSAGE2SEND4BPU': '<ready4xi>ready2copy4bpu</ready4xi>',
    #   'ACK2INFO4BPU': False,
    #   'MESSAGE2GET4BPU': '<default4xi>default2copy4bpu</default4xi>',
    #   'MESSAGE2RECEIVE4BPU': '<ready4xi>ready2copy4bpu</ready4xi>',
    #   'CODE2ACK4BPU': ' 0D-2C-',
    #   'MESSAGE2PUT4BPU': '<default4xi>default2copy4bpu</default4xi>',
    #   'CODE2NACK4BPU': ' ',
    #   'STATE2MANOEUVRE4BPU': 2}; time: 05:25:15 10/03/11 Pacific Daylight Time}

   
    #3 {'MESSAGE2SEND4BPU': '<ack4xi>ack2imageinfo4bpu</ack4xi>',
    #   'ACK2INFO4BPU': True,
    #   'MESSAGE2GET4BPU': '<default4xi>default2copy4bpu</default4xi>',
    #   'MESSAGE2RECEIVE4BPU': '<ready4xi>ready2copy4bpu</ready4xi>',
    #   'CODE2ACK4BPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4BPU': '<default4xi>default2copy4bpu</default4xi>',
    #   'CODE2NACK4BPU': ' ', 'STATE2MANOEUVRE4BPU': 3}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    
    #4 {'MESSAGE2SEND4BPU': '<default4xi>default2copy4bpu</default4xi>',
    #   'ACK2INFO4BPU': False,
    #   'MESSAGE2GET4BPU': '<default4xi>default2copy4bpu</default4xi>',
    #   'MESSAGE2RECEIVE4BPU': '<default4xi>default2copy4bpu</default4xi>',
    #   'CODE2ACK4BPU': ' 0D-',
    #   'MESSAGE2PUT4BPU': '<default4xi>default2copy4bpu</default4xi>',
    #   'CODE2NACK4BPU': ' ', 'STATE2MANOEUVRE4BPU': 0}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    return response2manouvre4copy           
          
#</function4xi>


# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimagetobuffer4bpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4BPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4bpu is True:
#       print("(Step 62A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4BPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimagetobuffer4bpu(queue4copy2imagetobuffer4bpu, channel4copy2imagetobufferconsume4bpu, channel4copy2imagetobufferpublish4bpu)
#           print("(Step 62B/700) OF BPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4BPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # { 'MESSAGE2SEND4BPU': '<roger4xi>roger2copyimagetobuffer4bpu</roger4xi>',
#           #   'ACK2INFO4BPU': False,
#           #   'MESSAGE2GET4BPU': '<default4xi>default2copyimagetobuffer4bpu</default4xi>',
#           #   'MESSAGE2RECEIVE4BPU': '<copy4xi>copy2copyimagetobuffer4bpu</copy4xi>',
#           #   'CODE2ACK4BPU': ' 0D-1C-',
#           #   'MESSAGE2PUT4BPU': '<default4xi>default2copyimagetobuffer4bpu</default4xi>',
#           #   'CODE2NACK4BPU': ' ',
#           #   'STATE2MANOEUVRE4BPU': 1}
#        except:
#           print("(Step 62Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4BPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimagetobuffer4bpu(queue2copyimagetobuffermanoeuvre4bpu, consumerchannel2copyimagetobuffermanoeuvre4bpu, publisherchannel2copyimagetobuffermanoeuvre4bpu):
    '''
    manoeuvres copy control and command for copy2imagetobuffer4bpu
    '''
    global state2copyimagetobuffer4bpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagetobufferstate4bpu                                                                                        # MAX FOR STATE
    global ack2copyimagetobufferinfo4bpu                                                                                         # ACK FOR IMAGE INFO
    global message2copyimagetobufferget4bpu                                 
    global message2copyimagetobufferput4bpu
    global message2copyimagetobufferreceive4bpu
    global message2copyimagetobuffersend4bpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimagetobuffer4bpu" 
    method2print = "manoeuvre2copyimagetobuffer4bpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimagetobuffer4bpu-started: " + str(manoeuvre2copyimagetobuffer4bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copyimagetobuffermanoeuvre4bpu-to-manoeuvre2copyimagetobuffer4bpu: " + str(queue2copyimagetobuffermanoeuvre4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copyimagetobuffermanoeuvre4bpu-to-manoeuvre2copyimagetobuffer4bpu: " + str(consumerchannel2copyimagetobuffermanoeuvre4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copyimagetobuffermanoeuvre4bpu-to-manoeuvre2copyimagetobuffer4bpu: " + str(publisherchannel2copyimagetobuffermanoeuvre4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copyimagetobuffer = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimagetobuffer4bpu-try-by-manoeuvre2copyimagetobuffer4bpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimagetobuffer4bpu is None:
            state2copyimagetobuffer4bpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimagetobuffer4bpu-initialized-by-manoeuvre2copyimagetobuffer4bpu to: " + str(state2copyimagetobuffer4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimagetobuffer4bpu-used-by-manoeuvre2copyimagetobuffer4bpu: " + str(state2copyimagetobuffer4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimagetobuffer4bpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimagetobuffer4bpu-initialized-by-manoeuvre2copyimagetobuffer4bpu to: " + str(state2copyimagetobuffer4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimagetobufferinfo4bpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimagetobufferinfo4bpu-initialized-by-manoeuvre2copyimagetobuffer4bpu to: " + str(ack2copyimagetobufferinfo4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimagetobufferget4bpu     = '<default4xi>default2copyimagetobuffer4bpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimagetobufferget4bpu-initialized-by-manoeuvre2copyimagetobuffer4bpu to: " + str(message2copyimagetobufferget4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4bpu     = '<default4xi>default2copyimagetobuffer4bpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimagetobufferput4bpu-initialized-by-manoeuvre2copyimagetobuffer4bpu to: " + str(message2copyimagetobufferput4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4bpu = '<default4xi>default2copyimagetobuffer4bpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagetobufferreceive4bpu-initialized-by-manoeuvre2copyimagetobuffer4bpu to: " + str(message2copyimagetobufferreceive4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4bpu    = '<default4xi>default2copyimagetobuffer4bpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagetobuffersend4bpu-initialized-by-manoeuvre2copyimagetobuffer4bpu to: " + str(message2copyimagetobuffersend4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4bpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagetobufferstate4bpu-initialized-by-manoeuvre2copyimagetobuffer4bpu to: " + str( max2copyimagetobufferstate4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimagetobuffer4bpu = state2copyimagetobuffer4bpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimagetobuffer4bpu-added-by-manoeuvre2copyimagetobuffer4bpu: " + str(state2copyimagetobuffer4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copyimagetobuffer4bpu > max2copyimagetobufferstate4bpu:
        state2copyimagetobuffer4bpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimagetobuffer4bpu-set-by-manoeuvre2copyimagetobuffer4bpu to: " + str(state2copyimagetobuffer4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copyimagetobufferinfo4bpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimagetobufferinfo4bpu-set-by-manoeuvre2copyimagetobuffer4bpu to: " + str(ack2copyimagetobufferinfo4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copyimagetobufferget4bpu     = '<default4xi>default2copyimagetobuffer4bpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimagetobufferget4bpu-set-by-manoeuvre2copyimagetobuffer4bpu to: " + str(message2copyimagetobufferget4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4bpu     = '<default4xi>default2copyimagetobuffer4bpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimagetobufferput4bpu-set-by-manoeuvre2copyimagetobuffer4bpu to: " + str(message2copyimagetobufferput4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4bpu = '<default4xi>default2copyimagetobuffer4bpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagetobufferreceive4bpu-set-by-manoeuvre2copyimagetobuffer4bpu to: " + str(message2copyimagetobufferreceive4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4bpu    = '<default4xi>default2copyimagetobuffer4bpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagetobuffersend4bpu-set-by-manoeuvre2copyimagetobuffer4bpu to: " + str(message2copyimagetobuffersend4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4bpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagetobufferstate4bpu-set-by-manoeuvre2copyimagetobuffer4bpu to: " + str( max2copyimagetobufferstate4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4bpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4bpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4bpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4bpu = consumerchannel2copyimagetobuffermanoeuvre4bpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagetobufferreceive4bpu-received-by-manoeuvre2copyimagetobuffer4bpu: " + str(message2copyimagetobufferreceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4bpu == '<copy4xi>copy2copyimagetobuffer4bpu</copy4xi>': 
                message2copyimagetobuffersend4bpu = '<roger4xi>roger2copyimagetobuffer4bpu</roger4xi>'
            else:
                message2copyimagetobuffersend4bpu = '<negative4xi><negative2copyimagetobuffer4bpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4bpu.send(message2copyimagetobuffersend4bpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagetobuffersend4bpu-sent-by-manoeuvre2copyimagetobuffer4bpu: " + str(message2copyimagetobuffersend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagetobufferreceive4bpu-failed-by-manoeuvre2copyimagetobuffer4bpu: " + str(message2copyimagetobufferreceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4bpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4bpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4bpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4bpu = consumerchannel2copyimagetobuffermanoeuvre4bpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagetobufferreceive4bpu-received-by-manoeuvre2copyimagetobuffer4bpu: " + str(message2copyimagetobufferreceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4bpu == '<ready4xi>ready2copyimagetobuffer4bpu</ready4xi>':
                message2copyimagetobuffersend4bpu = '<ready4xi>ready2copyimagetobuffer4bpu</ready4xi>'
            else:
                message2copyimagetobuffersend4bpu = '<negative4xi><negative2copyimagetobuffer4bpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4bpu.send(message2copyimagetobuffersend4bpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagetobuffersend4bpu-sent-by-manoeuvre2copyimagetobuffer4bpu: " + str(message2copyimagetobuffersend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagetobufferreceive4bpu-failed-by-manoeuvre2copyimagetobuffer4bpu: " + str(message2copyimagetobufferreceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4bpu == 3:    
        info2image4bpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4bpu-try-by-manoeuvre2copyimagetobuffer4bpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4bpu = queue2copyimagetobuffermanoeuvre4bpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4bpu-got-by-manoeuvre2copyimagetobuffer4bpu: " + str(info2image4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4bpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimagetobufferinfo4bpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimagetobufferinfo4bpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copyimagetobuffermanoeuvre4bpu-failed-by-manoeuvre2copyimagetobuffer4bpu: " + str(queue2copyimagetobuffermanoeuvre4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagetobuffersend4bpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimagetobufferinfo4bpu is True:
                message2copyimagetobuffersend4bpu = '<ack4xi>ack2imageinfo4bpu</ack4xi>'
            else:
                message2copyimagetobuffersend4bpu = '<nack4xi>nack2imageinfo4bpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagetobuffersend4bpu-set-by-manoeuvre2copyimagetobuffer4bpu to: " + str(message2copyimagetobuffersend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4bpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4bpu-failed-by-manoeuvre2copyimagetobuffer4bpu: " + str(message2copyimagetobuffersend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copyimagetobuffermanoeuvre4bpu-calling-by-manoeuvre2copyimagetobuffer4bpu: " + str(publisherchannel2copyimagetobuffermanoeuvre4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagetobuffersend4bpu-passing-to-send-by-manoeuvre2copyimagetobuffer4bpu: " + str(message2copyimagetobuffersend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4bpu-failed-by-manoeuvre2copyimagetobuffer4bpu: " + str(message2copyimagetobuffersend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copyimagetobuffermanoeuvre4bpu.send(message2copyimagetobuffersend4bpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagetobuffersend4bpu-sent-by-manoeuvre2copyimagetobuffer4bpu: " + str(message2copyimagetobuffersend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4bpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagetobuffersend4bpu-failed-by-manoeuvre2copyimagetobuffer4bpu: " + str(message2copyimagetobuffersend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copyimagetobuffer['ACK2INFO4BPU']        = ack2copyimagetobufferinfo4bpu
    response2manouvre4copyimagetobuffer['CODE2ACK4BPU']        = code2ack
    response2manouvre4copyimagetobuffer['CODE2NACK4BPU']       = code2nack
    response2manouvre4copyimagetobuffer['MESSAGE2GET4BPU']     = message2copyimagetobufferget4bpu
    response2manouvre4copyimagetobuffer['MESSAGE2PUT4BPU']     = message2copyimagetobufferput4bpu
    response2manouvre4copyimagetobuffer['MESSAGE2RECEIVE4BPU'] = message2copyimagetobufferreceive4bpu
    response2manouvre4copyimagetobuffer['MESSAGE2SEND4BPU']    = message2copyimagetobuffersend4bpu
    response2manouvre4copyimagetobuffer['STATE2MANOEUVRE4BPU'] = state2copyimagetobuffer4bpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copyimagetobuffer-returning-by-manoeuvre2copyimagetobuffer4bpu: " + str(response2manouvre4copyimagetobuffer) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4BPU': '<roger4xi>roger2copyimagetobuffer4bpu</roger4xi>',
    #   'ACK2INFO4BPU': False,
    #   'MESSAGE2GET4BPU': '<default4xi>default2copyimagetobuffer4bpu</default4xi>',
    #   'MESSAGE2RECEIVE4BPU': '<copy4xi>copy2copyimagetobuffer4bpu</copy4xi>',
    #   'CODE2ACK4BPU': ' 0D-1C-',
    #   'MESSAGE2PUT4BPU': '<default4xi>default2copyimagetobuffer4bpu</default4xi>',
    #   'CODE2NACK4BPU': ' ',
    #   'STATE2MANOEUVRE4BPU': 1}; time: 02:00:20 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4BPU': '<ready4xi>ready2copyimagetobuffer4bpu</ready4xi>',
    #   'ACK2INFO4BPU': False,
    #   'MESSAGE2GET4BPU': '<default4xi>default2copyimagetobuffer4bpu</default4xi>',
    #   'MESSAGE2RECEIVE4BPU': '<ready4xi>ready2copyimagetobuffer4bpu</ready4xi>',
    #   'CODE2ACK4BPU': ' 0D-2C-',
    #   'MESSAGE2PUT4BPU': '<default4xi>default2copyimagetobuffer4bpu</default4xi>',
    #   'CODE2NACK4BPU': ' ',
    #   'STATE2MANOEUVRE4BPU': 2}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #3 {'MESSAGE2SEND4BPU': '<ack4xi>ack2imageinfo4bpu</ack4xi>',
    #   'ACK2INFO4BPU': True,
    #   'MESSAGE2GET4BPU': '<default4xi>default2copyimagetobuffer4bpu</default4xi>',
    #   'MESSAGE2RECEIVE4BPU': '<ready4xi>ready2copyimagetobuffer4bpu</ready4xi>',
    #   'CODE2ACK4BPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4BPU': '<default4xi>default2copyimagetobuffer4bpu</default4xi>',
    #   'CODE2NACK4BPU': ' ',
    #   'STATE2MANOEUVRE4BPU': 3}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #4 {'MESSAGE2SEND4BPU': '<default4xi>default2copyimagetobuffer4bpu</default4xi>',
    #   'ACK2INFO4BPU': False,
    #   'MESSAGE2GET4BPU': '<default4xi>default2copyimagetobuffer4bpu</default4xi>',
    #   'MESSAGE2RECEIVE4BPU': '<default4xi>default2copyimagetobuffer4bpu</default4xi>',
    #   'CODE2ACK4BPU': ' 0D-',
    #   'MESSAGE2PUT4BPU': '<default4xi>default2copyimagetobuffer4bpu</default4xi>',
    #   'CODE2NACK4BPU': ' ',
    #   'STATE2MANOEUVRE4BPU': 0}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    return response2manouvre4copyimagetobuffer           
          
#</function4xi>

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copybuffertoimage4bpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4BPU - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4bpu is True:
#       print("(Step 82A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4BPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copybuffertoimage4bpu(queue4copybuffertoimage4bpu, channel4copybuffertoimageconsume4bpu, channel4copybuffertoimagepublish4bpu)
#           print("(Step 82B/700) OF BPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4BPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4BPU': '<roger4xi>roger2copybuffertoimage4bpu</roger4xi>',
#           #   'ACK2INFO4BPU': False,
#           #   'MESSAGE2GET4BPU': '<default4xi>default2copybuffertoimage4bpu</default4xi>',
#           #   'MESSAGE2RECEIVE4BPU': '<copy4xi>copy2copybuffertoimage4bpu</copy4xi>',
#           #   'CODE2ACK4BPU': ' 0D-1C-',
#           #   'MESSAGE2PUT4BPU': '<default4xi>default2copybuffertoimage4bpu</default4xi>',
#           #   'CODE2NACK4BPU': ' ',
#           #   'STATE2MANOEUVRE4BPU': 1}
#       except:
#           print("(Step 82Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4BPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copybuffertoimage4bpu(queue2copybuffertoimagemanoeuvre4bpu, consumerchannel2copybuffertoimagemanoeuvre4bpu, publisherchannel2copybuffertoimagemanoeuvre4bpu):
    '''
    manoeuvres copy control and command for copy2buffertoimage4bpu
    '''
    global state2copybuffertoimage4bpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copybuffertoimagestate4bpu                                                                                        # MAX FOR STATE
    global ack2copybuffertoimageinfo4bpu                                                                                         # ACK FOR IMAGE INFO
    global message2copybuffertoimageget4bpu                                 
    global message2copybuffertoimageput4bpu
    global message2copybuffertoimagereceive4bpu
    global message2copybuffertoimagesend4bpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copybuffertoimage4bpu" 
    method2print = "manoeuvre2copybuffertoimage4bpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copybuffertoimage4bpu-started: " + str(manoeuvre2copybuffertoimage4bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copybuffertoimagemanoeuvre4bpu-to-manoeuvre2copybuffertoimage4bpu: " + str(queue2copybuffertoimagemanoeuvre4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copybuffertoimagemanoeuvre4bpu-to-manoeuvre2copybuffertoimage4bpu: " + str(consumerchannel2copybuffertoimagemanoeuvre4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copybuffertoimagemanoeuvre4bpu-to-manoeuvre2copybuffertoimage4bpu: " + str(publisherchannel2copybuffertoimagemanoeuvre4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copybuffertoimage = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copybuffertoimage4bpu-try-by-manoeuvre2copybuffertoimage4bpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copybuffertoimage4bpu is None:
            state2copybuffertoimage4bpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copybuffertoimage4bpu-initialized-by-manoeuvre2copybuffertoimage4bpu to: " + str(state2copybuffertoimage4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copybuffertoimage4bpu-used-by-manoeuvre2copybuffertoimage4bpu: " + str(state2copybuffertoimage4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copybuffertoimage4bpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copybuffertoimage4bpu-initialized-by-manoeuvre2copybuffertoimage4bpu to: " + str(state2copybuffertoimage4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copybuffertoimageinfo4bpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copybuffertoimageinfo4bpu-initialized-by-manoeuvre2copybuffertoimage4bpu to: " + str(ack2copybuffertoimageinfo4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copybuffertoimageget4bpu     = '<default4xi>default2copybuffertoimage4bpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copybuffertoimageget4bpu-initialized-by-manoeuvre2copybuffertoimage4bpu to: " + str(message2copybuffertoimageget4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4bpu     = '<default4xi>default2copybuffertoimage4bpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copybuffertoimageput4bpu-initialized-by-manoeuvre2copybuffertoimage4bpu to: " + str(message2copybuffertoimageput4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4bpu = '<default4xi>default2copybuffertoimage4bpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copybuffertoimagereceive4bpu-initialized-by-manoeuvre2copybuffertoimage4bpu to: " + str(message2copybuffertoimagereceive4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4bpu    = '<default4xi>default2copybuffertoimage4bpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copybuffertoimagesend4bpu-initialized-by-manoeuvre2copybuffertoimage4bpu to: " + str(message2copybuffertoimagesend4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4bpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copybuffertoimagestate4bpu-initialized-by-manoeuvre2copybuffertoimage4bpu to: " + str( max2copybuffertoimagestate4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copybuffertoimage4bpu = state2copybuffertoimage4bpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copybuffertoimage4bpu-added-by-manoeuvre2copybuffertoimage4bpu: " + str(state2copybuffertoimage4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copybuffertoimage4bpu > max2copybuffertoimagestate4bpu:
        state2copybuffertoimage4bpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copybuffertoimage4bpu-set-by-manoeuvre2copybuffertoimage4bpu to: " + str(state2copybuffertoimage4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copybuffertoimageinfo4bpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copybuffertoimageinfo4bpu-set-by-manoeuvre2copybuffertoimage4bpu to: " + str(ack2copybuffertoimageinfo4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copybuffertoimageget4bpu     = '<default4xi>default2copybuffertoimage4bpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copybuffertoimageget4bpu-set-by-manoeuvre2copybuffertoimage4bpu to: " + str(message2copybuffertoimageget4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4bpu     = '<default4xi>default2copybuffertoimage4bpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copybuffertoimageput4bpu-set-by-manoeuvre2copybuffertoimage4bpu to: " + str(message2copybuffertoimageput4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4bpu = '<default4xi>default2copybuffertoimage4bpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copybuffertoimagereceive4bpu-set-by-manoeuvre2copybuffertoimage4bpu to: " + str(message2copybuffertoimagereceive4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4bpu    = '<default4xi>default2copybuffertoimage4bpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copybuffertoimagesend4bpu-set-by-manoeuvre2copybuffertoimage4bpu to: " + str(message2copybuffertoimagesend4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4bpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copybuffertoimagestate4bpu-set-by-manoeuvre2copybuffertoimage4bpu to: " + str( max2copybuffertoimagestate4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4bpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4bpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4bpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4bpu = consumerchannel2copybuffertoimagemanoeuvre4bpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copybuffertoimagereceive4bpu-received-by-manoeuvre2copybuffertoimage4bpu: " + str(message2copybuffertoimagereceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4bpu == '<copy4xi>copy2copybuffertoimage4bpu</copy4xi>':
                message2copybuffertoimagesend4bpu = '<roger4xi>roger2copybuffertoimage4bpu</roger4xi>'
            else:
                message2copybuffertoimagesend4bpu = '<negative4xi><negative2copybuffertoimage4bpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4bpu.send(message2copybuffertoimagesend4bpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copybuffertoimagesend4bpu-sent-by-manoeuvre2copybuffertoimage4bpu: " + str(message2copybuffertoimagesend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copybuffertoimagereceive4bpu-failed-by-manoeuvre2copybuffertoimage4bpu: " + str(message2copybuffertoimagereceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4bpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4bpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4bpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4bpu = consumerchannel2copybuffertoimagemanoeuvre4bpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copybuffertoimagereceive4bpu-received-by-manoeuvre2copybuffertoimage4bpu: " + str(message2copybuffertoimagereceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4bpu == '<ready4xi>ready2copybuffertoimage4bpu</ready4xi>':
                message2copybuffertoimagesend4bpu = '<ready4xi>ready2copybuffertoimage4bpu</ready4xi>'
            else:
                message2copybuffertoimagesend4bpu = '<negative4xi><negative2copybuffertoimage4bpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4bpu.send(message2copybuffertoimagesend4bpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copybuffertoimagesend4bpu-sent-by-manoeuvre2copybuffertoimage4bpu: " + str(message2copybuffertoimagesend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copybuffertoimagereceive4bpu-failed-by-manoeuvre2copybuffertoimage4bpu: " + str(message2copybuffertoimagereceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4bpu == 3:    
        info2image4bpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4bpu-try-by-manoeuvre2copybuffertoimage4bpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4bpu = queue2copybuffertoimagemanoeuvre4bpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4bpu-got-by-manoeuvre2copybuffertoimage4bpu: " + str(info2image4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4bpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copybuffertoimageinfo4bpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copybuffertoimageinfo4bpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copybuffertoimagemanoeuvre4bpu-failed-by-manoeuvre2copybuffertoimage4bpu: " + str(queue2copybuffertoimagemanoeuvre4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copybuffertoimagesend4bpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copybuffertoimageinfo4bpu is True:
                message2copybuffertoimagesend4bpu = '<ack4xi>ack2imageinfo4bpu</ack4xi>'
            else:
                message2copybuffertoimagesend4bpu = '<nack4xi>nack2imageinfo4bpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copybuffertoimagesend4bpu-set-by-manoeuvre2copybuffertoimage4bpu to: " + str(message2copybuffertoimagesend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4bpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4bpu-failed-by-manoeuvre2copybuffertoimage4bpu: " + str(message2copybuffertoimagesend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copybuffertoimagemanoeuvre4bpu-calling-by-manoeuvre2copybuffertoimage4bpu: " + str(publisherchannel2copybuffertoimagemanoeuvre4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copybuffertoimagesend4bpu-passing-to-send-by-manoeuvre2copybuffertoimage4bpu: " + str(message2copybuffertoimagesend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4bpu-failed-by-manoeuvre2copybuffertoimage4bpu: " + str(message2copybuffertoimagesend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copybuffertoimagemanoeuvre4bpu.send(message2copybuffertoimagesend4bpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copybuffertoimagesend4bpu-sent-by-manoeuvre2copybuffertoimage4bpu: " + str(message2copybuffertoimagesend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4bpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copybuffertoimagesend4bpu-failed-by-manoeuvre2copybuffertoimage4bpu: " + str(message2copybuffertoimagesend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copybuffertoimage['ACK2INFO4BPU']        = ack2copybuffertoimageinfo4bpu
    response2manouvre4copybuffertoimage['CODE2ACK4BPU']        = code2ack
    response2manouvre4copybuffertoimage['CODE2NACK4BPU']       = code2nack
    response2manouvre4copybuffertoimage['MESSAGE2GET4BPU']     = message2copybuffertoimageget4bpu
    response2manouvre4copybuffertoimage['MESSAGE2PUT4BPU']     = message2copybuffertoimageput4bpu
    response2manouvre4copybuffertoimage['MESSAGE2RECEIVE4BPU'] = message2copybuffertoimagereceive4bpu
    response2manouvre4copybuffertoimage['MESSAGE2SEND4BPU']    = message2copybuffertoimagesend4bpu
    response2manouvre4copybuffertoimage['STATE2MANOEUVRE4BPU'] = state2copybuffertoimage4bpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copybuffertoimage-returning-by-manoeuvre2copybuffertoimage4bpu: " + str(response2manouvre4copybuffertoimage) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4BPU': '<roger4xi>roger2copybuffertoimage4bpu</roger4xi>',
    #   'ACK2INFO4BPU': False,
    #   'MESSAGE2GET4BPU': '<default4xi>default2copybuffertoimage4bpu</default4xi>',
    #   'MESSAGE2RECEIVE4BPU': '<copy4xi>copy2copybuffertoimage4bpu</copy4xi>',
    #   'CODE2ACK4BPU': ' 0D-1C-',
    #   'MESSAGE2PUT4BPU': '<default4xi>default2copybuffertoimage4bpu</default4xi>',
    #   'CODE2NACK4BPU': ' ',
    #   'STATE2MANOEUVRE4BPU': 1}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4BPU': '<ready4xi>ready2copybuffertoimage4bpu</ready4xi>',
    #   'ACK2INFO4BPU': False,
    #   'MESSAGE2GET4BPU': '<default4xi>default2copybuffertoimage4bpu</default4xi>',
    #   'MESSAGE2RECEIVE4BPU': '<ready4xi>ready2copybuffertoimage4bpu</ready4xi>',
    #   'CODE2ACK4BPU': ' 0D-2C-',
    #   'MESSAGE2PUT4BPU': '<default4xi>default2copybuffertoimage4bpu</default4xi>',
    #   'CODE2NACK4BPU': ' ',
    #   'STATE2MANOEUVRE4BPU': 2}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    #3 {'MESSAGE2SEND4BPU': '<ack4xi>ack2imageinfo4bpu</ack4xi>',
    #   'ACK2INFO4BPU': True,
    #   'MESSAGE2GET4BPU': '<default4xi>default2copybuffertoimage4bpu</default4xi>',
    #   'MESSAGE2RECEIVE4BPU': '<ready4xi>ready2copybuffertoimage4bpu</ready4xi>',
    #   'CODE2ACK4BPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4BPU': '<default4xi>default2copybuffertoimage4bpu</default4xi>',
    #   'CODE2NACK4BPU': ' ',
    #   'STATE2MANOEUVRE4BPU': 3}; time: 02:18:55 10/04/11 Pacific Daylight Time}
 

    #4 {'MESSAGE2SEND4BPU': '<default4xi>default2copybuffertoimage4bpu</default4xi>',
    #   'ACK2INFO4BPU': False,
    #   'MESSAGE2GET4BPU': '<default4xi>default2copybuffertoimage4bpu</default4xi>',
    #   'MESSAGE2RECEIVE4BPU': '<default4xi>default2copybuffertoimage4bpu</default4xi>',
    #   'CODE2ACK4BPU': ' 0D-',
    #   'MESSAGE2PUT4BPU': '<default4xi>default2copybuffertoimage4bpu</default4xi>',
    #   'CODE2NACK4BPU': ' ',
    #   'STATE2MANOEUVRE4BPU': 0}

    return response2manouvre4copybuffertoimage           
          
#</function4xi>



# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2readimage4bpu" usage = "control function for read2image4bpu function in this module">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2readimage4bpu(queue2readmanoeuvre4bpu, consumerchannel2readmanoeuvre4bpu, publisherchannel2readmanoeuvre4bpu):
    '''
    manoeuvres read control and command
    '''
    global state2read4bpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2readstate4bpu                                                                                        # MAX FOR STATE
    global ack2readinfo4bpu                                                                                         # ACK FOR IMAGE INFO
    global message2readget4bpu                                 
    global message2readput4bpu
    global message2readreceive4bpu
    global message2readsend4bpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2readimage4bpu" 
    method2print = "manoeuvre2readimage4bpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2readimage4bpu-started: " + str(manoeuvre2readimage4bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2readimage4bpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2readmanoeuvre4bpu-to-manoeuvre2readimage4bpu: " + str(queue2readmanoeuvre4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2readmanoeuvre4bpu-to-manoeuvre2readimage4bpu: " + str(consumerchannel2readmanoeuvre4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2readmanoeuvre4bpu-to-manoeuvre2readimage4bpu: " + str(publisherchannel2readmanoeuvre4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4read = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2read4bpu-try-by-manoeuvre2readimage4bpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2read4bpu is None:
            state2read4bpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2read4bpu-initialized-by-manoeuvre2readimage4bpu to: " + str(state2read4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2read4bpu-used-by-manoeuvre2readimage4bpu: " + str(state2read4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2read4bpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2read4bpu-initialized-by-manoeuvre2readimage4bpu to: " + str(state2read4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4bpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2readinfo4bpu-initialized-by-manoeuvre2readimage4bpu to: " + str(ack2readinfo4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4bpu     = '<default4xi>default2read4bpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2readget4bpu-initialized-by-manoeuvre2readimage4bpu to: " + str(message2readget4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4bpu</default4xi>
        message2readput4bpu     = '<default4xi>default2read4bpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2readput4bpu-initialized-by-manoeuvre2readimage4bpu to: " + str(message2readput4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4bpu</default4xi>
        message2readreceive4bpu = '<default4xi>default2read4bpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2readreceive4bpu-initialized-by-manoeuvre2readimage4bpu to: " + str(message2readreceive4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4bpu</default4xi>
        message2readsend4bpu    = '<default4xi>default2read4bpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2readsend4bpu-initialized-by-manoeuvre2readimage4bpu to: " + str(message2readsend4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4bpu</default4xi>
        max2readstate4bpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2readstate4bpu-initialized-by-manoeuvre2readimage4bpu to: " + str( max2readstate4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2read4bpu = state2read4bpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2read4bpu-added-by-manoeuvre2readimage4bpu: " + str(state2read4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2read4bpu > max2readstate4bpu:
        state2read4bpu = 0
        message2print = "{(Step 0I/33) set" + "; state2read4bpu-set-by-manoeuvre2readimage4bpu to: " + str(state2read4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4bpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2readinfo4bpu-set-by-manoeuvre2readimage4bpu to: " + str(ack2readinfo4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4bpu     = '<default4xi>default2read4bpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2readget4bpu-set-by-manoeuvre2readimage4bpu to: " + str(message2readget4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4bpu</default4xi>
        message2readput4bpu     = '<default4xi>default2read4bpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2readput4bpu-set-by-manoeuvre2readimage4bpu to: " + str(message2readput4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4bpu</default4xi>
        message2readreceive4bpu = '<default4xi>default2read4bpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2readreceive4bpu-set-by-manoeuvre2readimage4bpu to: " + str(message2readreceive4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4bpu</default4xi>
        message2readsend4bpu    = '<default4xi>default2read4bpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2readsend4bpu-set-by-manoeuvre2readimage4bpu to: " + str(message2readsend4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4bpu</default4xi>
        max2readstate4bpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2readstate4bpu-set-by-manoeuvre2readimage4bpu to: " + str( max2readstate4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4bpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2readimage4bpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4bpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4bpu = consumerchannel2readmanoeuvre4bpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2readreceive4bpu-received-by-manoeuvre2readimage4bpu: " + str(message2readreceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2read4bpu</copy4xi>
                         
            if message2readreceive4bpu == '<copy4xi>copy2read4bpu</copy4xi>':
                message2readsend4bpu = '<roger4xi>roger2read4bpu</roger4xi>'
            else:
                message2readsend4bpu = '<negative4xi><negative2read4bpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4bpu.send(message2readsend4bpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2readsend4bpu-sent-by-manoeuvre2readimage4bpu: " + str(message2readsend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2read4bpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2readreceive4bpu-failed-by-manoeuvre2readimage4bpu: " + str(message2readreceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4bpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2readimage4bpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4bpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4bpu = consumerchannel2readmanoeuvre4bpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2readreceive4bpu-received-by-manoeuvre2readimage4bpu: " + str(message2readreceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2read4bpu</ready4xi> 
                         
            if message2readreceive4bpu == '<ready4xi>ready2read4bpu</ready4xi>':
                message2readsend4bpu = '<ready4xi>ready2read4bpu</ready4xi>'
            else:
                message2readsend4bpu = '<negative4xi><negative2ready4bpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4bpu.send(message2readsend4bpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2readsend4bpu-sent-by-manoeuvre2readimage4bpu: " + str(message2readsend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2read4bpu</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2readreceive4bpu-failed-by-manoeuvre2readimage4bpu: " + str(message2readreceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4bpu == 3:    
        info2image4bpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4bpu-try-by-manoeuvre2readimage4bpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4bpu = queue2readmanoeuvre4bpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4bpu-got-by-manoeuvre2readimage4bpu: " + str(info2image4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4bpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2readinfo4bpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2readinfo4bpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2readmanoeuvre4bpu-failed-by-manoeuvre2readimage4bpu: " + str(queue2readmanoeuvre4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2readsend4bpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2readinfo4bpu is True:
                message2readsend4bpu = '<ack4xi>ack2imageinfo4bpu</ack4xi>'
            else:
                message2readsend4bpu = '<nack4xi>nack2imageinfo4bpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2readsend4bpu-set-by-manoeuvre2readimage4bpu to: " + str(message2readsend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4bpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4bpu-failed-by-manoeuvre2readimage4bpu: " + str(message2readsend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2readmanoeuvre4bpu-calling-by-manoeuvre2readimage4bpu: " + str(publisherchannel2readmanoeuvre4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2readsend4bpu-passing-to-send-by-manoeuvre2readimage4bpu: " + str(message2readsend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4bpu-failed-by-manoeuvre2readimage4bpu: " + str(message2readsend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2readmanoeuvre4bpu.send(message2readsend4bpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2readsend4bpu-sent-by-manoeuvre2readimage4bpu: " + str(message2readsend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4bpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2readsend4bpu-failed-by-manoeuvre2readimage4bpu: " + str(message2readsend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4read['ACK2INFO4BPU']        = ack2readinfo4bpu
    response2manouvre4read['CODE2ACK4BPU']        = code2ack
    response2manouvre4read['CODE2NACK4BPU']       = code2nack
    response2manouvre4read['MESSAGE2GET4BPU']     = message2readget4bpu
    response2manouvre4read['MESSAGE2PUT4BPU']     = message2readput4bpu
    response2manouvre4read['MESSAGE2RECEIVE4BPU'] = message2readreceive4bpu
    response2manouvre4read['MESSAGE2SEND4BPU']    = message2readsend4bpu
    response2manouvre4read['STATE2MANOEUVRE4BPU'] = state2read4bpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4read-returning-by-manoeuvre2readimage4bpu: " + str(response2manouvre4read) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4BPU': '<roger4xi>roger2read4bpu</roger4xi>',
    #  'ACK2INFO4BPU': False,
    #  'MESSAGE2GET4BPU': '<default4xi>default2read4bpu</default4xi>',
    #  'MESSAGE2RECEIVE4BPU': '<copy4xi>copy2read4bpu</copy4xi>',
    #  'CODE2ACK4BPU': ' 0D-1C-',
    #  'MESSAGE2PUT4BPU': '<default4xi>default2read4bpu</default4xi>',
    #  'CODE2NACK4BPU': ' ', 'STATE2MANOEUVRE4BPU': 1}; time: 01:44:40 09/29/11 Pacific Daylight Time}
 
    #2 {'MESSAGE2SEND4BPU': '<ready4xi>ready2read4bpu</ready4xi>',
    #  'ACK2INFO4BPU': False,
    #  'MESSAGE2GET4BPU': '<default4xi>default2read4bpu</default4xi>',
    #  'MESSAGE2RECEIVE4BPU': '<ready4xi>ready2read4bpu</ready4xi>',
    #  'CODE2ACK4BPU': ' 0D-2C-',
    #  'MESSAGE2PUT4BPU': '<default4xi>default2read4bpu</default4xi>',
    #  'CODE2NACK4BPU': ' ', 'STATE2MANOEUVRE4BPU': 2}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #3 {'MESSAGE2SEND4BPU': '<ack4xi>ack2imageinfo4bpu</ack4xi>',
    #  'ACK2INFO4BPU': True,
    #  'MESSAGE2GET4BPU': '<default4xi>default2read4bpu</default4xi>',
    #  'MESSAGE2RECEIVE4BPU': '<ready4xi>ready2read4bpu</ready4xi>',
    #  'CODE2ACK4BPU': ' 0D-3C-4E-',
    #  'MESSAGE2PUT4BPU': '<default4xi>default2read4bpu</default4xi>',
    #  'CODE2NACK4BPU': ' ',
    #  'STATE2MANOEUVRE4BPU': 3}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4BPU': '<default4xi>default2read4bpu</default4xi>',
    #  'ACK2INFO4BPU': False,
    #  'MESSAGE2GET4BPU': '<default4xi>default2read4bpu</default4xi>',
    #  'MESSAGE2RECEIVE4BPU': '<default4xi>default2read4bpu</default4xi>',
    #  'CODE2ACK4BPU': ' 0D-',
    #  'MESSAGE2PUT4BPU': '<default4xi>default2read4bpu</default4xi>',
    #  'CODE2NACK4BPU': ' ',
    #  'STATE2MANOEUVRE4BPU': 0}; time: 02:11:49 09/29/11 Pacific Daylight Time}


 
    
    
    return response2manouvre4read           
          
#</function4xi>


# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2writeimage4bpu" usage = "control function for write2image4bpu function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2writeimage4bpu(queue2writemanoeuvre4bpu, consumerchannel2writemanoeuvre4bpu, publisherchannel2writemanoeuvre4bpu):
    '''
    manoeuvres control and command
    '''
    global state2write4bpu                                                       # STATE FOR COMMAND AND CONTROL
    global max2writestate4bpu                                                    # MAX FOR STATE
    global ack2writeinfo4bpu                                                     # ACK FOR IMAGE INFO
    global message2writeget4bpu                                 
    global message2writeput4bpu
    global message2writereceive4bpu
    global message2writesend4bpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2writeimage4bpu" 
    method2print = "manoeuvre2writeimage4bpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2writeimage4bpu-started: " + str(manoeuvre2writeimage4bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    # <function manoeuvre2writeimage4bpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2writemanoeuvre4bpu-to-manoeuvre2writeimage4bpu: " + str(queue2writemanoeuvre4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x000000000422D9E8>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2writemanoeuvre4bpu-to-manoeuvre2writeimage4bpu: " + str(consumerchannel2writemanoeuvre4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # read-write PipeConnection, handle 568>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2writemanoeuvre4bpu-to-manoeuvre2writeimage4bpu: " + str(publisherchannel2writemanoeuvre4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 572> 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4write = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2write4bpu-try-by-manoeuvre2writeimage4bpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2write4bpu is None:
            state2write4bpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2write4bpu-initialized-by-manoeuvre2writeimage4bpu to: " + str(state2write4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2write4bpu-used-by-manoeuvre2writeimage4bpu: " + str(state2write4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2write4bpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2write4bpu-initialized-by-manoeuvre2writeimage4bpu to: " + str(state2write4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4bpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2writeinfo4bpu-initialized-by-manoeuvre2writeimage4bpu to: " + str(ack2writeinfo4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4bpu     = '<default4xi>default2write4bpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2writeget4bpu-initialized-by-manoeuvre2writeimage4bpu to: " + str(message2writeget4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4bpu</default4xi>
        message2writeput4bpu     = '<default4xi>default2write4bpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2writeput4bpu-initialized-by-manoeuvre2writeimage4bpu to: " + str(message2writeput4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4bpu</default4xi>
        message2writereceive4bpu = '<default4xi>default2write4bpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2writereceive4bpu-initialized-by-manoeuvre2writeimage4bpu to: " + str(message2writereceive4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4bpu</default4xi>
        message2writesend4bpu    = '<default4xi>default2write4bpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2writesend4bpu-initialized-by-manoeuvre2writeimage4bpu to: " + str(message2writesend4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4bpu</default4xi>
        max2writestate4bpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2writestate4bpu-initialized-by-manoeuvre2writeimage4bpu to: " + str( max2writestate4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2write4bpu = state2write4bpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2write4bpu-added-by-manoeuvre2writeimage4bpu: " + str(state2write4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2write4bpu > max2writestate4bpu:
        state2write4bpu = 0
        message2print = "{(Step 0I/33) set" + "; state2write4bpu-set-by-manoeuvre2writeimage4bpu to: " + str(state2write4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4bpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2writeinfo4bpu-set-by-manoeuvre2writeimage4bpu to: " + str(ack2writeinfo4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4bpu     = '<default4xi>default2write4bpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2writeget4bpu-set-by-manoeuvre2writeimage4bpu to: " + str(message2writeget4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4bpu</default4xi>
        message2writeput4bpu     = '<default4xi>default2write4bpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2writeput4bpu-set-by-manoeuvre2writeimage4bpu to: " + str(message2writeput4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4bpu</default4xi>
        message2writereceive4bpu = '<default4xi>default2write4bpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2writereceive4bpu-set-by-manoeuvre2writeimage4bpu to: " + str(message2writereceive4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4bpu</default4xi>
        message2writesend4bpu    = '<default4xi>default2write4bpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2writesend4bpu-set-by-manoeuvre2writeimage4bpu to: " + str(message2writesend4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4bpu</default4xi>
        max2writestate4bpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2writestate4bpu-set-by-manoeuvre2writeimage4bpu to: " + str( max2writestate4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4bpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2writeimage4bpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4bpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4bpu = consumerchannel2writemanoeuvre4bpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2writereceive4bpu-received-by-manoeuvre2writeimage4bpu: " + str(message2writereceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2write4bpu</copy4xi>
                         
            if message2writereceive4bpu == '<copy4xi>copy2write4bpu</copy4xi>':
                message2writesend4bpu = '<roger4xi>roger2write4bpu</roger4xi>'
            else:
                message2writesend4bpu = '<negative4xi><negative2write4bpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4bpu.send(message2writesend4bpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2writesend4bpu-sent-by-manoeuvre2writeimage4bpu: " + str(message2writesend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2write4bpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2writereceive4bpu-failed-by-manoeuvre2writeimage4bpu: " + str(message2writereceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4bpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2writeimage4bpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4bpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4bpu = consumerchannel2writemanoeuvre4bpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2writereceive4bpu-received-by-manoeuvre2writeimage4bpu: " + str(message2writereceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2write4bpu</ready4xi>
                         
            if message2writereceive4bpu == '<ready4xi>ready2write4bpu</ready4xi>':
                message2writesend4bpu = '<ready4xi>ready2write4bpu</ready4xi>'
            else:
                message2writesend4bpu = '<negative4xi><negative2ready4bpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4bpu.send(message2writesend4bpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2writesend4bpu-sent-by-manoeuvre2writeimage4bpu: " + str(message2writesend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2write4bpu</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2writereceive4bpu-failed-by-manoeuvre2writeimage4bpu: " + str(message2writereceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4bpu == 3:    
        info2image4bpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4bpu-try-by-manoeuvre2writeimage4bpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4bpu = queue2writemanoeuvre4bpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4bpu-got-by-manoeuvre2writeimage4bpu: " + str(info2image4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4bpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2writeinfo4bpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2writeinfo4bpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2writemanoeuvre4bpu-failed-by-manoeuvre2writeimage4bpu: " + str(queue2writemanoeuvre4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2writesend4bpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2writeinfo4bpu is True:
                message2writesend4bpu = '<ack4xi>ack2imageinfo4bpu</ack4xi>'
            else:
                message2writesend4bpu = '<nack4xi>nack2imageinfo4bpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2writesend4bpu-set-by-manoeuvre2writeimage4bpu to: " + str(message2writesend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4bpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4bpu-failed-by-manoeuvre2writeimage4bpu: " + str(message2writesend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2writemanoeuvre4bpu-calling-by-manoeuvre2writeimage4bpu: " + str(publisherchannel2writemanoeuvre4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2writesend4bpu-passing-to-send-by-manoeuvre2writeimage4bpu: " + str(message2writesend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4bpu-failed-by-manoeuvre2writeimage4bpu: " + str(message2writesend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2writemanoeuvre4bpu.send(message2writesend4bpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2writesend4bpu-sent-by-manoeuvre2writeimage4bpu: " + str(message2writesend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4bpu</ack4xi>          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2writesend4bpu-failed-by-manoeuvre2writeimage4bpu: " + str(message2writesend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4write['ACK2INFO4BPU']        = ack2writeinfo4bpu
    response2manouvre4write['CODE2ACK4BPU']        = code2ack
    response2manouvre4write['CODE2NACK4BPU']       = code2nack
    response2manouvre4write['MESSAGE2GET4BPU']     = message2writeget4bpu
    response2manouvre4write['MESSAGE2PUT4BPU']     = message2writeput4bpu
    response2manouvre4write['MESSAGE2RECEIVE4BPU'] = message2writereceive4bpu
    response2manouvre4write['MESSAGE2SEND4BPU']    = message2writesend4bpu
    response2manouvre4write['STATE2MANOEUVRE4BPU'] = state2write4bpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4write-returning-by-manoeuvre2writeimage4bpu: " + str(response2manouvre4write) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1
    # {'MESSAGE2SEND4BPU': '<roger4xi>roger2write4bpu</roger4xi>', 
    #  'ACK2INFO4BPU': False, 'MESSAGE2GET4BPU': '<default4xi>default2write4bpu</default4xi>', 
    #  'MESSAGE2RECEIVE4BPU': '<copy4xi>copy2write4bpu</copy4xi>', 
    #  'CODE2ACK4BPU': ' 0D-1C-', 
    #  'MESSAGE2PUT4BPU': '<default4xi>default2write4bpu</default4xi>', 
    #  'CODE2NACK4BPU': ' ', 
    #  'STATE2MANOEUVRE4BPU': 1}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #2
    # {'MESSAGE2SEND4BPU': '<ready4xi>ready2write4bpu</ready4xi>', 
    #  'ACK2INFO4BPU': False, 'MESSAGE2GET4BPU': '<default4xi>default2write4bpu</default4xi>', 
    #  'MESSAGE2RECEIVE4BPU': '<ready4xi>ready2write4bpu</ready4xi>', 
    #  'CODE2ACK4BPU': ' 0D-2C-', 
    #  'MESSAGE2PUT4BPU': '<default4xi>default2write4bpu</default4xi>', 
    #  'CODE2NACK4BPU': ' ', 
    #  'STATE2MANOEUVRE4BPU': 2}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #3
    # {'MESSAGE2SEND4BPU': '<ack4xi>ack2imageinfo4bpu</ack4xi>', 
    #  'ACK2INFO4BPU': True, 'MESSAGE2GET4BPU': '<default4xi>default2write4bpu</default4xi>', 
    #  'MESSAGE2RECEIVE4BPU': '<ready4xi>ready2write4bpu</ready4xi>', 
    #  'CODE2ACK4BPU': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4BPU': '<default4xi>default2write4bpu</default4xi>', 
    #  'CODE2NACK4BPU': ' ', 
    #  'STATE2MANOEUVRE4BPU': 3}; time: 00:11:53 10/03/11 Pacific Daylight Time}

    return response2manouvre4write           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2mapimage4bpu" usage = "control function for map2image4bpu function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2mapimage4bpu(queue2mapmanoeuvre4bpu, consumerchannel2mapmanoeuvre4bpu, publisherchannel2mapmanoeuvre4bpu):
    '''
    manoeuvres control and command
    '''
    global state2map4bpu                                                       # STATE FOR COMMAND AND CONTROL
    global max2mapstate4bpu                                                    # MAX FOR STATE
    global ack2mapinfo4bpu                                                     # ACK FOR IMAGE INFO
    global message2mapget4bpu                                 
    global message2mapput4bpu
    global message2mapreceive4bpu
    global message2mapsend4bpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2mapimage4bpu" 
    method2print = "manoeuvre2mapimage4bpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2mapimage4bpu-started: " + str(manoeuvre2mapimage4bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    #  
    
    message2print = "{(Step 0B/33) passed" + "; queue2mapmanoeuvre4bpu-to-manoeuvre2mapimage4bpu: " + str(queue2mapmanoeuvre4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2mapmanoeuvre4bpu-to-manoeuvre2mapimage4bpu: " + str(consumerchannel2mapmanoeuvre4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2mapmanoeuvre4bpu-to-manoeuvre2mapimage4bpu: " + str(publisherchannel2mapmanoeuvre4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4map = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2map4bpu-try-by-manoeuvre2mapimage4bpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2map4bpu is None:
            state2map4bpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2map4bpu-initialized-by-manoeuvre2mapimage4bpu to: " + str(state2map4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2map4bpu-used-by-manoeuvre2mapimage4bpu: " + str(state2map4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2map4bpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2map4bpu-initialized-by-manoeuvre2mapimage4bpu to: " + str(state2map4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4bpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2mapinfo4bpu-initialized-by-manoeuvre2mapimage4bpu to: " + str(ack2mapinfo4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4bpu     = '<default4xi>default2map4bpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2mapget4bpu-initialized-by-manoeuvre2mapimage4bpu to: " + str(message2mapget4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4bpu</default4xi>
        message2mapput4bpu     = '<default4xi>default2map4bpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2mapput4bpu-initialized-by-manoeuvre2mapimage4bpu to: " + str(message2mapput4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4bpu</default4xi>
        message2mapreceive4bpu = '<default4xi>default2map4bpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2mapreceive4bpu-initialized-by-manoeuvre2mapimage4bpu to: " + str(message2mapreceive4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4bpu</default4xi>
        message2mapsend4bpu    = '<default4xi>default2map4bpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2mapsend4bpu-initialized-by-manoeuvre2mapimage4bpu to: " + str(message2mapsend4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4bpu</default4xi>
        max2mapstate4bpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2mapstate4bpu-initialized-by-manoeuvre2mapimage4bpu to: " + str( max2mapstate4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2map4bpu = state2map4bpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2map4bpu-added-by-manoeuvre2mapimage4bpu: " + str(state2map4bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2map4bpu > max2mapstate4bpu:
        state2map4bpu = 0
        message2print = "{(Step 0I/33) set" + "; state2map4bpu-set-by-manoeuvre2mapimage4bpu to: " + str(state2map4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4bpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2mapinfo4bpu-set-by-manoeuvre2mapimage4bpu to: " + str(ack2mapinfo4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4bpu     = '<default4xi>default2map4bpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2mapget4bpu-set-by-manoeuvre2mapimage4bpu to: " + str(message2mapget4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4bpu</default4xi>
        message2mapput4bpu     = '<default4xi>default2map4bpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2mapput4bpu-set-by-manoeuvre2mapimage4bpu to: " + str(message2mapput4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4bpu</default4xi>
        message2mapreceive4bpu = '<default4xi>default2map4bpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2mapreceive4bpu-set-by-manoeuvre2mapimage4bpu to: " + str(message2mapreceive4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4bpu</default4xi>
        message2mapsend4bpu    = '<default4xi>default2map4bpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2mapsend4bpu-set-by-manoeuvre2mapimage4bpu to: " + str(message2mapsend4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4bpu</default4xi>
        max2mapstate4bpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2mapstate4bpu-set-by-manoeuvre2mapimage4bpu to: " + str( max2mapstate4bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4bpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2mapimage4bpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4bpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4bpu = consumerchannel2mapmanoeuvre4bpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2mapreceive4bpu-received-by-manoeuvre2mapimage4bpu: " + str(message2mapreceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4bpu == '<copy4xi>copy2mapimage4bpu</copy4xi>':
                message2mapsend4bpu = '<roger4xi>roger2mapimage4bpu</roger4xi>'
            else:
                message2mapsend4bpu = '<negative4xi><negative2mapimage4bpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4bpu.send(message2mapsend4bpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2mapsend4bpu-sent-by-manoeuvre2mapimage4bpu: " + str(message2mapsend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2mapreceive4bpu-failed-by-manoeuvre2mapimage4bpu: " + str(message2mapreceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4bpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2mapimage4bpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4bpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4bpu = consumerchannel2mapmanoeuvre4bpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2mapreceive4bpu-received-by-manoeuvre2mapimage4bpu: " + str(message2mapreceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4bpu == '<ready4xi>ready2mapimage4bpu</ready4xi>':
                message2mapsend4bpu = '<ready4xi>ready2mapimage4bpu</ready4xi>'
            else:
                message2mapsend4bpu = '<negative4xi><negative2ready4bpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4bpu.send(message2mapsend4bpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2mapsend4bpu-sent-by-manoeuvre2mapimage4bpu: " + str(message2mapsend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2mapreceive4bpu-failed-by-manoeuvre2mapimage4bpu: " + str(message2mapreceive4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4bpu == 3:    
        info2image4bpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4bpu-try-by-manoeuvre2mapimage4bpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4bpu = queue2mapmanoeuvre4bpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4bpu-got-by-manoeuvre2mapimage4bpu: " + str(info2image4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info2image4bpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2mapinfo4bpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2mapinfo4bpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2mapmanoeuvre4bpu-failed-by-manoeuvre2mapimage4bpu: " + str(queue2mapmanoeuvre4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2mapsend4bpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2mapinfo4bpu is True:
                message2mapsend4bpu = '<ack4xi>ack2imageinfo4bpu</ack4xi>'
            else:
                message2mapsend4bpu = '<nack4xi>nack2imageinfo4bpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2mapsend4bpu-set-by-manoeuvre2mapimage4bpu to: " + str(message2mapsend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #      
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4bpu-failed-by-manoeuvre2mapimage4bpu: " + str(message2mapsend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2mapmanoeuvre4bpu-calling-by-manoeuvre2mapimage4bpu: " + str(publisherchannel2mapmanoeuvre4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message2mapsend4bpu-passing-to-send-by-manoeuvre2mapimage4bpu: " + str(message2mapsend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4bpu-failed-by-manoeuvre2mapimage4bpu: " + str(message2mapsend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2mapmanoeuvre4bpu.send(message2mapsend4bpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2mapsend4bpu-sent-by-manoeuvre2mapimage4bpu: " + str(message2mapsend4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2mapsend4bpu-failed-by-manoeuvre2mapimage4bpu: " + str(message2mapsend4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4map['ACK2INFO4BPU']        = ack2mapinfo4bpu
    response2manouvre4map['CODE2ACK4BPU']        = code2ack
    response2manouvre4map['CODE2NACK4BPU']       = code2nack
    response2manouvre4map['MESSAGE2GET4BPU']     = message2mapget4bpu
    response2manouvre4map['MESSAGE2PUT4BPU']     = message2mapput4bpu
    response2manouvre4map['MESSAGE2RECEIVE4BPU'] = message2mapreceive4bpu
    response2manouvre4map['MESSAGE2SEND4BPU']    = message2mapsend4bpu
    response2manouvre4map['STATE2MANOEUVRE4BPU'] = state2map4bpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4map-returning-by-manoeuvre2mapimage4bpu: " + str(response2manouvre4map) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4BPU': '<roger4xi>roger2mapimage4bpu</roger4xi>', 
    #  'ACK2INFO4BPU': False, 
    #  'MESSAGE2GET4BPU': '<default4xi>default2map4bpu</default4xi>', 
    #  'MESSAGE2RECEIVE4BPU': '<copy4xi>copy2mapimage4bpu</copy4xi>', 
    #  'CODE2ACK4BPU': ' 0D-1C-', 
    #  'MESSAGE2PUT4BPU': '<default4xi>default2map4bpu</default4xi>', 
    #  'CODE2NACK4BPU': ' ', 
    #  'STATE2MANOEUVRE4BPU': 1}
     
    #2 {'MESSAGE2SEND4BPU': '<ready4xi>ready2mapimage4bpu</ready4xi>', 
    #  'ACK2INFO4BPU': False, 
    #  'MESSAGE2GET4BPU': '<default4xi>default2map4bpu</default4xi>', 
    #  'MESSAGE2RECEIVE4BPU': '<ready4xi>ready2mapimage4bpu</ready4xi>', 
    #  'CODE2ACK4BPU': ' 0D-2C-', 
    #  'MESSAGE2PUT4BPU': '<default4xi>default2map4bpu</default4xi>', 
    #  'CODE2NACK4BPU': ' ', 
    #  'STATE2MANOEUVRE4BPU': 2}; time: 06:49:48 10/04/11 Pacific Daylight Time}
     
    #3 {'MESSAGE2SEND4BPU': '<ack4xi>ack2imageinfo4bpu</ack4xi>', 
    #  'ACK2INFO4BPU': True, 
    #  'MESSAGE2GET4BPU': '<default4xi>default2map4bpu</default4xi>', 
    #  'MESSAGE2RECEIVE4BPU': '<ready4xi>ready2mapimage4bpu</ready4xi>', 
    #  'CODE2ACK4BPU': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4BPU': '<default4xi>default2map4bpu</default4xi>', 
    #  'CODE2NACK4BPU': ' ', 
    #  'STATE2MANOEUVRE4BPU': 3}; time: 06:49:48 10/04/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4BPU': '<default4xi>default2map4bpu</default4xi>', 
    #  'ACK2INFO4BPU': False, 
    #  'MESSAGE2GET4BPU': '<default4xi>default2map4bpu</default4xi>', 
    #  'MESSAGE2RECEIVE4BPU': '<default4xi>default2map4bpu</default4xi>', 
    #  'CODE2ACK4BPU': ' 0D-', 
    #  'MESSAGE2PUT4BPU': '<default4xi>default2map4bpu</default4xi>', 
    #  'CODE2NACK4BPU': ' ', 'STATE2MANOEUVRE4BPU': 0}; time: 06:49:48 10/04/11 Pacific Daylight Time}

    return response2manouvre4map           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "write2image4bpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4BPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF BPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4BPU ', time4xi())    
#   respon4write2image = {} 
#   task4write2image = None
#   try:                               
#       response4write2image = write2image4bpu(task4write2image)                                                                                                          
#       print('(Step 16B/700) OF BPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4BPU : ', response4write2image)
#       queue2write4bpu      = response4write2image['QUEUE']
#       channel2publish4bpu = response4write2image['PUBLISHERCHANNEL']
#       channel2consume4bpu = response4write2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4BPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4BPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF BPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4BPU ', time4xi())     
#   try:                               
#       message2writesend4bpu = "<copy4xi>copy2write4bpu</copy4xi>"
#       channel2publish4bpu = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4bpu.send(message2writesend4bpu)
#       print("(Step 17B/700) OF BPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4BPU: ", channel2publish4bpu, " MESSAGE2SEND4BPU: " , message2writesend4bpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF BPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4BPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4BPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi()) 
#   try:        
#       response2manoeuvre4bpu = manoeuvre2readimage4bpu(queue2write4bpu, channel2consume4bpu, channel2publish4bpu)
#       print("(Step 18D/700) OF BPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", manoeuvre2readimage4bpu)
#   except:
#       print("(Step 18Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4BPU ', time4xi())     
#   try:
#       channel2consume4bpu = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4bpu = channel2consume4bpu.recv()
#       print("(Step 19B/700) OF BPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4BPU: ", channel2consume4bpu, " CONTROL & COMMAND: ", message2writereceive4bpu)                                                                                                  
#       # <roger4xi>roger2write4bpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4BPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4BPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF BPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4BPU ', time4xi())     
#   try:                               
#       message2writesend4bpu = "<write4xi>write2write4bpu</write4xi>"
#       channel2publish4bpu = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4bpu.send(message2writesend4bpu)
#       print("(Step 20B/700) OF BPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4BPU: ", channel2publish4bpu, " MESSAGE2SEND4BPU: " , message2writesend4bpu)                                                                                                
#       # <write4xi>write2write4bpu</write4xi> 
#   except:
#        print('(Step 20Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4BPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4BPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi()) 
#   try:        
#       response2manoeuvre4bpu = manoeuvre2readimage4bpu(queue2write4bpu, channel2consume4bpu, channel2publish4bpu)
#       print("(Step 21B/700) OF BPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", manoeuvre2readimage4bpu)
#   except:
#       print("(Step 21Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4BPU ', time4xi())     
#   try:
#       channel2consume4bpu = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4bpu = channel2consume4bpu.recv()
#       print("(Step 22B/700) OF BPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4BPU: ", channel2consume4bpu, " CONTROL & COMMAND: ", message2writereceive4bpu)                                                                                                  
#       # <write4xi>write2write4bpu</write4xi> 
#   except:
#       print('(Step 22Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4BPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2write4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4BPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi()) 
#   try:        
#       response2manoeuvre4bpu = manoeuvre2readimage4bpu(queue2write4bpu, channel2consume4bpu, channel2publish4bpu)
#       print("(Step 25B/700) OF BPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", manoeuvre2readimage4bpu)
#   except:
#       print("(Step 25Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2writereceive4bpu = channel2consume4bpu.recv()
#       print("(Step 26D/700) OF BPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4BPU: ", channel2consume4bpu, " CONTROL & COMMAND: ", message2writereceive4bpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4bpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4BPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF BPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi()) 
#   try:        
#       response2manoeuvre4bpu = manoeuvre2readimage4bpu(queue2write4bpu, channel2consume4bpu, channel2publish4bpu)
#       print("(Step 27B/700) OF BPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4BPU ", response2manoeuvre4bpu)
#   except:
#       print("(Step 27Z/700) OF BPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4BPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def write2image4bpu(task4imagewrite2bpu=None):

    '''
    writes 2D images for BPU
    '''
    global bpu2image4xi                                                     # 2D-IMAGES FOR BPU  
    global control2writeimage4bpu                                              # COMMAND AND CONTROL FOR BPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS        
    # -----------------------------------------------------------------------------------------          
    class2print = "write2image4bpu"
    method2print = "write2image4bpu"

    message2print = "{(Step 1A/33) started" + "; write2image4bpu-started: " + str(write2image4bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function write2image4bpu at 0x00000000038E8EC8>
    
    message2print = "{(Step 1B/33) passed" + "; task4imagewrite2bpu-to-write2image4bpu: " + str(task4imagewrite2bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE BPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if bpu2image4xi is None:
            bpu2image4xi = create2image4bpu()                                                                              # INITIALIZE IMAGE FOR BPU 
            message2print = "{(Step 1C/70) created" + "; bpu2image4xi-created-by-write2image4bpu to: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4bpu-skipped-by-write2image4bpu: " + str(write2image4bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4bpu-skipped-by-write2image4bpu: <function write2image4bpu at 0x00000000038E8EC8>                                 
    except:
        try:
            bpu2image4xi = create2image4bpu()
            message2print = "{(Step 1E/70) set" + "; bpu2image4xi-set-by-write2image4bpu to: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            bpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; bpu2image4xi-yielded-by-write2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagewrite2bpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagewrite2bpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagewrite2bpu-created-by-write2image4bpu to: " + str(task4imagewrite2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4bpu = trace2image4bpu                                                                 # CALLBACK FUNCTION FOR BPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4bpu-created-by-write2image4bpu to: " + str(function2callback4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4bpu at 0x00000000038E2A48>
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4bpu"                                                            # MODULE OF BPUIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4bpu"                                                            # FUNCTION OF BPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4bpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR BPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-write2image4bpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; write2image4bpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'write2image4bpu', 
            #  'METHOD': 'write2image4bpu'}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2bpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO BPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2bpu['FUNCTION2CALLBACK'] = function2callback4bpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagewrite2bpu-created-by-write2image4bpu to: " + str(task4imagewrite2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4bpu at 0x00000000038E2A48>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; write2image4bpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}',
            #  'CLASS': 'write2image4bpu',
            #  'METHOD': 'write2image4bpu'}}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-write2image4bpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-write2image4bpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 02:32:29 10/05/11 Pacific Daylight Time}

                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-write2image4bpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagewrite2bpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2bpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2bpu-trying-by-write2image4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2bpu = get2context4bpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2bpu-got-by-write2image4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4bpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4bpu', ...
   
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2bpu-failed-by-write2image4bpu: " + str(context2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagewrite2bpu ['CONTEXT'] = context2bpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2bpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2bpu-trying-by-write2image4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2bpu = get2bpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2bpu-got-by-write2image4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ...
               
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2bpu-failed-by-write2image4bpu: " + str(device2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagewrite2bpu ['DEVICE'] = device2bpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2bpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2bpu-trying-by-write2image4bpu: " + str(imageinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2bpu = get2imageinfo4bpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2bpu-got-by-write2image4bpu: " + str(imageinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004494E10>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4bpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4bpu', 
                #  'METHOD': 'create2context4bpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4bpu at 0x000000000387E348>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
                #  'COUNT2REFERENCE': 1, 
                #  'FUNCTION2CALLBACK': <function trace2image4bpu at 0x00000000038E2A48>, 
                #  'COMMANDQUEUE': ...

                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2bpu-failed-by-write2image4bpu: " + str(imageinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagewrite2bpu ['IMAGE'] = imageinfo2bpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY BPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2bpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2bpu-trying-by-write2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2bpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2bpu-got-by-write2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003D4FA90>
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2bpu-failed-by-write2image4bpu: " + str(queue2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagewrite2bpu ['QUEUE'] = queue2bpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY BPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2bpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2bpu-trying-by-write2image4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2bpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2bpu-got-by-write2image4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 568>, <read-write PipeConnection, handle 572>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2bpu-failed-by-write2image4bpu: " + str(pipe2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagewrite2bpu ['PIPE'] = pipe2bpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2bpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2bpu-trying-by-write2image4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2bpu = get2commandqueueinfo4bpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2bpu-got-by-write2image4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4bpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4bpu', 
                #  'METHOD': 'create2commandqueue4bpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4bpu at 0x000000000388DA48> ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2bpu-failed-by-write2image4bpu: " + str(commandqueueinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagewrite2bpu ['COMMANDQUEUE'] = commandqueueinfo2bpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY BPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2bpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2bpu-trying-by-write2image4bpu: " + str(bufferinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2bpu = get2bufferinfo4bpu()                                                               # SYSTEM CREATES BUFFER FOR BPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2bpu-got-by-write2image4bpu: " + str(bufferinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4bpu-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2buffer4bpu', 
                #  'METHOD': 'create2buffer4bpu'}, ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2bpu-failed-by-write2image4bpu: " + str(bufferinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagewrite2bpu ['BUFFER'] = bufferinfo2bpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2bpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2bpu-trying-by-write2image4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2bpu = get2eventinfo4bpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2bpu-got-by-write2image4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  {'CODE2ERROR': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4bpu-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2event4bpu', 
                #  'METHOD': 'create2event4bpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2event4bpu at 0x00000000038A3F48>, ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2bpu-failed-by-write2image4bpu: " + str(eventinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagewrite2bpu ['EVENT'] = eventinfo2bpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2bpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2bpu-trying-by-write2image4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2bpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2bpu-set-by-write2image4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2bpu-failed-by-write2image4bpu: " + str(eventwaitlist2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagewrite2bpu ['EVENTWAITLIST'] = eventwaitlist2bpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2bpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2bpu-trying-by-write2image4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2bpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2bpu = eventwaitlistquantity2bpu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2bpu-set-by-write2image4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2bpu-failed-by-write2image4bpu: " + str(eventwaitlistquantity2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagewrite2bpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2bpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2bpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2bpu-trying-by-write2image4bpu: " + str(blocking2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2bpu = 'CL_FALSE'                                                                           # the write command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2bpu-set-by-write2image4bpu to: " + str(blocking2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE 
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2bpu-failed-by-write2image4bpu: " + str(blocking2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagewrite2bpu ['BLOCKING'] = blocking2bpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2bpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2bpu-trying-by-write2image4bpu: " + str(origin2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2bpu['X'] = 0 
                origin2bpu['Y'] = 0  
                origin2bpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2bpu-set-by-write2image4bpu to: " + str(origin2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2bpu-failed-by-write2image4bpu: " + str(origin2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagewrite2bpu ['ORIGIN'] = origin2bpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2bpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2bpu-trying-by-write2image4bpu: " + str(region2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2bpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2bpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2bpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2bpu-set-by-write2image4bpu to: " + str(region2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2bpu-failed-by-write2image4bpu: " + str(region2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagewrite2bpu ['REGION'] = region2bpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.

            height2bpu = 800
            width2bpu = 800
            rowpitch2bpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2bpu-trying-by-write2image4bpu: " + str(rowpitch2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #  
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2bpu = int(width2bpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2bpu-set-by-write2image4bpu to: " + str(rowpitch2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                            
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2bpu-failed-by-write2image4bpu: " + str(rowpitch2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagewrite2bpu ['ROWPITCH'] = rowpitch2bpu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2bpu = None
            size2bpu = int(width2bpu * height2bpu)                                                                  # CALCULATED MEMORY
            if size2bpu == 0:
                size2bpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2bpu-trying-by-write2image4bpu: " + str(pointer2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2bpu = Arena4Xi(size2bpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2bpu-got-by-write2image4bpu: " + str(arena2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041348D0>
                
                size2arena = arena2bpu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-write2buffer4bpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2bpu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-write2buffer4bpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x000000000421A450> 
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2bpu:
                    size2bpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2bpu-set-by-write2buffer4bpu to: " + str(size2bpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2bpu = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2bpu-set-by-write2buffer4bpu to: " + str(pointer2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x000000000421A450>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2bpu-failed-by-write2image4bpu: " + str(pointer2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagewrite2bpu ['POINTER'] = pointer2bpu                            
            task4imagewrite2bpu ['SIZE']    = size2bpu                                                               # AUTOMATIC BY BPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2bpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2bpu-trying-by-write2image4bpu: " + str(slicepitch2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2bpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2bpu-set-by-write2image4bpu to: " + str(slicepitch2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2bpu-failed-by-write2image4bpu: " + str(slicepitch2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagewrite2bpu ['SLICEPITCH'] = slicepitch2bpu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2bpu.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-write2image4bpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4bpu = trace2image4bpu                                                             # CALLBACK FUNCTION FOR BPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4bpu-created-by-write2image4bpu to: " + str(function2callback4bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4bpu"                                                        # MODULE OF BPUIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4bpu"                                                        # FUNCTION OF BPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4bpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR BPUIZED OPENCL TO RETURN    
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-write2image4bpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2bpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO BPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2bpu['FUNCTION2CALLBACK'] = function2callback4bpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagewrite2bpu-created-by-write2image4bpu to: " + str(task4imagewrite2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-write2image4bpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-write2image4bpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                     
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-write2image4bpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                     
            task4imagewrite2bpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
              
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2bpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2bpu-trying-by-write2image4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2bpu = get2context4bpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2bpu-got-by-write2image4bpu: " + str(context2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2bpu-failed-by-write2image4bpu: " + str(context2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagewrite2bpu ['CONTEXT'] = context2bpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2bpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2bpu-trying-by-write2image4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2bpu = get2bpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2bpu-got-by-write2image4bpu: " + str(device2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2bpu-failed-by-write2image4bpu: " + str(device2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagewrite2bpu ['DEVICE'] = device2bpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2bpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2bpu-trying-by-write2image4bpu: " + str(imageinfo2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2bpu = get2imageinfo4bpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2bpu-got-by-write2image4bpu: " + str(imageinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2bpu-failed-by-write2image4bpu: " + str(imageinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagewrite2bpu ['IMAGE'] = imageinfo2bpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY BPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2bpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2bpu-trying-by-write2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2bpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2bpu-got-by-write2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2bpu-failed-by-write2image4bpu: " + str(queue2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagewrite2bpu ['QUEUE'] = queue2bpu

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY BPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2bpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2bpu-trying-by-write2image4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2bpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2bpu-got-by-write2image4bpu: " + str(pipe2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2bpu-failed-by-write2image4bpu: " + str(pipe2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagewrite2bpu ['PIPE'] = pipe2bpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2bpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2bpu-trying-by-write2image4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2bpu = get2commandqueueinfo4bpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2bpu-got-by-write2image4bpu: " + str(commandqueueinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2bpu-failed-by-write2image4bpu: " + str(commandqueueinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagewrite2bpu ['COMMANDQUEUE'] = commandqueueinfo2bpu
            
            
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY BPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2bpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2bpu-trying-by-write2image4bpu: " + str(bufferinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2bpu = get2bufferinfo4bpu()                                                           # SYSTEM CREATES BUFFER FOR BPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2bpu-got-by-write2image4bpu: " + str(bufferinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2bpu-failed-by-write2image4bpu: " + str(bufferinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagewrite2bpu ['BUFFER'] = bufferinfo2bpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2bpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2bpu-trying-by-write2image4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2bpu = get2eventinfo4bpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2bpu-got-by-write2image4bpu: " + str(eventinfo2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2bpu-failed-by-write2image4bpu: " + str(eventinfo2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagewrite2bpu ['EVENT'] = eventinfo2bpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2bpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2bpu-trying-by-write2image4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2bpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2bpu-set-by-write2image4bpu: " + str(eventwaitlist2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2bpu-failed-by-write2image4bpu: " + str(eventwaitlist2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagewrite2bpu ['EVENTWAITLIST'] = eventwaitlist2bpu                     

                 
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2bpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2bpu-trying-by-write2image4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2bpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2bpu = eventwaitlistquantity2bpu + 1                                                                     
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2bpu-set-by-write2image4bpu: " + str(eventwaitlistquantity2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2bpu-failed-by-write2image4bpu: " + str(eventwaitlistquantity2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagewrite2bpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2bpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2bpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2bpu-trying-by-write2image4bpu: " + str(blocking2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2bpu = 'CL_FALSE'                                                                       # the write command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2bpu-set-by-write2image4bpu to: " + str(blocking2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2bpu-failed-by-write2image4bpu: " + str(blocking2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagewrite2bpu ['BLOCKING'] = blocking2bpu             
                 
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2bpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2bpu-trying-by-write2image4bpu: " + str(origin2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2bpu['X'] = 0 
                origin2bpu['Y'] = 0  
                origin2bpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2bpu-set-by-write2image4bpu to: " + str(origin2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2bpu-failed-by-write2image4bpu: " + str(origin2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagewrite2bpu ['ORIGIN'] = origin2bpu
                
                 
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2bpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2bpu-trying-by-write2image4bpu: " + str(region2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2bpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2bpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2bpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2bpu-set-by-write2image4bpu to: " + str(region2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2bpu-failed-by-write2image4bpu: " + str(region2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagewrite2bpu ['REGION'] = region2bpu
            
                
                 
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2bpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2bpu-trying-by-write2image4bpu: " + str(rowpitch2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2bpu = int(width2bpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2bpu-set-by-write2image4bpu to: " + str(rowpitch2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2bpu-failed-by-write2image4bpu: " + str(rowpitch2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagewrite2bpu ['ROWPITCH'] = rowpitch2bpu     
                 
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2bpu = None
            size2bpu = int(width2bpu * height2bpu)                                                                  # CALCULATED MEMORY
            if size2bpu == 0:
                size2bpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2bpu-trying-by-write2image4bpu: " + str(pointer2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2bpu = Arena4Xi(size2bpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2bpu-got-by-write2image4bpu: " + str(arena2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2bpu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-write2buffer4bpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2bpu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-write2buffer4bpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2bpu:
                    size2bpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2bpu-set-by-write2buffer4bpu to: " + str(size2bpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2bpu = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2bpu-set-by-write2buffer4bpu to: " + str(pointer2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2bpu-failed-by-write2image4bpu: " + str(pointer2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imagewrite2bpu ['POINTER'] = pointer2bpu                            
            task4imagewrite2bpu ['SIZE']    = size2bpu                                                               # AUTOMATIC BY BPUXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2bpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2bpu-trying-by-write2image4bpu: " + str(slicepitch2bpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2bpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2bpu-set-by-write2image4bpu to: " + str(slicepitch2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2bpu-failed-by-write2image4bpu: " + str(slicepitch2bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagewrite2bpu ['SLICEPITCH'] = slicepitch2bpu                  

        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2bpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-write2image4bpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}
             
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagewrite2bpu-yielded-by-write2image4bpu: " + str(task4imagewrite2bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2writeimage4bpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2bpu = False
    channels2bpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2bpu-trying-by-write2image4bpu: " + str(channels2bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2bpu, publisherchannel2bpu = task4imagewrite2bpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2bpu-got-by-write2image4bpu: " + str(consumerchannel2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 568>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2bpu-got-by-write2image4bpu: " + str(publisherchannel2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 572>                     
        ack4channels2bpu = True
        
        control2writeimage4bpu['CONSUMERCHANNEL']  = consumerchannel2bpu
        
        
        control2writeimage4bpu['PUBLISHERCHANNEL'] = publisherchannel2bpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2bpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2bpu = False
    queue2bpu = None
    message2print = "{(Step 41A/70) trying" + "queue2bpu-trying-by-write2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2bpu = task4imagewrite2bpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2bpu-got-by-write2image4bpu: " + str(queue2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003C83A90> 
              
        ack4queue2bpu = True
        control2writeimage4bpu['QUEUE']  = queue2bpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2bpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4bpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4bpu-trying-by-write2image4bpu: " + str(dummy2image4bpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4bpu = DummyProcess4Xi(target=manoeuvre2readimage4bpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2bpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2bpu, publisherchannel2bpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4bpu-created-by-write2image4bpu: " + str(dummy2image4bpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-write2image4bpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-write2image4bpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2writeimage4bpu-returning-by-write2image4bpu: " + str(control2writeimage4bpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003C83A90>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 572>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 568>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

    return control2writeimage4bpu
                    
#</function4xi> 


 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "release2image4bpu" usage = "
#
#   task4release2image  = {} 
#   task4release2image  ['COUNT2REFERENCE'] = 1 
#   response4release2image = {}
#    
#   print('(Step 12A/33) OF BPU2IMAGE4XI5TEST - TRYING TO RELEASE IMAGE OF BPU DEVICE ', time4xi())   
#   try:                               
#       response4release2image = release2image4bpu(task4release2image)                                                                                             
#       print('(Step 12B/33) OF BPU2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4release2image)                           
#   except:
#       print('(Step 12Z1/33) OF BPU2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF BPU DEVICE ', time4xi())  
#   
#                 
#">

# -------------------------------------------------------------------------------------------------   
def release2image4bpu(task4release2bpu=None):

    '''
    releases bpu2image4xi
    '''
    global bpu2image4xi  
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "release2image4bpu"
    method2print = "release2image4bpu" 

    message2print = "{(Step 1A/33) started" + "; release2image4bpu-started: " + str(release2image4bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function release2image4bpu at 0x0000000003A9E8C8>
    
    message2print = "{(Step 1B/33) passed" + "; task4release2bpu-to-release2image4bpu: " + str(task4release2bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1} 

             
    code2ack  = ' '
    code2nack = ' '

    try:
        if bpu2image4xi is None:
            bpu2image4xi = create2image4bpu()                                                                       # INITIALIZE IMAGE                 
    except:
        try:
            bpu2image4xi = create2image4bpu()
        except:
            bpu2image4xi = {}
            message2print = "{(Step 2Z0/70) yielded" + "; bpu2image4xi-yielded-by-release2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '2Z0-' 
                
    message2print = "{(Step 1A/70) try" + "; release-try-by-release2image4bpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    if task4release2bpu is None:
        task4release2bpu = {}
        task4release2bpu['COUNT2REFERENCE'] = 1
           
    try:
        # -----------------------------------------------------------------------------------------
        # RELEASE
        # -----------------------------------------------------------------------------------------               
        bpu2image4xi['COUNT2REFERENCE'] = bpu2image4xi['COUNT2REFERENCE'] - task4release2bpu['COUNT2REFERENCE']
        
        message2print = "{(Step 1A/70) calculated" + "; count2reference-calculated-by-release2image4bpu: " + str(bpu2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
         
        if bpu2image4xi['COUNT2REFERENCE'] < 0:
            bpu2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 1B/70) warning" + "; count2reference-warning-by-release2image4bpu: " + " ** " + str(bpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            # { release2image4bpu : release2image4bpu ; (Step 8Z1/70) warning; count2reference-warning-by-release2image4bpu:  ** 0 ** ; time: 23:41:37 09/06/11 Pacific Daylight Time}    
            code2nack  = code2nack + '1B-'
        else:
            code2ack  = code2ack + '1B-'
    except:
        try:
            bpu2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 8Z1/70) warning" + "; count2reference-warning-by-release2image4bpu: " + " ** " + str(bpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            code2nack  = code2nack + '8Z1-'
            
        except:
            message2print = "{(Step 8Z2/70) yielded" + "; bpu2image4xi-yielded-by-release2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '8Z2-'
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    bpu2image4xi ['ACK2RETURN']  = code2ack
    bpu2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; bpu2image4xi-returning-by-release2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0,
    #  'ACK2RETURN': ' ',
    #  'ROWPITCH': 800,
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4bpu-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'create2image4bpu',
    #  'METHOD': 'create2image4bpu'},
    #  'WIDTH': 800,
    #  'BUFFER': {'CODE2ERROR': 0, 
    #  'COUNT2REFERENCE': 0,
    
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4bpu-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}', 'CLASS': 'create2buffer4bpu', 'METHOD': 'create2buffer4bpu'},
    #  'FUNCTION2CALLBACK': <function trace2buffer4bpu at 0x0000000003A97348>,
    #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CDE298>,
    #  'FLAGS': 0, 'CONTEXT': {'COUNT2REFERENCE': 0, ...
    
    
    return bpu2image4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "retain2image4bpu" usage = "
#   task4retain2image  = {}
#   task4retain2image  ['DIMENSION'] = 2 
#   task4retain2image  ['COUNT2REFERENCE'] = 1 
#   response4retain2Dimage = {}
#   print('(Step 13A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF BPU DEVICE ', time4xi())  
#   try:                               
#       response4retain2Dimage = retain2image4bpu(task4retain2image)                                                                                                 
#       print('(Step 13B/700) OF BPU2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', response4retain2Dimage)                          
#       # 
#   except:
#       print('(Step 13Z1/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF BPU DEVICE ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def retain2image4bpu(task4retain2bpu=None):

    '''
    retains either bpu2image4xi
    '''
    global bpu2image4xi 
    
     
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "retain2image4bpu"
    method2print = "retain2image4bpu"

    message2print = "{(Step 1A/33) started" + "; retain2image4bpu-started: " + str(retain2image4bpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function retain2image4bpu at 0x0000000003A9E948>   
    
    message2print = "{(Step 1B/33) passed" + "; task4retain2bpu-to-retain2image4bpu: " + str(task4retain2bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 2}
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 3}
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # (Step 2/70) - PROVIDE TASKS, IF NECESSARY AND CHECK GIVEN TASKS 
    # -----------------------------------------------------------------------------------------    
    if task4retain2bpu is None:
        task4retain2bpu = {}
        task4retain2bpu['DIMENSION'] = 2
        task4retain2bpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 2A/33) set" + "; task4retain2bpu-set-by-retain2image4bpu to: " + str(task4retain2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    else:
        pass
        
    if 'DIMENSION' in task4retain2bpu.keys():
        if task4retain2bpu['DIMENSION'] == 2:
            pass
        else:
            task4retain2bpu['DIMENSION'] = 2 
            message2print = "{(Step 2B/33) set" + "; dimension-set-by-retain2image4bpu to: " + str(task4retain2bpu['DIMENSION'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                       
    else:
        task4retain2bpu['DIMENSION'] = 2
    
    # -----------------------------------------------------------------------------------------
    # (Step 3/70) - CHECK COUNT2REFERENCE
    # -----------------------------------------------------------------------------------------         
    if 'COUNT2REFERENCE' in task4retain2bpu.keys():
        pass
    else:
        task4retain2bpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3A/33) set" + "; count2reference-set-by-retain2image4bpu to: " + str(task4retain2bpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                                   
    if task4retain2bpu['COUNT2REFERENCE'] < 1:
        task4retain2bpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3B/33) set" + "; count2reference-set-by-retain2image4bpu to: " + str(task4retain2bpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
        
    # -----------------------------------------------------------------------------------------
    # (Step 4/70) - INITIALIZE 2D IMAGE IF NECESSARY
    # -----------------------------------------------------------------------------------------
    try:
        if bpu2image4xi is None:
            bpu2image4xi = create2image4bpu()
            message2print = "{(Step 4A/70) created" + "; bpu2image4xi-created-by-retain2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                                                                          
    except:
        try:
            bpu2image4xi = create2image4bpu()
            message2print = "{(Step 4B/70) created" + "; bpu2image4xi-created-by-retain2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
        except:
            bpu2image4xi = {}
            message2print = "{(Step 4Z/70) yielded" + "; bpu2image4xi-yielded-by-retain2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '4Z-' 


    # -----------------------------------------------------------------------------------------
    # (Step 5/70) - RETAIN
    # -----------------------------------------------------------------------------------------             
    try:
        bpu2image4xi['COUNT2REFERENCE'] = bpu2image4xi['COUNT2REFERENCE'] + task4retain2bpu['COUNT2REFERENCE']
        message2print = "{(Step 6A/70) calculated" + "; count2reference-calculated-by-retain2image4bpu: " + str(bpu2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #              
        if bpu2image4xi['COUNT2REFERENCE'] <= 0:
            bpu2image4xi['COUNT2REFERENCE'] = 1
            message2print = "{(Step 6Z1/70) warning" + "; count2reference-warning-by-retain2image4bpu: " + " ** " + str(bpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            code2nack  = code2nack + '6Z1-'
        else:
            code2ack  = code2ack + '6C-'

    except:
        try:
            bpu2image4xi['COUNT2REFERENCE'] = 1
        
            message2print = "{(Step 7Z1/70) warning" + "; count2reference-warning-by-retain2image4bpu: " + " ** " + str(bpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            code2nack  = code2nack + '7Z1-'
        
        except:
            message2print = "{(Step 7Z2/70) yielded" + "; bpu2image4xi-yielded-by-retain2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '7Z2-'
        
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70) - RETURN           
    # -----------------------------------------------------------------------------------------
    
    bpu2image4xi ['ACK2RETURN']  = code2ack
    bpu2image4xi ['NACK2RETURN'] = code2nack     
    message2print = "{(Step 70A/70) returning" + "; bpu2image4xi-returning-by-retain2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 6C-', 'ROWPITCH': 800, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4bpu-failed ; time: 23:25:02 10/05/11 Pacific Daylight Time}', 'CLASS': 'create2image4bpu', 'METHOD': 'create2image4bpu'}, 'WIDTH': 800, 'BUFFER': {'CODE2ERROR': 0, 
    #                   'COUNT2REFERENCE': 0, ...
    # 'COUNT2REFERENCE': 1, 
    # 'FLAGS': 0, 'CONTEXT': ...
    
    return bpu2image4xi

#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2imageinfo4bpu" usage = "
#   #1
#   print('(Step 3A/33) OF BPU2IMAGE4XI5TEST - TRYING TO GET 2D IMAGE INFO FOR BPU', time4xi())  
#   try:
#       imageinfo2bpu = get2imageinfo4bpu()                                                                       
#       print('(Step 3B/33) OF BPU2IMAGE4XI5TEST - GOT 2D IMAGE INFO FOR BPU: ', imageinfo2bpu) 
#   except:
#       print('(Step 3Z/33) OF BPU2IMAGE4XI5TEST *** FAILED TO GET 2D IMAGE INFO FOR BPU', time4xi())
#   #2
#   task4getinfo2image = {}
#   task4getinfo2image ['DIMENSION'] = 2
#   task4getinfo2image ['WIDTH'] = 0
#   task4getinfo2image ['HEIGHT'] = 0
#   response4getinfo2image = {}
#   print('(Step 12C/33) OF BPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 3D IMAGE INFO OF BPU DEVICE ', time4xi())    
# 
#   try:                               
#       response4getinfo2image = get2imageinfo4bpu (task4getinfo2image)                                                                                             
#       print('(Step 12D/33) OF BPU2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4getinfo2image)                           
#       #
#   except:
#       print('(Step 12Z2/33) OF BPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 3D IMAGE INFO OF BPU DEVICE ', time4xi()) 
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2imageinfo4bpu(task4getinfo2image=None):

    '''
    gets bpu2image4xi
    '''
    
    global bpu2image4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2imageinfo4bpu"
    method2print = "get2imageinfo4bpu"

    message2print = "{(Step 0A/33) started" + "; get2imageinfo4bpu-started: " + str(get2imageinfo4bpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # <function get2imageinfo4bpu at 0x0000000003A9E9C8>                                                              
    
    message2print = "{(Step 0A/33) passed" + "; task4getinfo2image-passed-to-get2imageinfo4bpu: " + str(task4getinfo2image) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # {'WIDTH': 0, 'DIMENSION': 2, 'HEIGHT': 0}
    # {'WIDTH': 0, 'DEPTH': 0, 'DIMENSION': 3, 'HEIGHT': 0}
    # {'DOMAIN': 'ALL', 'DIMENSION': 2}
    
    image2bpu = {}
    try: 
        if bpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE 2D IMAGE FOR BPU           
            # -------------------------------------------------------------------------------------           
            image2bpu = create2image4bpu()
            message2print = "{(Step 1B2/33) initialized" + "; image2bpu-initialized-by-get2imageinfo4bpu to: " + str(image2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        else:
            message2print = "{(Step 1B2/33) using" + "; bpu2image4xi-using-by-get2imageinfo4bpu: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-', 
            #  'ROWPITCH': 800, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4bpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2image4bpu', 
            #  'METHOD': 'create2image4bpu'}, 
            #  'WIDTH': 800, 
            #  'BUFFER': ...
            
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF THE IMAGE INFO OF BPU          
        # -------------------------------------------------------------------------------------     
        dimension2xi = 0
        
        if task4getinfo2image is None:
            task4getinfo2image = {}
            task4getinfo2image ['DIMENSION'] = 2 
            task4getinfo2image ['DOMAIN'] = 'ALL'
            image2bpu = bpu2image4xi
            message2print = "{(Step 1C/33) using" + "; image2bpu-got-by-get2imageinfo4bpu: " + str(image2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0, 
            #  'DOMAIN': 'ALL', 
            #  'CONTEXT': ...
            
        else:
            pass    

        keys2task4imageinfo = task4getinfo2image.keys()
        if 'DIMENSION' in keys2task4imageinfo:
            dimension2xi = task4getinfo2image['DIMENSION']
            if dimension2xi == 2:
                pass
            else:
                dimension2xi = 2    
        else:
            pass
            
        if 'DOMAIN' in keys2task4imageinfo:
            domain2xi = task4getinfo2image['DOMAIN']
            if domain2xi == 'ALL':
                pass
            else:
                domain2xi = 'ALL'

            keys2domain = bpu2image4xi.keys()
            for key2domain in keys2domain:
                task4getinfo2image[key2domain] = bpu2image4xi[key2domain]
                    
            message2print = "{(Step 1C/33) using" + "; task4getinfo2image-using-by-get2imageinfo4bpu: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                          
        else:
            pass            
            
                    

        keys2task = task4getinfo2image.keys()
        for key2task in keys2task:
            try:
                message2print = "{(Step 2D2/33) using" + "; key2task-using-by-get2image4bpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # WIDTH; DIMENSION; HEIGHT  
                image2bpu [key2task] = bpu2image4xi[key2task]
                
                message2print = "{(Step 2D2/33) set" + "; info-of-image2bpu-set-by-get2imageinfo4bpu to: " + str(image2bpu [key2task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 800; 800  
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key2task-yielded-by-get2image4bpu: " + str(key2task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # DIMENSION;
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo2image-using-by-get2image4bpu: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; bpu2image4xi-using-by-get2image4bpu: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                           
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2bpu = create2image4bpu()
            message2print = "{(Step 1F/33) initialized" + "; image2bpu-initialized-by-get2imageinfo4bpu to: " + str(image2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
            if task4getinfo2image is None:
                image2bpu = bpu2image4xi
                message2print = "{(Step 1G/33) got" + "; image2bpu-got-by-get2imageinfo4bpu: " + str(image2bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            else:
                keys2task = task4getinfo2image.keys()
                for key2task in keys2task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key2task-using-by-get2image4bpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        image2bpu [key2task] = bpu2image4xi[key2task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-image2bpu-set-by-get2imageinfo4bpu to: " + str(image2bpu [key2task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key2task-yielded-by-get2image4bpu: " + str(key2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                            
        except:
            image2bpu = {}
            message2print = "{(Step 1Z3/70) yielded" + "; image2bpu-yielded-by-get2image4bpu: " + str(image2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; image2bpu-returning-by-get2imageinfo4bpu: " + str(image2bpu) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)
    # 2D: {'WIDTH': 800, 'HEIGHT': 800}
    # 3D: {'WIDTH': 800, 'DEPTH': 800, 'HEIGHT': 800}
    # 2D ALL: {'CODE2ERROR': 0, 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4bpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create2image4bpu', 
    #  'METHOD': 'create2image4bpu'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ...
    
    # 3D ALL: {'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create3image4bpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create3image4bpu', 
    #  'METHOD': 'create3image4bpu'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ....
    #  'HEIGHT': 800, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004504400>, 
    #  'WIDTH': 800, 
    #  'DEPTH': 800, 
    #  'FLAGS': 0, 
    #  'CONTEXT': ...
    
    return image2bpu
            
#</function4xi> 
 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2supportedimageformats4bpu" usage = "
#
#   print('(Step 3A/33) OF BPU2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR BPU', time4xi())  
#   try:
#       imageformats = get2supportedimageformats4bpu()                                                                       
#       print('(Step 3B/33) OF BPU2IMAGE4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR BPU: ', imageformats) 
#   except:
#       print('(Step 3Z/33) OF BPU2IMAGE4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR BPU', time4xi())
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2supportedimageformats4bpu(task4supportedimageformats2bpu=None):  

    '''
    gets supported image formats for bpu
    '''
    
    global bpu2image4xi

    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2supportedimageformats4bpu"
    method2print = "get2supportedimageformats4bpu"

    message2print = "{(Step 0A/33) started" + "; get2supportedimageformats4bpu-started: " + str(get2supportedimageformats4bpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    # <function get2supportedimageformats4bpu at 0x0000000003BBE8C8>                                                          
    
    message2print = "{(Step 0B/33) passed" + "; task4supportedimageformats2bpu-passed-to-get2supportedimageformats4bpu: " + str(task4supportedimageformats2bpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # None; {'DIMENSION': 3} 
    
    
    if task4supportedimageformats2bpu is None:
        task4supportedimageformats2bpu = {} 
        task4supportedimageformats2bpu['DIMENSION'] = 2                                                             # 2D image is the default
    
    dimension2xi = 2
    try:
        keys2supportedimageformats = task4supportedimageformats2bpu.keys()    
        if 'DIMENSION' in keys2supportedimageformats:
            dimension2xi = task4supportedimageformats2bpu['DIMENSION']
            if ((dimension2xi <=0) or (dimension2xi > 2)): 
                dimension2xi = 2
        else:
            dimension2xi = 2
    except:
        pass        
    
    message2print = "{(Step 1A/33) using" + "; dimension2xi-using-by-dimension2xi: " + str(dimension2xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # 2; 3
                        
    supportedimageformats2bpu = {} 
    try: 
        if bpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2bpu = create2image4bpu()                                                                          # create bpu2image4xi
            message2print = "{(Step 1B/33) initialized" + "; image2bpu-initialized-by-get2supportedimageformats4bpu to: " + str(image2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                
        else:
            pass

                        
        # -------------------------------------------------------------------------------------
        # SUPPORTED IMAGE FORMATS FOR BPU       
        # -------------------------------------------------------------------------------------     
        supportedimageformats2bpu = bpu2image4xi['FORMAT']
        message2print = "{(Step 1E/33) got" + "; supportedimageformats2bpu-got-by-get2supportedimageformats4bpu: " + str(supportedimageformats2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
            
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2bpu = create2image4bpu()
            message2print = "{(Step 1F1/33) initialized" + "; image2bpu-initialized-by-get2supportedimageformats4bpu to: " + str(image2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
     
            
            # -------------------------------------------------------------------------------------
            # SUPPORTED IMAGE FORMATS FOR BPU       
            # -------------------------------------------------------------------------------------     
            supportedimageformats2bpu = bpu2image4xi['FORMAT']
            message2print = "{(Step 1H/33) got" + "; supportedimageformats2bpu-got-by-get2supportedimageformats4bpu: " + str(supportedimageformats2bpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
            #  
                                              
        except:
            message2print = "{(Step 1Z/70) yielded" + "; supportedimageformats2bpu-yielded-by-get2image4bpu: " + str(supportedimageformats2bpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; supportedimageformats2bpu-returning-by-get2supportedimageformats4bpu: " + str(supportedimageformats2bpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # 2: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    # 3: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    
    return supportedimageformats2bpu
            
#</function4xi> 

 



# -------------------------------------------------------------------------------------------------
#<function4xi function = "wait2image4bpu" usage = "
#   print('=' *33)          
#   print('(Step 16A/700) OF BPU2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE FOR BPU ', time4xi())   
#   print('=' *33)      
#   try:
#       response4wait2bpu = wait2image4bpu()
#       keys2response = response4wait2bpu.keys()
#       print('(Step 16B/700) OF BPU2IMAGE4XI5TEST - WAITED 2D IMAGE FOR BPU ', time4xi()) 
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4wait2bpu[key2response])
#           # ACK2RETURN  :   6C-
#           # BUFFER  :  {'CODE2ERROR': 0 ...
#           # CALLBACK  :  <function trace2image4bpu at 0x0000000004E93B48>
#           # CODE2ERROR  :  0
#           # COMMANDQUEUE  :  {'CODE2ERROR': 0, ...
#           # CONTEXT  :  {'COUNT2REFERENCE': 0 ...
#           # COUNT2REFERENCE  :  1
#           # DATA2CALLBACK  :  {'MESSAGE': '{(Step 1Z1/33) failed; create2image4bpu-failed ; time: 02:00:43 10/06/11 Pacific Daylight Time}', 'CLASS': 'create2image4bpu', 'METHOD': 'create2image4bpu'}
#           # DEPTH  :  0
#           # DEVICE  :  {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK',...
#           # EMPTY2QUEUE  :  True
#           # EVENT  :  {'CODE2ERROR': 0, 'DATA2CALLBACK':...
#           # FLAGS  :  0
#           # FORMAT  :  {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
#           # FULL2QUEUE  :  False
#           # FUNCTION2CALLBACK  :  <function trace2image4bpu at 0x0000000004E93B48>
#           # HEIGHT  :  800
#           # NACK2RETURN  :   
#           # PIPE  :  (<read-write PipeConnection, handle 556>, <read-write PipeConnection, handle 560>)
#           # PLATFORM  :  {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
#           # POINTER  :  <mmap.mmap object at 0x0000000004351500>
#           # QUEUE  :  <queue4xi.Queue4Xi object at 0x00000000044E95C0>
#           # ROWPITCH  :  800
#           # SIZE2QUEUE  :  2147483646 
#   except:
#       print('(Step 16Z1/700) OF BPU2IMAGE4XI5TEST *** FAILED TO WAIT 2D IMAGE FOR BPU: ', wait2bpu)            
#">

# -------------------------------------------------------------------------------------------------   
def wait2image4bpu(task4wait2image=None):

    '''
    waits bpu2image4bpu
    '''
    
    global bpu2image4xi
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "wait2image4bpu"
    method2print = "wait2image4bpu"

    message2print = "{(Step 1A1/33) started" + "; wait2image4bpu-started: " + str(wait2image4bpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    #  
    
    message2print = "{(Step 1A2/33) passed" + "; task4wait2image-passed-by-wait2image4bpu to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    
    try: 
        if bpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE - KEEP SYSTEM RUNNING           
            # -------------------------------------------------------------------------------------           
            bpu2image4xi = create2image4bpu()
            
            message2print = "{(Step 1B/33) created" + "; bpu2image4xi-created-by-wait2image4bpu: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                                                
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4bpu = None
            try:
                queue2image4bpu = bpu2image4xi['QUEUE']
                message2print = "{(Step 1C/33) got" + "; queue2image4bpu-got-by-wait2image4bpu to: " + str(queue2image4bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z1/70) yielded" + "; bpu2xi-yielded-by-wait2image4bpu: " + str(bpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            size2image4bpu = 0
            try:                    
                size2image4bpu = queue2image4bpu.qsize()
                message2print = "{(Step 1D/33) got" + "; size2image4bpu-got-by-wait2image4bpu to: " + str(size2image4bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
                               
                
            except:     
                message2print = "{(Step 1Z2/70) yielded" + "; bpu2xi-yielded-by-wait2image4bpu: " + str(bpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
             
            empty2image4bpu = True       
            try:                    
                empty2image4bpu= queue2image4bpu.empty()
                message2print = "{(Step 1E/33) got" + "; empty2image4bpu-got-by-wait2image4bpu to: " + str(empty2image4bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z3/70) yielded" + "; empty2image4bpu-yielded-by-wait2image4bpu: " + str(empty2image4bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
             
            full2image4bpu = False       
            try:                                
                full2image4bpu = queue2image4bpu.full()
                message2print = "{(Step 1F/33) got" + "; full2image4bpu-got-by-wait2image4bpu to: " + str(full2image4bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z4/70) yielded" + "; full2image4bpu-yielded-by-wait2image4bpu: " + str(full2image4bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                            
            message2print = "{(Step 1G/33) finished" + "; bpu2image4xi-finished-by-wait2image4bpu to: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                            
        else:
            # -------------------------------------------------------------------------------------
            # USE THE EXISTING IMAGE FOR BPU          
            # -------------------------------------------------------------------------------------     
            
            message2print = "{(Step 1H/33) used" + "; bpu2image4xi-used-by-wait2image4bpu: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4bpu = None
            try:
                queue2image4bpu = bpu2image4xi['QUEUE']
                message2print = "{(Step 1I/33) got" + "; queue2image4bpu-got-by-wait2image4bpu to: " + str(queue2image4bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            except:     
                message2print = "{(Step 1Z5/70) yielded" + "; bpu2xi-yielded-by-wait2image4bpu: " + str(bpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                message2print = "{(Step 1J1/33) passed" + "; task4wait2image-passed-by-wait2image4bpu to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                queue2task = {}    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1J2/33) set" + "; queue2task-set-by-wait2image4bpu to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                except:     
                    message2print = "{(Step 1Z61/70) yielded" + "; queue2task-yielded-by-wait2image4bpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                
                try:                        
                    if queue2task == queue2image4bpu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1K/33) accepted" + "; queue2task-accepted-by-wait2image4bpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        try:
                            message2print = "{(Step 1L/33) warned" + "; queue2task-warned-by-wait2image4bpu: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print) 
                            # 
                        
                            message2print = "{(Step 1M/33) warned" + "; queue2image4bpu-warned-by-wait2image4bpu: " + " ** " + str(queue2image4bpu)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                            # 
                        except:
                            pass
                                                     
                except:     
                    message2print = "{(Step 1Z62/70) yielded" + "; queue2task-yielded-by-wait2image4bpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                   
            
            size2queue4bpu = 0        
            try:                    
                size2queue4bpu = queue2image4bpu.qsize()
                message2print = "{(Step 1N1/33) got" + "; size2queue4bpu-got-by-wait2image4bpu to: " + str(size2queue4bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                bpu2image4xi ['SIZE2QUEUE'] = size2queue4bpu
                message2print = "{(Step 1N2/33) set" + "; size2queue4image-got-by-wait2image4bpu to: " + str(bpu2image4xi ['SIZE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                # 
            except:     
                message2print = "{(Step 1Z7/70) yielded" + "; size2queue4bpu-yielded-by-wait2image4bpu: " + str(size2queue4bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            empty2queue4bpu = False
            try:                    
                empty2queue4bpu = queue2image4bpu.empty()
                message2print = "{(Step 1O1/33) got" + "; empty2queue4bpu-got-by-wait2image4bpu to: " + str(empty2queue4bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                bpu2image4xi ['EMPTY2QUEUE'] = empty2queue4bpu
                message2print = "{(Step 1O2/33) set" + "; empty2queue4image-got-by-wait2image4bpu to: " + str(bpu2image4xi ['EMPTY2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #               
            except:     
                message2print = "{(Step 1Z8/70) yielded" + "; empty2queue4bpu-yielded-by-wait2image4bpu: " + str(empty2queue4bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            full2queue4bpu = False
            try:                                
                full2queue4bpu = queue2image4bpu.full()
                message2print = "{(Step 1P1/33) got" + "; full2queue4bpu-got-by-wait2image4bpu to: " + str(full2queue4bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                bpu2image4xi ['FULL2QUEUE'] = full2queue4bpu
                message2print = "{(Step 1P2/33) set" + "; full2queue4image-got-by-wait2image4bpu to: " + str(bpu2image4xi ['FULL2QUEUE'])  + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)
                #                          
            except:     
                message2print = "{(Step 1Z9/70) yielded" + "; full2queue4bpu-yielded-by-wait2image4bpu: " + str(full2queue4bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            state2queue4bpu = None
            try:                                
                state2queue4bpu = queue2image4bpu.__getstate__()
                message2print = "{(Step 1P3/33) got" + "; state2queue4bpu-got-by-wait2image4bpu to: " + str(state2queue4bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                bpu2image4xi ['STATE2QUEUE'] = state2queue4bpu
                message2print = "{(Step 1P4/33) set" + "; state2queue4image-set-by-wait2image4bpu to: " + str(bpu2image4xi ['STATE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #           
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; state2queue4bpu-yielded-by-wait2image4bpu: " + str(state2queue4bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)               
                
                                                            
            message2print = "{(Step 1Q/33) finished" + "; bpu2image4xi-finished-by-wait2image4bpu: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            bpu2image4bpu = create2image4bpu()
            message2print = "{(Step 1R/33) got" + "; bpu2image4xi-got-by-wait2image4bpu to: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4bpu = None
            try:
                queue2image4bpu = bpu2image4xi['QUEUE']
                message2print = "{(Step 1S/33) got" + "; queue2image4bpu-got-by-wait2image4bpu to: " + str(queue2image4bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; queue2image4bpu-yielded-by-wait2image4bpu: " + str(queue2image4bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE QUEUE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                queue2task = None    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1T/33) passed" + "; queue2task-passed-by-wait2image4bpu to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #
                    
                    if queue2task == queue2image4bpu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1U/33) accepted" + "; queue2task-accepted-by-wait2image4bpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        message2print = "{(Step 1X/33) warned" + "; queue2task-warned-by-wait2image4bpu: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                        
                        message2print = "{(Step 1Y1/33) warned" + "; queue2image4bpu-warned-by-wait2image4bpu: " + " ** " + str(queue2image4bpu)  + " ** " + "; time: " + time4xi() + "}" 
                        print4xi(class2print, method2print, message2print)    
                        #                      
                except:     
                    message2print = "{(Step 1Z11/70) yielded" + "; queue2task-yielded-by-wait2image4bpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
            
            size2image4bpu = 0                             
            try:                    
                size2image4bpu = queue2image4bpu.qsize()
                message2print = "{(Step 1Y2/33) got" + "; size2image4bpu-got-by-wait2image4bpu to: " + str(size2image4bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z12/70) yielded" + "; size2image4bpu-yielded-by-wait2image4bpu: " + str(size2image4bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            empty2image4bpu = True          
            try:                    
                empty2image4bpu = queue2image4bpu.empty()
                message2print = "{(Step 1Y3/33) got" + "; empty2image4bpu-got-by-wait2image4bpu to: " + str(empty2image4bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z13/70) yielded" + "; bpu2xi-yielded-by-wait2image4bpu: " + str(bpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            full2image4bpu = False          
            try:                                
                full2image4bpu= queue2image4bpu.full()
                message2print = "{(Step 1Y4/33) got" + "; full2image4bpu-got-by-wait2image4bpu to: " + str(full2image4bpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z14/70) yielded" + "; full2image4bpu-yielded-by-wait2image4bpu: " + str(full2image4bpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                                        
            message2print = "{(Step 1Y5/33) finished" + "; bpu2image4xi-finished-by-wait2image4bpu: " + str(bpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                    
        except:
            bpu2image4bpu= {}
            message2print = "{(Step 1Z15/70) yielded" + "; bpu2image4xi-yielded-by-wait2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; bpu2image4xi-returning-by-wait2image4bpu: " + str(bpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # {'SIZE2QUEUE': 2147483646, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004420E10>, 
    #  'WIDTH': 800, 
    #  'CONTEXT': ...
    
    return bpu2image4xi
    
   
#</function4xi> 
                                               
# =================================================================================================
#<class4xi class = "BPU2Image4Xi" base = "object" child = "None"
#   synopsis = "offers methods for BPU 2D images" usage = "2D Image4Xi of BPU"
#>

# =================================================================================================
class BPU2Image4Xi(object): 
    ''' 
    BPU2Image4Xi offers methods for BPU 2D images
    '''            

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    classvariable2bpu = cpu_corecount4xi()
    #"
    #/>
        
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = (
            'create2image',                    # ACK 5.31   p76    
            'copy2image',                      # ACK 5.31   p88
            'copy2imagetobuffer',              # ACK 5.3.4  p90
            'copy2buffertoimage',              # ACK 5.3.4  p92
            'read2image',                      # ACK 5.3.3  p84
            'write2image',                     # ACK 5.3.3  p84            
            'map2image',                       # ACK 5.3.5  p94            
            'get2imageinfo',                   # ACK 5.3.6  p97
            'get2supportedimageformats',       # ACK 5.3.2  p82 nb: a separate method to get format whereas using get2imageinfo you can access any element by key such as 'FORMAT'   
            'retain2image',                    # ACK 5.4.1  p99 nb: for compability to OpenCL - you can get any element of dictionary by get2imageinfo
            'release2image',                   # ACK 5.4.1  p99
            'trace2image',                     # ACK modernized 
            'set2imagecallback',               # ACK modernized
            'set2image',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    
            'wait2image',                      # ACK modernized nb: checks Queue object and thereby generates delay   
            'sense',                           # ACK modernized  
            'reason',                          # ACK modernized  
            'ace')                             # ACK modernized           
    #"
    #/>
 
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-BPU2Image4Xi" usage = "
    #   print('(Step 5A/33) OF BPU2IMAGE4XI5TEST - CREATING IMAGE4BPU OBJECT ', time4xi())     
    #   try:
    #       imageobject2bpu = BPU2Image4Xi()
    #       print('(Step 5B/33) OF BPU2IMAGE4XI5TEST - CREATED IMAGE4BPU OBJECT: ', imageobject2bpu)                                  
    #   except:
    #       print('(Step 5Z/33) OF BPU2IMAGE4XI5TEST *** FAILED TO CREATE IMAGE4BPU OBJECT')
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def __init__(self, task4imagecreate2bpu = None):
        '''
        _init__ method initializes BPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------
                    
        class2print = "BPU2Image4Xi"
        method2print = "__init__"
                     
        message2print = "{(Step 1A/33) started" + "; init-of-BPU2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 23:41:37 09/06/11 Pacific Daylight Time 

        message2print = "{(Step 1B/33) borning" + "; bpu2image2xi-of-BPU2Image4Xi-borning: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
                
        message2print = "{(Step 1C/33) inheriting" + "; object-inheriting-by-bpu2image2xi-of-BPU2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'>
        
        message2print = "{(Step 1D/33) sharing" + "; classvariable2bpu-sharing-by-bpu2image2xi-of-BPU2Image4Xi: " + str(BPU2Image4Xi.classvariable2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4             
                                
        message2print = "{(Step 1E/33) passed" + "; task4imagecreate2bpu-passed-to-init-of-BPU2Image4Xi: " + str(task4imagecreate2bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None                            
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)             
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) passing" + "; task4imagecreate2bpu-passing-to-create2image4bpu-by-init-of-BPU2Image4Xi: " + str(task4imagecreate2bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None  
        
        bpu2image2xi = create2image4bpu(task4imagecreate2bpu)
        
        message2print = "{(Step 2B/33) performed" + "; create2image4bpu-performed-by-init-of-BPU2Image4Xi: " + str(create2image4bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <function create2image4bpu at 0x0000000003A9E7C8>
                        
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)             
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3A/33) activated" + "; bpu2image2xi-of-BPU2Image4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # { BPU2Image4Xi : __init__ ; (Step 3A/33) activated; bpu2image2xi-of-BPU2Image4Xi-activated: ; time: 00:14:05 09/07/11 Pacific Daylight Time} 
                
        message2print = "{(Step 3B/33) inherited" + "; object-inherited-by-bpu2image2xi-of-BPU2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'> 
        
        message2print = "{(Step 3C/33) shared" + "; classvariable2bpu-shared-by-bpu2image2xi-of-BPU2Image4Xi: " + str(BPU2Image4Xi.classvariable2bpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4           
       
        message2print = "{(Step 3D/33) returning" + "; init-of-BPU2Image4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 00:14:05 09/07/11 Pacific Daylight Time
        

    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create2image-of-BPU2Image4Xi" usage = "
    #   print('(Step 23A/33) OF BPU2IMAGE4XI5TEST - TRYING TO CREATE BPU IMAGE', time4xi()) 
    #   try:
    #       xi2bpu = BPU2Image4Xi()                            
    #       image2bpu = xi2bpu.create2image()                                  
    #       print('(Step 23B/33) OF BPU2IMAGE4XI5TEST - CREATED BPU IMAGE: ', image2bpu)
    #   except:
    #       print('(Step 23Z/33) OF BPU2IMAGE4XI5TEST *** FAILED TO CREATE BPU IMAGE ***', time4xi())                  
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def create2image(self, task4image2create = None):
        '''
        create2image method creates 2D image objects of BPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "BPU2Image4Xi"
        method2print = "create2image"
                     
        message2print = "{(Step 1A/33) started" + "; create2image-of-BPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4image2create-passed-to-create-of-BPU2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4image2create-passing-to-create2image4bpu-by-create2image-of-BPU2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None      
  
        bpu2image2xi = create2image4bpu(task4image2create)

        message2print = "{(Step 2B/33) performed" + "; create2image4bpu-performed-by-create2image-of-BPU2Image4Xi: " + str(create2image4bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function create2image4bpu at 0x00000000039DF7C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; bpu2image2xi-returning-by-create2image-of-BPU2Image4Xi: " + str(bpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4bpu-failed ; time: 00:53:55 10/05/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4bpu',
        #  'METHOD': 'create2image4bpu'},
        #  'WIDTH': 800 ...

        return bpu2image2xi
                                
    #                        
    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2image-of-BPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF BPU2IMAGE4XI - TRYING TO SET BPU IMAGE')
    #   try:
    #       cc2xi = BPU2Image4Xi()                            
    #       image2bpu = cc2xi.set2image()                                  
    #       print('(Step 1B/33) OF BPU4XI5TEST - SET BPU IMAGE: ', image2bpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF BPU4XI5TEST *** FAILED TO SET BPU IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2image(self, task4set2image = None):
        '''
        sets 2D image BPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "BPU2Image4Xi"
        method2print = "set2image"
                     
        message2print = "{(Step 1A/33) started" + "; set2image-of-BPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4set2image-passed-to-set2image-of-BPU2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE2ERROR': 0,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2image-passing-to-set2image4bpu-by-set2image-of-BPU2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'CODE2ERROR': 0, 
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>, 
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...

        bpu2image2xi = set2image4bpu(task4set2image)

        message2print = "{(Step 2B/33) performed" + "; set2image4bpu-performed-by-set2image-of-BPU2Image4Xi: " + str(set2image4bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function set2image4bpu at 0x0000000003AB36C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; bpu2image2xi-returning-by-set2image-of-BPU2Image4Xi: " + str(bpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4bpu-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4bpu', 'METHOD': 'create2image4bpu'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4bpu-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4bpu',
        #  'METHOD': 'create2buffer4bpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4bpu at 0x0000000003AA81C8>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CC92F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                    
                
        return bpu2image2xi
                                
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2imageinfo-of-BPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF BPU2IMAGE4XI - TRYING TO GET BPU 2D IMAGE')
    #   try:
    #       cc2xi = BPU2Image4Xi()                            
    #       image2bpu = cc2xi.get2imageinfo()                                  
    #       print('(Step 1B/33) OF BPU4XI5TEST - GOT BPU IMAGE: ', image2bpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF BPU4XI5TEST *** FAILED TO GET BPU 2D IMAGE ***')   
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2imageinfo(self, task4getinfo2image=None):
        '''
        get2imageinfo method gets 2D objects of BPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "BPU2Image4Xi"
        method2print = "get2imageinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2imageinfo-of-BPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        bpu2image2xi = get2imageinfo4bpu(task4getinfo2image)
        
        message2print = "{(Step 2A/33) returning" + "; bpu2image2xi-returning-by-get2imageinfo-of-BPU2Image4Xi: " + str(bpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4bpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4bpu', 'METHOD': 'create2image4bpu'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4bpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4bpu',
        #  'METHOD': 'create2buffer4bpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4bpu at 0x00000000039D8348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000037D32F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        # or when only two elemets required
        # {'WIDTH': 800, 'HEIGHT': 800}
        
        return bpu2image2xi
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2supportedimageformats-of-BPU2Image4Xi" usage = "
    #  print('(Step 6B1/33) OF BPU2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR BPU')
    #  try:
    #      cc2xi = BPU2Image4Xi()                            
    #      supportedimageformats2bpu = cc2xi.get2supportedimageformats()                                  
    #      print('(Step 6B2/33) OF BPU4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR BPU: ', supportedimageformats2bpu)
    #                                                   
    #  except:
    #      print('(Step 2Z/33) OF BPU4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR BPU***')     
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2supportedimageformats(self):
        '''
        get2supportedimageformats method gets supported imageformats by objects of BPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "BPU2Image4Xi"
        method2print = "get2supportedimageformats"
                     
        message2print = "{(Step 1A/33) started" + "; get2supportedimageformats-of-BPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:43:00 09/07/11 Pacific Daylight Time

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        supportedimageformats2bpu = get2supportedimageformats4bpu()
        
        message2print = "{(Step 2A/33) returning" + "; supportedimageformats2bpu-returning-by-get2imageinfo-of-BPU2Image4Xi: " + str(supportedimageformats2bpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        
        return supportedimageformats2bpu
                            
    #</methodize4xi>    
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "read2image-of-BPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF BPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi()) 
    #   response4read2bpu = {}
    #   try:
    #       xi2bpu = BPU2Image4Xi()    
    #       response4read2bpu = xi2bpu.read2image()
    #       print('(Step 127B/700) OF BPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH READ IMAGE OBJECT, RESPONSE4READ2BPU: ', response4read2bpu) 
    #   except:
    #       print('(Step 127Z1/700) OF BPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi())
    #   ... continue as with read2image4bpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def read2image(self, task4read2image = None):
        '''
        reads BPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "BPU2Image4Xi"
        method2print = "read2image"
                     
        message2print = "{(Step 1A/33) started" + "; read2image-of-BPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:20:02 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4read2image-passed-to-read2image-of-BPU2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4read2image-passing-to-read2image4bpu-by-read2image-of-BPU2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4readimage2bpu = read2image4bpu(task4read2image)

        message2print = "{(Step 2B/33) performed" + "; read2image4bpu-performed-by-read2image-of-BPU2Image4Xi: " + str(read2image4bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function read2image4bpu at 0x000000000385DA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4readimage2bpu-returning-by-read2image-of-BPU2Image4Xi: " + str(response4readimage2bpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000417DD68>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 936>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 932>}; time: 00:20:02 10/05/11 Pacific Daylight Time}
  
        return response4readimage2bpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "map2image-of-BPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF BPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi()) 
    #   xi2bpu = BPU2Image4Xi() 
    #   ...
    #   response4map2bpu = {}
    #   try:
    #       response4map2bpu = xi2bpu.map2image()
    #       print('(Step 127B/700) OF BPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH MAP IMAGE OBJECT, RESPONSE4READ2BPU: ', response4map2bpu) 
    #   except:
    #       print('(Step 127Z1/700) OF BPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi())
    #   ... continue as with map2image4bpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def map2image(self, task4map2image = None):
        '''
        maps BPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "BPU2Image4Xi"
        method2print = "map2image"
                     
        message2print = "{(Step 1A/33) started" + "; map2image-of-BPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4map2image-passed-to-map2image-of-BPU2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4map2image-passing-to-map2image4bpu-by-map2image-of-BPU2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4mapimage2bpu = map2image4bpu(task4map2image)

        message2print = "{(Step 2B/33) performed" + "; map2image4bpu-performed-by-map2image-of-BPU2Image4Xi: " + str(map2image4bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function map2image4bpu at 0x000000000390BAC8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4mapimage2bpu-returning-by-map2image-of-BPU2Image4Xi: " + str(response4mapimage2bpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x00000000050E9C18>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1100>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1096>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
        return response4mapimage2bpu
                                                           
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "release2image-of-BPU2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RELEASE 2D IMAGE BY DECREMENTING REFERENCE COUNT WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4release2image  = {} 
    #   task4release2image  ['COUNT2REFERENCE'] = 1 
    #   response4release2image = {}
    #   print('(Step 133A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RELEASE 2D IMAGE OF BPU DEVICE ', time4xi())    
    # 
    #   try:                               
    #       response4release2Dimage = xi2bpu.release2image(task4release2image)                                                                                             
    #       print('(Step 133B/700) OF BPU2IMAGE4XI5TEST - RELEASED 2D IMAGE OF BPU DEVICE ', time4xi())                           
    #       keys2response = response4release2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4release2Dimage[key2response])
    #           #
    #   except:
    #       print('(Step 133Z1/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF BPU DEVICE ', time4xi())  
    #   
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 133C/700) OF BPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF BPU DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2bpu.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 133D/700) OF BPU2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF BPU DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 133Z2/700) OF BPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF BPU DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def release2image(self, task4release2image = None):
        '''
        releases BPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "BPU2Image4Xi"
        method2print = "release2image"
                     
        message2print = "{(Step 1A/33) started" + "; release2image-of-BPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4release2image-passed-to-release2image-of-BPU2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4release2image-passing-to-release2image4bpu-by-release2image-of-BPU2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        bpu2image2xi = release2image4bpu(task4release2image)

        message2print = "{(Step 2B/33) performed" + "; release2image4bpu-performed-by-release2image-of-BPU2Image4Xi: " + str(release2image4bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; bpu2image2xi-returning-by-release2image-of-BPU2Image4Xi: " + str(bpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #                          
                
        return bpu2image2xi
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "retain2image-of-BPU2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RETAIN 2D IMAGE BY DECREASING REFERENCE WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4retain2image  = {}
    #   task4retain2image  ['COUNT2REFERENCE'] = 1 
    #   response4retain2Dimage = {}
    #   print('(Step 134A/700) OF BPU2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF BPU DEVICE ', time4xi())   
    #   try:                               
    #       response4retain2Dimage = xi2bpu.retain2image(task4retain2image)                                                                                                 
    #       print('(Step 134B/700) OF BPU2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', time4xi())                         
    #       keys2response = response4retain2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4retain2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z1/700) OF BPU2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF BPU DEVICE ', time4xi())
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 134C/700) OF BPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF BPU DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2bpu.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 134D/700) OF BPU2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF BPU DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z2/700) OF BPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF BPU DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def retain2image(self, task4retain2image = None):
        '''
        retains BPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "BPU2Image4Xi"
        method2print = "retain2image"
                     
        message2print = "{(Step 1A/33) started" + "; retain2image-of-BPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4retain-passed-to-retain2image-of-BPU2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'COUNT2REFERENCE': 1}
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4retain-passing-to-retain2image4bpu-by-retain-of-BPU2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'COUNT2REFERENCE': 1}
  
        bpu2image2xi = retain2image4bpu(task4retain2image)

        message2print = "{(Step 2B/33) performed" + "; retain2image4bpu-performed-by-retain2image-of-BPU2Image4Xi: " + str(retain2image4bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function retain2image4bpu at 0x00000000039DF948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; bpu2image2xi-returning-by-retain2image-of-BPU2Image4Xi: " + str(bpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 
        #  'ACK2RETURN': ' 1D-', 
        #  'ROWPITCH': 800, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4bpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2image4bpu', 
        #  'METHOD': 'create2image4bpu'}, 
        #  'WIDTH': 800, 
        #  'BUFFER': {'CODE2ERROR': 0, 
        #  ...
        #  'COUNT2REFERENCE': 1,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                         
                
        return bpu2image2xi
                                                           
    #</methodize4xi>   
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "wait2image-of-BPU2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 126A/700) OF BPU2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE OPERATION FOR BPU ', time4xi()) 
    #   print('=' *33)      
    #   wait2bpu = {}
    #   try:
    #       response4wait2Dimage = xi2bpu.wait2image()
    #       print('(Step 122B/700) OF BPU2IMAGE4XI5TEST - WAITED 2D IMAGE OPERATION FOR BPU ', time4xi()) 
    #       keys2response = response4wait2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4wait2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 126Z1/700) OF BPU2IMAGE4XI5TEST *** FAILED TO 2D IMAGE OPERATION FOR BPU ', time4xi())    
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def wait2image(self, task4wait2image = None):
        '''
        waits BPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "BPU2Image4Xi"
        method2print = "wait2image"
                     
        message2print = "{(Step 1A/33) started" + "; wait2image-of-BPU2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4wait2image-passed-to-wait2image-of-BPU2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4wait-passing-to-wait2image4bpu-by-wait2image-of-BPU2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        bpu2image2xi = wait2image4bpu(task4wait2image)

        message2print = "{(Step 2B/33) performed" + "; wait2image4bpu-performed-by-wait2image-of-BPU2Image4Xi: " + str(wait2image4bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # semaphore and lock analysis by multiprocessing4xi
        # <function wait2image4bpu at 0x00000000039DFA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; bpu2image2xi-returning-by-wait2image-of-BPU2Image4Xi: " + str(bpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'SIZE2QUEUE': 2147483646,
        #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'},
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000434DDA0>,
        #  'WIDTH': 800,
        #  'CONTEXT': ...
                
        return bpu2image2xi
                                                           
    #</methodize4xi> 
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "write2image-of-BPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF BPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4write2bpu = {}
    #   try:
    #       response4write2bpu = xi2bpu.write2image()
    #       print('(Step 127B/700) OF BPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2BPU: ', response4write2bpu) 
    #   except:
    #       print('(Step 127Z1/700) OF BPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with write2image4bpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def write2image(self, task4write2image = None):
        '''
        writes BPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "BPU2Image4Xi"
        method2print = "write2image"
                     
        message2print = "{(Step 1A/33) started" + "; write2image-of-BPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 02:45:31 10/05/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4write2image-passed-to-write2image-of-BPU2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4write2image-passing-to-write2image4bpu-by-write2image-of-BPU2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4writeimage2bpu = write2image4bpu(task4write2image)

        message2print = "{(Step 2B/33) performed" + "; write2image4bpu-performed-by-write2image-of-BPU2Image4Xi: " + str(write2image4bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function write2image4bpu at 0x000000000380AEC8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4writeimage2bpu-returning-by-write2image-of-BPU2Image4Xi: " + str(response4writeimage2bpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004D10E10>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 968>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 964>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

                     
                
        return response4writeimage2bpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2image-of-BPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF BPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4copy2bpu = {}
    #   try:
    #       response4copy2bpu = xi2bpu.copy2image()
    #       print('(Step 127B/700) OF BPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2BPU: ', response4copy2bpu) 
    #   except:
    #       print('(Step 127Z1/700) OF BPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with copy2image4bpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2image(self, task4copy2image = None):
        '''
        copies BPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "BPU2Image4Xi"
        method2print = "copy2image"
                     
        message2print = "{(Step 1A/33) started" + "; copy2image-of-BPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2image-passed-to-copy2image-of-BPU2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2image-passing-to-copy2image4bpu-by-copy2image-of-BPU2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None 
  
        response4copyimage2bpu = copy2image4bpu(task4copy2image)

        message2print = "{(Step 2B/33) performed" + "; copy2image4bpu-performed-by-copy2image-of-BPU2Image4Xi: " + str(copy2image4bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2image4bpu at 0x000000000390B8C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2bpu-returning-by-copy2image-of-BPU2Image4Xi: " + str(response4copyimage2bpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004EC4128>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1000>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 996>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
        
        return response4copyimage2bpu
                                                           
    #</methodize4xi>
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2imagetobuffer-of-BPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF BPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER OBJECT', time4xi()) 
    #   response4copyimagetobuffer = {}
    #   try:
    #       response4copyimagetobuffer = xi2bpu.copy2imagetobuffer()
    #       print('(Step 127B/700) OF BPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER  OBJECT, RESPONSE4WRITE2BPU: ', response4copyimagetobuffer) 
    #   except:
    #       print('(Step 127Z1/700) OF BPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER ', time4xi()) 
    #
    #   ... continue as with copy2imagetobuffer4bpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2imagetobuffer(self, task4copy2imagetobuffer = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "BPU2Image4Xi"
        method2print = "copy2imagetobuffer"
                     
        message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer-of-BPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2imagetobuffer-passed-to-copy2imagetobuffer-of-BPU2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2imagetobuffer-passing-to-copy2imagetobuffer4bpu-by-copy2imagetobuffer-of-BPU2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2bpu = copy2imagetobuffer4bpu(task4copy2imagetobuffer)

        message2print = "{(Step 2B/33) performed" + "; copy2imagetobuffer4bpu-performed-by-copy2imagetobuffer-of-BPU2Image4Xi: " + str(copy2imagetobuffer4bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2imagetobuffer4bpu at 0x000000000390B948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2bpu-returning-by-copy2imagetobuffer-of-BPU2Image4Xi: " + str(response4copyimage2bpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004FE5FD0>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1036>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1032>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
          
        return response4copyimage2bpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2buffertoimage-of-BPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF BPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE OBJECT', time4xi()) 
    #   response4copybuffertoimage = {}
    #   try:
    #       response4copybuffertoimage = xi2bpu.copy2buffertoimage()
    #       print('(Step 127B/700) OF BPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE  OBJECT, RESPONSE4WRITE2BPU: ', response4copybuffertoimage) 
    #   except:
    #       print('(Step 127Z1/700) OF BPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE ', time4xi()) 
    #
    #   ... continue as with copy2buffertoimage4bpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2buffertoimage(self, task4copy2buffertoimage = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "BPU2Image4Xi"
        method2print = "copy2buffertoimage"
                     
        message2print = "{(Step 1A/33) started" + "; copy2buffertoimage-of-BPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2buffertoimage-passed-to-copy2buffertoimage-of-BPU2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2buffertoimage-passing-to-copy2buffertoimage4bpu-by-copy2buffertoimage-of-BPU2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2bpu = copy2buffertoimage4bpu(task4copy2buffertoimage)

        message2print = "{(Step 2B/33) performed" + "; copy2buffertoimage4bpu-performed-by-copy2buffertoimage-of-BPU2Image4Xi: " + str(copy2buffertoimage4bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2buffertoimage4bpu at 0x000000000390B9C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2bpu-returning-by-copy2buffertoimage-of-BPU2Image4Xi: " + str(response4copyimage2bpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000505A6D8>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1068>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1064>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

                
        return response4copyimage2bpu
                                                           
    #</methodize4xi>    
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "trace2image-of-BPU2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 15A/25) OF BPU2IMAGE4XI5TEST - TRYING TO TRACE BPU: ', time4xi())      
    #   print('=' *33)      
    #   trace2bpu = None
    #   try:
    #       trace2bpu = trace2image4bpu() 
    #       print('(Step 15B/25) OF BPU2IMAGE4XI5TEST - TRACE2BPU: ', trace2bpu)  
    #       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4bpu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4bpu', 'METHOD': 'trace2image4bpu'}
    #    except:
    #       print('(Step 15Z1/25) OF BPU2IMAGE4XI5TEST *** FAILED TO TRACE BPU: ', trace2bpu)     
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def trace2image(self, task4trace2image = None):
        '''
        traces BPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "BPU2Image4Xi"
        method2print = "trace2image"
                     
        message2print = "{(Step 1A/33) started" + "; trace2image-of-BPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4trace2image-passed-to-trace-of-BPU2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None   
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4trace2image-passing-to-trace2image4bpu-by-trace2image-of-BPU2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None  
  
        trace2xi = trace2image4bpu(task4trace2image)

        message2print = "{(Step 2B/33) performed" + "; trace2image4bpu-performed-by-trace2image-of-BPU2Image4Xi: " + str(trace2image4bpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function trace2image4bpu at 0x00000000039DF348>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace2xi-returning-by-trace2image-of-BPU2Image4Xi: " + str(trace2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4bpu-failed {}; time: 01:05:20 09/08/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4bpu',
        #  'METHOD': 'trace2image4bpu'}; time: 01:05:20 09/08/11 Pacific Daylight Time}
                 
                
        return trace2xi
                                                           
    #</methodize4xi>       


    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2imagecallback-of-BPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF BPU2IMAGE4XI - TRYING TO SET BPU IMAGE')
    #   try:
    #       cc2xi = BPU2Image4Xi()                            
    #       image2bpu = cc2xi.set2imagecallback()                                  
    #       print('(Step 1B/33) OF BPU4XI5TEST - SET BPU IMAGE: ', image2bpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF BPU4XI5TEST *** FAILED TO SET BPU IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2imagecallback(self, task4set2imagecallback = None): 
        '''
        set2imagecallback of BPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "BPU2Image4Xi"
        method2print = "set2imagecallback"
                     
        message2print = "{(Step 1A/33) started" + "; set2imagecallback-of-BPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4set2imagecallback-passed-to-set2imagecallback-of-BPU2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None    
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2imagecallback-passing-to-set2imagecallback -by-set2imagecallback-of-BPU2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
        
        callback2bpu4image = {} 
        try:
            callback2bpu4image = set2imagecallback4bpu (task4set2imagecallback)

            message2print = "{(Step 2B/33) callback2bpu4image" + "; callback2bpu4image-set-by-set2imagecallback-of-BPU2Image4Xi to: " + str(callback2bpu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # <function trace2image4bpu at 0x00000000039DF348>
        except:
            message2print = "{(Step 4A/33) failed" + "; callback2bpu4image-failed-by-set2imagecallback-of-BPU2Image4Xi: " + str(callback2bpu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                            
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; callback2bpu4image-returning-by-set2imagecallback-of-BPU2Image4Xi: " + str(callback2bpu4image) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4bpu at 0x0000000003AB31C8> 
                                             
        return callback2bpu4image
                                                           
    #</methodize4xi>     
    
                                                              
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-BPU2Image4Xi" usage = "
    #   try:
    #       print(xi2bpu.sense())
    #       print('(Step 24B/33) OF BPU2IMAGE4XI5TEST - SENSED ', time4xi()) 
    #   except:
    #       print('(Step 24Z/33) OF BPU2IMAGE4XI5TEST *** FAILED TO SENSE ', time4xi())  
    # ">
    
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses BPU2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'BPU2Image4Xi'
        method2print = 'sense'
        
                     
        message2print = "{(Step 1A/33) started" + "; sense-of-BPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        
        message2print = "{(Step 2A/33) sensing" + "; BPU2Image4Xi-sensing-by-sense-of-BPU2Image4Xi: " + str(BPU2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'bpu2image4xi.BPU2Image4Xi'>         
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3A/33) got" + "; kb2xi-got-by-sense-of-BPU2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object': <bpu2image4xi.BPU2Image4Xi object at 0x0000000004188400>}   
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['classvariable2bpu-of-BPU2Image4Xi'] = BPU2Image4Xi.classvariable2bpu
        except:
            message2print = "{(Step 4A/33) yielded" + "; kb2xi-yielded-by-sense-of-BPU2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
      
        message2print = "{(Step 5A/33) sensed" + "; BPU2Image4Xi-sensed-by-sense-of-BPU2Image4Xi: " + str(BPU2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # <class 'bpu2image4xi.BPU2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        
        message2print = "{(Step 6A/33) returning" + "; kb2xi-returning-by-sense-of-BPU2Image4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object': <bpu2image4xi.BPU2Image4Xi object at 0x0000000004188400>, 'classvariable2bpu-of-BPU2Image4Xi': 4}             
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-BPU2Image4Xi" usage = "
    #   print('(Step 25A/33) OF BPU2IMAGE4XI5TEST - REASONING ', time4xi())
    #   try:
    #       print(xi2bpu.reason())
    #       print('(Step 25B/33) OF BPU2IMAGE4XI5TEST - REASONED ', time4xi())
    #   except:
    #       print('(Step 25Z/33) OF BPU2IMAGE4XI5TEST *** FAILED TO REASON ', time4xi())   
    #">
    
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons BPU2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'BPU2Image4Xi'
        method2print = 'reason'

                     
        message2print = "{(Step 1A/33) started" + "; reason-of-BPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) reasoning" + "; BPU2Image4Xi-reasoning-by-reason-of-BPU2Image4Xi: " + str(BPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <class 'bpu2image4xi.BPU2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{(Step 3A/33) got" + "; kb4xi-got-by-reason-of-BPU2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object-reason-object4xi': <bpu2image4xi.BPU2Image4Xi object at 0x0000000004188400>}      
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        try:
            kb4xi['time-reason-of-BPU2Image4Xi'] = time4xi() #adding time
        except:
            message2print = "{(Step 4Z/33) yielded" + "; kb4xi-yielded-by-reason-of-BPU2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
          
        
        message2print = "{(Step 5A/33) reasoned" + "; BPU2Image4Xi-reasoned-by-reason-of-BPU2Image4Xi: " + str(BPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # <class 'bpu2image4xi.BPU2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        
        message2print = "{(Step 6A/33) returning" + "; kb4xi-returning-by-reason-of-BPU2Image4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object-reason-object4xi': <bpu2image4xi.BPU2Image4Xi object at 0x0000000004188400>, 'time-reason-of-BPU2Image4Xi': '01:02:22 09/07/11 Pacific Daylight Time'}                
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-BPU2Image4Xi" usage = "
    #   print('(Step 26A/33) OF BPU2IMAGE4XI5TEST - ACING ', time4xi())   
    #   try:
    #       print(xi2bpu.ace())
    #       print('(Step 26B/33) OF BPU2IMAGE4XI5TEST - ACED ', time4xi())                                                 
    #   except:
    #       print('(Step 26Z/33) OF BPU2IMAGE4XI5TEST *** FAILED TO ACE ', time4xi())                                       
    #    
    #">
    
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces BPU2Image4Xi class objects 
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------       
        
        
        class2print = 'BPU2Image4Xi'
        method2print = 'ace'
        
                     
        message2print = "{(Step 1A/33) started" + "; ace-of-BPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) acing" + "; BPU2Image4Xi-acing-by-ace-of-BPU2Image4Xi: " + str(BPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'bpu2image4xi.BPU2Image4Xi'>           
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3A/33) got" + "; kb8xi-got-by-ace-of-BPU2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object8xi': <bpu2image4xi.BPU2Image4Xi object at 0x0000000004188400>}   


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        try:
             kb8xi['cores-of-BPU2Image4Xi'] = BPU2Image4Xi.classvariable2bpu
        except:
             message2print = "{(Step 4A/33) yielded" + "; kb8xi-yielded-by-ace-of-BPU2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
             print4xi(class2print, method2print, message2print) 
  
        message2print = "{(Step 5A/33) aced" + "; BPU2Image4Xi-aced-by-ace-of-BPU2Image4Xi: " + str(BPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'bpu2image4xi.BPU2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 6A/33) returning" + "; kb8xi-returning-by-ace-of-BPU2Image4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'cores-of-BPU2Image4Xi': 4, 'object8xi': <bpu2image4xi.BPU2Image4Xi object at 0x0000000004188400>}
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   bpu2image4xi5test.txt
#   sense-bpu2image4xi5test.xml
#   reason-bpu2image4xi5test.xml
#   ace-bpu2image4xi5test.xml
#"
#>

#<acknowledge4xi acknowledgement = "33" tester = "Small & Smart"/>

# -------------------------------------------------------------------------------------------------
# to-be-done                              
# -------------------------------------------------------------------------------------------------    
#<do4xi do = "
# nothing
#"
#/>

#<note4xi note = "
# parse4xi.py - to parse this module
# "/>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of bpu2image4xi.py                              
# ------------------------------------------------------------------------------------------------- 
#"
#/>
